/* Copyright Statement:
*
* This software/firmware and related documentation ("AutoChips Software") are
* protected under relevant copyright laws. The information contained herein is
* confidential and proprietary to AutoChips Inc. and/or its licensors. Without
* the prior written permission of AutoChips inc. and/or its licensors, any
* reproduction, modification, use or disclosure of AutoChips Software, and
* information contained herein, in whole or in part, shall be strictly
* prohibited.
*
* AutoChips Inc. (C) 2020. All rights reserved.
*
* BY OPENING THIS FILE, RECEIVER HEREBY UNEQUIVOCALLY ACKNOWLEDGES AND AGREES
* THAT THE SOFTWARE/FIRMWARE AND ITS DOCUMENTATIONS ("AUTOCHIPS SOFTWARE")
* RECEIVED FROM AUTOCHIPS AND/OR ITS REPRESENTATIVES ARE PROVIDED TO RECEIVER
* ON AN "AS-IS" BASIS ONLY. AUTOCHIPS EXPRESSLY DISCLAIMS ANY AND ALL
* WARRANTIES, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
* WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
* NONINFRINGEMENT. NEITHER DOES AUTOCHIPS PROVIDE ANY WARRANTY WHATSOEVER WITH
* RESPECT TO THE SOFTWARE OF ANY THIRD PARTY WHICH MAY BE USED BY,
* INCORPORATED IN, OR SUPPLIED WITH THE AUTOCHIPS SOFTWARE, AND RECEIVER AGREES
* TO LOOK ONLY TO SUCH THIRD PARTY FOR ANY WARRANTY CLAIM RELATING THERETO.
* RECEIVER EXPRESSLY ACKNOWLEDGES THAT IT IS RECEIVER'S SOLE RESPONSIBILITY TO
* OBTAIN FROM ANY THIRD PARTY ALL PROPER LICENSES CONTAINED IN AUTOCHIPS
* SOFTWARE. AUTOCHIPS SHALL ALSO NOT BE RESPONSIBLE FOR ANY AUTOCHIPS SOFTWARE
* RELEASES MADE TO RECEIVER'S SPECIFICATION OR TO CONFORM TO A PARTICULAR
* STANDARD OR OPEN FORUM. RECEIVER'S SOLE AND EXCLUSIVE REMEDY AND AUTOCHIPS'S
* ENTIRE AND CUMULATIVE LIABILITY WITH RESPECT TO THE AUTOCHIPS SOFTWARE
* RELEASED HEREUNDER WILL BE, AT AUTOCHIPS'S OPTION, TO REVISE OR REPLACE THE
* AUTOCHIPS SOFTWARE AT ISSUE, OR REFUND ANY SOFTWARE LICENSE FEES OR SERVICE
* CHARGE PAID BY RECEIVER TO AUTOCHIPS FOR SUCH AUTOCHIPS SOFTWARE AT ISSUE.
*/


#ifndef __DRIVE_PARAMETERS_DEFINE_H__
#define __DRIVE_PARAMETERS_DEFINE_H__

/*!
* @file drive_parameters_define.h
*
* @brief This file provides drive parameter define.
*
*/

/* ===========================================  Includes  =========================================== */
#include "motor_parameters_define.h"
#include "hwboard_parameters_define.h"
#include "iqmath.h"
#include "system_ac7840x.h"

/* ============================================  Define  ============================================ */
/*!
* @brief Pwm peripheral configuration.
*/
#define PWM_FREQ_SETTING                    8000UL             /*!< Set PWM count frequency */
#define PWM_PRSC                            0UL                 /*!< PWM prescale */
#define PWM_COUNT_UP_MODE                   0UL                 /*!< PWM count up mode */
#define PWM_COUNT_UP_DOWN_MODE              1UL                 /*!< PWM count up_down mode */
#define PWM_MODE_SELECT                     PWM_COUNT_UP_DOWN_MODE  /*!<0: pwm count up mode; 1: up_down mode */

/*!
* @brief Foc frequency division configuration.
*/
#define FOC_EXECUTION_RATE                  1UL                 /*!< FOC execution rate in number of PWM cycles */
#define FOC_FREQ                            (PWM_FREQ / FOC_EXECUTION_RATE) /*!< FOC execution frequency */

#define FPWM_Hz                             PWM_FREQ            /*!< PWM count frequency */
#define FTBS_Hz                             1000UL              /*!< Time base 1ms frequency */
#define FTBC_Hz                             FOC_FREQ            /*!< Current loop execution frequency */
#define PWM_FREQ_K                          (FOC_FREQ / 1000)   /*!< Pwm frequency coefficient */
#define MS_UINT_INT                         PWM_FREQ_K          /*!< lms, used as a timing unit */

/*!
* @brief Hall module settings.
*/
#define HALL_CLK_PRESCALER                  63UL                /*!< Hall module clock prescale is 2^6 */
#define HALL_FILTER_PSC                     PWM_CHANNEL_INPUT_FILTER_PS_4       /*!< Hall module input signal filter clock prescale */
#define HALL_FILTVAL                        10UL                /*!< Hall module input signal filter value */

#define SEVEN_SEGMENT                       0UL                 /*!< SEVEN_SEGMENT means seven segments svpwm. */
#define FIVE_SEGMENT                        1UL                 /*!< FIVE_SEGMENT means five segments svpwm. */

#define SPEED_CONTROL_MODE                  1                   /*!< speed control mode */
#define TORQUE_CONTROL_WITH_SPDLIMIT        2                   /*!< torque control mode with speed limit */

#define CURVE_TYPE_STRAIGHT                 0                   /*!< straight line */
#define CURVE_TYPE_CUSTOMER                 1                   /*!< 5-segment customer curve */

#define FREE_RUN_STOP                       0                   /*!< Motor free stop mode, the MOSFETs are closed immediately after pressing the stop key*/
#define SLOW_DOWN_STOP                      1                   /*!< Motor slow down stop mode, the MOSFETs are closed after the motor is controlled to
                                                                *    decelerate to the stop frequency when the stop key is pressed.
                                                                */

#define FORWARD_ROTATE                      ((int16_t)1)
#define REVERSE_ROTATE                      (-(int16_t)1)

#define PMSM_LDQ_METHOD_HFP                 (1UL)               /*!< PMSM Ldq identification method using high frequency pulse */
#define PMSM_LDQ_METHOD_HFS                 (2UL)               /*!< PMSM Ldq identification method using high frequency sine wave */

#define T_METHOD_HZ                         10000               /*!< Set T method measure timer high frequency value */


#if (defined MOTOR_ONE_TYPE)
/*!
* @brief This macro define determines whether to execute the parameters access to flash.
*/
#define DATA_ACCESS_FLASH_ENABLE

/*!
* @brief There are two main control modes for three phase brushless motor(include MOTOR_BLDC,MOTOR_SPMSM...in motor_parameters_define.h), which are six step square wave mode and foc sine wave mode.
*        Six step square wave mode are configured in bldc_parameters_define.h; Foc sine wave mode are configured in drive_parameters_define.h.
*        If user choose foc sine wave mode, should undefine SIX_STEP_SQUARE_WAVE in bldc_parameters_define.h, and define FOC_SINE_WAVE in this file.
*        There are four foc sine wave control sub modes, which are sensorless control, hall sensor control, encoder control and encoder_hall control.
*        After define FOC_SINE_WAVE, user can continue choose the sub modes by define SENSORLESS,HALL_SENSORS...and so on, and please undefine the other three sub mode meanwhile.
*        IF user choose six step square wave mode, make sure to undefine FOC_SINE_WAVE and all the four sub modes.
*/
#define FOC_SINE_WAVE

#define FOC_PMSM
//#define FOC_ACIM

#if (defined FOC_PMSM)
#define SENSORLESS
//#define HALL_SENSORS
//#define ENCODER_SENSORS
//#define ENCODER_HALL_SENSORS

#elif (defined FOC_ACIM)
#define SENSORLESS_ACIM
//#define SENSOR_ACIM

#else
#endif

/*!
* @brief If need to apply V/f control, the FOC_SINE_WAVE macro definition must be shield and turn on VF_CONTROL macro definition.
*/
//#define VF_CONTROL

/*!
* @brief There are three startup methods for sensorless control: VF startup, IF startup and Direct startup.
*        When using high frequency injection sensorless mode, all of the three startup methods need to be shielded.
*/
//#define VF_STARTUP
#define IF_STARTUP
//#define DIRECT_STARTUP

/*!< High frequence injection control mode when in low speed, need to close IF_STARTUP, VF_STARTUP, DIRECT_STARTUP and CLOSE_TO_OPEN_SWITCH before use */
//#define HIGH_FREQUENCE_INJECTION

/*!
* @brief There are three sensorless control modes: namely the sliding mode control, flux observer mode and mras observer mode.
*        They are only suitable for medium and high speed applications.
*/
//#define SMC_OBSERVE
#define FLUX_OBSERVE
//#define MRAS_OBSERVE

/*!
* @brief There are two sensorless angle obtain ways: namely the ATAN_ANGLE_ESTIMATION way and PLL_ANGLE_ESTIMATION way.
*/
//#define ATAN_ANGLE_ESTIMATION
#define PLL_ANGLE_ESTIMATION

/*!
* @brief There are two ways to pre-position the motor rotor: ALIGNMENT_ONCE, ALIGNMENT_TWICE.
*        In sensorless application, one of the two ways must be turned on.
*/
#define ALIGNMENT_ONCE
//#define ALIGNMENT_TWICE

/*!
* @brief If PULSE_INJECTION macro is defined, the initial position can be detect when the motor is stopped.
*/
//#define PULSE_INJECTION
#define PULSE_INJECTION_TIMES   2UL         /*!< Pulse injection times, When it is greater than 1, multiple pulse injection will be performed, the setting range is 1 ~ 5 */

/*!
* @brief If INIT_POS_DETEC macro is defined, use the rotation high frequency voltage injection method to check rotor position. Cannot be used at the same time with PULSE_INJECTION definition.
*/
//#define INIT_POS_DETEC                  /*!< Initial position check with injecting rotation high frequency voltage */

/*!
* Inject DC current into the d axis, manually rotate the motor rotor, derermine the number
* of motor pole pairs according to the position of the resistance point.
*/
//#define MOTOR_POLES_OBTAIN  /*!< Get the motor poles through the alignemnt method, close PULSE_INJECTION before use */

/*!
* There are two ways to obtain the sensorless input voltage: use voltage reconstruction and use svpwm voltage.
* Just need to open or close VOLTAGE_RESTRUCT to choose. Open the VOLTAGE_RESTRUCT means using the voltage reconstruction, close means using svpwm voltage.
*/
//#define VOLTAGE_RESTRUCT

/*!
* @brief Sensorless operation mode, including open loop operation and closed loop operation.
*/
//#define OPEN_LOOP
#define CLOSE_LOOP

/*!
* @brief Over modulation control
*/
#define OVER_MODULATION                     0UL                   /*!< Set 1 will enable over moduation. Set 0 will disable over modulation */

/*!
* @brief Svpwm base on zero sequence component, be effective while OVER_MODULATION is setting 0, only working within linear modulation region.
*/
#define ZERO_SVPWM                          0UL                   /*!< Set 1 will reduce SVPWM time consumption than Set 0 */

/*!
* @brief SVPWM output control, chose SVPWM_SEGMENT_TYPE is SEVEN_SEGMENT or FIVE_SEGMENT.
*/
#define SVPWM_SEGMENT_TYPE                  SEVEN_SEGMENT

/*!
* @brief Hall sensor order and install error identify.
*/
#define ORDER_ERROR_COMP_MODE               1                   /*!< 0:force degree compensation 1:disperse degree compensation */
#define HALL_INSTALL_TYPE                   0                   /*!< 0:120 degree hall install 1:60 degree hall install */

#define HALL_SENSOR_SELF_LEARN              DISABLE             /*!< Hall self-learn. If this macro definition is opened, will perform the Hall angle self-learning function */
#define HALL_SELF_LEARN_MODE                2                   /*!< Hall self learn mode, 1: learn order and theta; 2: learn order and theta and sector width */
#define HALL_SWAP_SECTORS                   3                   /*!< Hall run swap state setors, set range is 1~3 */
#define HALL_SWAP_POS_ANGLE                 30                  /*!< Init positive position while hall run swap state, set range is 0~60 */
#define HALL_SWAP_NEG_ANGLE                 30                  /*!< Init negative position while hall run swap state, set range is 0~60 */
#define HALL_SWAP_TRQ_COFF                  100                 /*!< Torque boost at low speed, set range is 0~100 */
#define HALL_SELF_LEARN_INTERVAL            500                 /*!< Hall self-learn time interval */
#define HALL_SELF_LEARN_CUR                 (0.5f)        /*!< Hall self-learn current value */
#define HALL_SELF_LEARN_ORDER               15                  /*!< Hall self-learn hall order */
#define HALL_SELF_LEARN_THETA               (3640.0f / 65536)                /*!< Hall self-learn hall installation error */

/*!
* Sensorless calculation frequency function location selection, ELEC_FREQUENCE_HIGH means to calculate in interrupt,
* ELEC_FREQUENCE_LOW means to calculate in lms task, only used in ATAN_ANGLE_ESTIMATION mode.
*/
//#define ELEC_FREQUENCE_HIGH
#define ELEC_FREQUENCE_LOW

#define ATAN_SPEED_FILT_H                   (0.9861f)   /*!< Low pass filter coefficient used for ELEC_FREQUENCE_HIGH and
                                                                *    ATAN_ANGLE_ESTIMATION, the larger coefficient, the heavier filtering.
                                                                */
#define ATAN_SPEED_FILT_L                   (0.9661f)   /*!< Low pass filter coefficient used for ELEC_FREQUENCE_LOW and
                                                                *    ATAN_ANGLE_ESTIMATION, the larger coefficient,t he heavier filtering.
                                                                */

/* M-T speed measure configuration */
#if ((defined ENCODER_SENSORS) || (defined SENSOR_ACIM))
#define MT_SPEED_MEASURE
#endif

/*!
* @brief Speed adjust method, only one method can be chosed.
*/
#define SPEED_ADJUST_KEY                                        /*!< If defined SPEED_ADJUST_KEY, use key adjust speed */
//#define SPEED_ADJUST_KNOB                                     /*!< If defined SPEED_ADJUST_KNOB, use knob adjust speed */

/*!
* @brief Motor control mode select, define MOTOR_CONTROL_MODE to chose. Only one mode can be chosed.
*/
#define MOTOR_CONTROL_MODE                  SPEED_CONTROL_MODE  /*!< Motor control mode selection */

/*!
* @brief Torque control mode with speed limit.
*/
#define TORQUE_MODE_IQ_ADD_STEP             (0.1f / 1000)     /*!< iq target command increase 3000 in 1000ms */
#define TORQUE_MODE_IQ_CMD_MAX              (0.95f)       /*!< iq target command max value */
#define TORQUE_MODE_IQ_CMD_MIN              (0.2f)        /*!< iq target command min value for initial iq default value */

/*!
* Foc voltage output limit configuration, when current sample mode is
* TWO_SHUNT_SAMPLE, VDQ_LIMIT_VALUE need to be set to (0.95 * Math_IQ(0.866))
* to ensure current sample accuracy. SINGLE_SHUNT_SAMPLE and THREE_SHUNT_SAMPLE
* do not need to limit, the VDQ_LIMIT_VALUE can be set to Math_IQ(1.0).
*/
#define VDQ_CIRCLE_LIMIT_CTRL                                   /*!< Voltage output limit enable switch. If this macro definition is open, voltage output limit in FOC is supported */
#define VDQ_LIMIT_VALUE                     (1.0f)        /*!< Voltage output limit value, 1.0 means the max amplitude of output voltage vector is 1.0 times of bus voltage */

/*!
* @brief Configuration of speed ramp.
*/
#define SPEED_CHANGE_STEP                   (2.0f / BASE_FREQ)                                /*!< Speed change step, 2.0 means increase 2Hz for each calculation cycle */
#define MAX_SPEED_VALUE                     (0.9f)                                            /*!< Speed upper limit, 0.8 means max limit of motor running speed is 0.8 * MOTOR_MAX_SPEED_RPM */
#define MIN_SPEED_VALUE                     (OPEN_LOOP_SPEED_HZ * POLE_PAIR_NUM / BASE_FREQ) /*!< Speed lower limit */
#define STOP_SPEED_VALUE                    (STOP_SPEED_HZ * POLE_PAIR_NUM / BASE_FREQ)      /*!< Speed cutoff value at motor stop */

/*!
* @brief Configuration of acceleration and deceleration speed.
*/
#define ACCELERATION_RPS                    ((600.0f * 10) / (MOTOR_MAX_SPEED_RPM * 100)) /*!< Acceleration of motor speed up in unit RPM/s, motor mechanical rotation speed increase per second.
                                                                                                *    600.0: 600rpm increase per second, must be float type. Can be modified to change the acceleration;
                                                                                                *    MOTOR_MAX_SPEED_RPM: motor speed max value, can not be modified;
                                                                                                *    100: 1s = 100 * (10ms), the speed increase every 10ms, can not be modified.
                                                                                                */
#define DECELERATION_RPS                    ((600.0f * 10) / (MOTOR_MAX_SPEED_RPM * 100)) /*!< Deceleration of motor speed up in unit RPM/s, motor mechanical rotation speed decrease per second.
                                                                                                *    600.0: 600rpm increase per second, must be float type. Can be modified to change the deceleration;
                                                                                                *    MOTOR_MAX_SPEED_RPM: motor speed max value, can not be modified;
                                                                                                *    100: 1s = 100 * (10ms), the speed decrease every 10ms, can not be modified.
                                                                                                */

/*!
* @brief Parameter configuration of FOC open loop control.
*/
#define OPEN_LOOP_SPEED_HZ                  (10.0f)                /*!< Final forced synchronize mechanical FREQ in Hz */
#define MAX_SYNC_SPEED_THRES                (float)(1.08f)       /*!< Final forced synchronize mechanical FREQ upper limit */
#define MIN_SYNC_SPEED_THRES                (float)(0.92f)       /*!< Final forced synchronize mechanical FREQ lower limit */
#define OPEN_LOOP_TIME_SYN                  10                  /*!< Reach openloop target speed synchronization times */

#define CUR_START_SUB                       5                   /*!< Current reduce step */

#define SPD_SMOOTH_CUR                      (0.1f)                /*!< Smooth the sensorless speed switching process */

#define DEFAULT_TARGET_FREQ_START           (OPEN_LOOP_SPEED_HZ * POLE_PAIR_NUM / BASE_FREQ)    /*!< Default target frequency for sensorless mode */

#define STOP_SPEED_HZ                       (2.0f)                 /*!< Mechanical frequency cutoff value in Hz at motor stop */

#define OPEN_LOOP_CURRENT                   (0.2f)        /*!< Three phase current amplitude in Q15 format if in IF_STARTUP mode */
#define OPEN_LOOP_VOLTAGE                   (0.4f)        /*!< Three phase voltage amplitude in Q15 format if in VF_STARTUP mode */

/*!
* @brief Sensorless foc close loop to open loop switch when in low speed, need to close HIGH_FREQUENCE_INJECTION before use.
*/
//#define CLOSE_TO_OPEN_SWITCH
#define SPEED_DELAY_ZONE    (2.0f * POLE_PAIR_NUM / BASE_FREQ)    /*!< Close and open loop switch speed delay zone, need to be set less than MIN_SPEED_VALUE */

/*!
* @brief Parameter configuration of FOC open loop start-up curve.
*/
#define CURVE_TYPE                          CURVE_TYPE_STRAIGHT /*!< select curve type */

#define TIME_DURATION1                      (uint32_t)1000U     /*!< 1000ms = 1s, time duration value of 1st segment, setted by tools */
#define TIME_DURATION2                      (uint32_t)1000U     /*!< 1000ms = 1s, time duration value of 2nd segment */
#define TIME_DURATION3                      (uint32_t)1000U     /*!< 1000ms = 1s, time duration value of 3rd segment */
#define TIME_DURATION4                      (uint32_t)1000U     /*!< 1000ms = 1s, time duration value of 4th segment */
#define TIME_DURATION5                      (uint32_t)1000U     /*!< 1000ms = 1s, time duration value of 5th segment */

#define CUSTOMER_CURVE_POINTS               1U                   /*!< curve segment number, configed by host tool */

#define FREQ_REF_INITVALUE                  (0.0f / BASE_FREQ)    /*!< initial state: 0Hz */
#define CUR_REF_INITVALUE                   OPEN_LOOP_CURRENT   /*!< initial current, unit: pu */
#define VOLT_REF_INITVALUE                  (0.1f)        /*!< initial voltage, unit: pu */

/*!
* If select CURVE_TYPE_STRAIGHT, FREQ_REF_VALUE_1 is the target frequency of open loop;
* If select 5 points CURVE_TYPE_CUSTOMER, FREQ_REF_VALUE_5 is the final target frequency of open loop.
*/
#define FREQ_REF_VALUE_1                    (OPEN_LOOP_SPEED_HZ * POLE_PAIR_NUM / BASE_FREQ)  /*!< set the 1st point frequency value */
#define FREQ_REF_VALUE_2                    (8.0f / BASE_FREQ)    /*!< set the 2nd point frequency value */
#define FREQ_REF_VALUE_3                    (10.0f / BASE_FREQ)   /*!< set the 3rd point frequency value */
#define FREQ_REF_VALUE_4                    (12.0f / BASE_FREQ)   /*!< set the 4th point frequency value */
#define FREQ_REF_VALUE_5                    MIN_SPEED_VALUE             /*!< set the 5th point frequency value */

#define CUR_REF_VALUE_1                     OPEN_LOOP_CURRENT           /*!< set the 1st point current value */
#define CUR_REF_VALUE_2                     (0.3f)                /*!< set the 2nd point current value */
#define CUR_REF_VALUE_3                     (0.15f)               /*!< set the 3rd point current value */
#define CUR_REF_VALUE_4                     (0.2f)                /*!< set the 4th point current value */
#define CUR_REF_VALUE_5                     OPEN_LOOP_CURRENT           /*!< set the 5th point current value */

#define VOLT_REF_VALUE_1                    (0.25f)               /*!< set the 1st point voltage value */
#define VOLT_REF_VALUE_2                    (0.1f)                /*!< set the 2nd point voltage value */
#define VOLT_REF_VALUE_3                    (0.11f)               /*!< set the 3rd point voltage value */
#define VOLT_REF_VALUE_4                    (0.125f)              /*!< set the 4th point voltage value */
#define VOLT_REF_VALUE_5                    (0.15f)               /*!< set the 5th point voltage value */

/*!
* @brief S-curve speed ramp definition. S-curve will be usde to generate speed ramp with different accelerations at different stages.
*/
//#define SCURVE_SPEED_RAMP
#define ACC_START_SEG_RATIO                 (0.3f)        /*!< The time proportion of the start segment(acceleration increase) of S-curve, m value */
#define ACC_END_SEG_RATIO                   (0.3f)        /*!< The time proportion of the end segment(acceleration reduction) of S-curve, n value */
#define DEC_START_SEG_RATIO                 (0.3f)        /*!< The time proportion of the start segment(deceleration increase) of S-curve, m value */
#define DEC_END_SEG_RATIO                   (0.3f)        /*!< The time proportion of the start segment(deceleration reduction) of S-curve, n value */

/*!
* @brief Motor stop mode selection.
*/
#define MOTOR_STOP_MODE                           FREE_RUN_STOP

/*!
* Motor rotation direction configuration.
*/
#define MOTOR_DIR                           FORWARD_ROTATE      /*!< Motor direction 1: foward rotate; -1: reverse rotate */

#define SPEED_FIFO_DEPTH                    64                  /*!< Depth of the FIFO used to average speed use in lib */

/*!
* @brief TAIL_HEAD_WIND macro definition is used to support fan applications even if the motor may be in motion before control.
*        If this macro is defined, the tailwind and headwind function will be turned on.
*/
//#define TAIL_HEAD_WIND

#define MOTOR_MOTION_BEMF_AMP_THRESHOLD     50                      /*!< Initial speed judgment back emf ad sample value amplitude threshold, higher than the threshold means the motor initial speed state is in motion, 0-4096 */
#define HIGH_REVERSE_THRESHOLD              (-0.15f)          /*!< High reverse threshold, higher than the threshold means the motor initial speed state is HIGH_REVERSE. */
#define STATIC_THRESHOLD                    (0.01f)           /*!< Static threshold, Lower than the threshold means the motor initial speed state is STATIC. */
#define HIGH_FORWARD_THRESHOLD              (0.05f)           /*!< High forward threshold, higher than the threshold means the motor initial speed state is HIGH_FORWARD. */

/*!
* @brief Short brake configuration.
*/
#define SHORT_BRAKE_CTRL                    0                   /*!< 0:disable 1:before startup brake 2:after stop brake 3:before startup and after stop, both brake */
#define SHORT_BRAKE_CUR_RATIO               (1.0f)                 /*!< Short brake max current pu value, set range is 0~100 */
#define SHORT_BRAKE_UDC_MAX                 35                  /*!< Short brake max Udc value */
#define SHORT_BRAKE_TIMES                   800                 /*!< Short brake finished wait time uint:ms, set range is 0~8000 */
#define SHORT_QUIT_CUR                      (0.1f)        /*!< Quit short brake function current threshold */
#define SHORT_TIME                          5000                /*!< Single R sample shortbrake time  uint: ms */

/*!
* @brief Electric brake parameters configuration.
*/
//#define ELEC_BRAKE_ENABLE                                     /*!< Electric brake enable switch, if this macro definition is opened, the electric brake function is supported */
#define ENTER_BRAKE_DEPTH                   (0.00152f)                  /*!< Enter electric brake min depth 0~32767 */
#define EXIT_BRAKE_DEPTH                    (0.00137f)                  /*!< Exit electric brake max depth */
#define ENTER_ELEC_BRAKE_FREQ               (30.0f / BASE_FREQ)   /*!< Enter electric brake min frequence */
#define EXIT_ELEC_BRAKE_FREQ                (20.0f / BASE_FREQ)   /*!< Exit electric brake max frequence */
#define ISQ_ADD_STEP                        (0.0003f)                  /*!< Isqref add step */
#define ISQ_DEC_STEP                        (0.0003f)                  /*!< Isqref dec step */
#define ELEC_BRAKE_CUR_RATIO                (0.5f)                  /*!< Max current higher than 50% * 32768 pwmoff */
#define ELEC_BRAKE_UDC_MAX                  15                  /*!< Dc voltage higher than 15V  pwmoff ,15 can adjust according to dc voltage*/
#define ELEC_BRAKE_DEPTH                    0                   /*!< Input electric brake depth */
#define ELEC_BRAKE_MAX_DEPTH                (0.999f)               /*!< Input electric brake max depth */

/*!
* @brief Power and bus current estimate parameters configuration.
*/
//#define POWER_DCCUR_EST_ENABLE                                  /*!< Power and bus current estimate enable switch, if this macro definition is opened, the input power and DC bus current estimated function is supported */
#define PWM_DELAY_COF                       (0.31f)               /*!< Power estimate pwm delay coffe */
#define UQCOMP                              (0.018f)                 /*!< Power estimate uq compensation */

/*!
* @brief Parameter configuration.
*/
#define PARAM_IDENTIFY_SEL                  (0x07UL)            /*!< bit0: rs identify. bit1: Ld Lq identify. bit2: flux identify. bit3: inertia identify */

/* Stator resistance identification phase select */
#define RS_ID_PHASE_SEL                     (1UL)               /*!< Select motor stator phase order to identify stator resistance, 1: One phase; 2: Two phase; 3: Three phase */
/* The following two current setting, 0.4 means the amplitude of the injection current is 0.4 * MAX_CURRENT. Excessive setting will lead to motor heating */
#define RS_ID_FIRST_INJECT                  (0.3f)        /*!< Rs identify first inject current threshold */
#define RS_ID_SECOND_INJECT                 (0.5f)        /*!< Rs identify second inject current threshold */
/* The following three time settings affect the duration of current injection, excessive setting will lead to motor heating */
#define RS_STABLE_DELAY                     (1000UL)            /*!< Unit: ms, wait for stable after changing the amplitude of the injection current */
#define RS_SETTING_DELAY                    (500UL)             /*!< Unit: ms, wait for changing the amplitude of the injection current */
#define RS_SAMPLE_DELAY                     (500UL)             /*!< Unit: ms, wait for sampling and saving current and voltage */

/* PMSM Ldq identify configuration */
#define PMSM_LDQ_METHOD_SEL                 PMSM_LDQ_METHOD_HFS /*!< PMSM Ldq identification method select */

/* PMSM Ldq method HFP */
#define PMSMID_LDQ_INIT_PRD                 (600UL)             /*!< Ldq identify initial pwm period value. Generally, it does not need to be modified*/
#define PMSMID_LDQ_CUR_LIMIT                (0.4f)        /*!< Ldq identify max current limit*/

#define PMSMID_ALIGN_TIME1                  T_ALIGNMENT1        /*!< First alignment time of Ldq identification in ms */
#define PMSMID_ALIGN_TIME2                  T_ALIGNMENT2        /*!< Second alignment time of Ldq identification in ms */
#define PMSMID_ALIGN_CUR                    I_ALIGNMENT1        /*!< Alignment current of Ldq identification */
#define PMSMID_ALIGN_WAIT                   (10UL)              /*!< Alignment done and wait time of Ldq identification in ms */

/* PMSM Ldq method HFS */
#define PMSMID_CUR_DC_SET                   (0.3f)       /*!< PMSM Ldq identification DC current set value */
#define PMSMID_CUR_AC_SETD                  (0.3f)       /*!< PMSM Ldq identification d-axis AC current set value */
#define PMSMID_CUR_AC_SETQ                  (0.3f)        /*!< PMSM Ldq identification q-axis AC current set value */
#define PMSMID_CUR_FREQ_SETD                (3.0f)        /*!< PMSM Ldq identification d-axis AC current frequency set value */
#define PMSMID_CUR_FREQ_SETQ                (3.0f)        /*!< PMSM Ldq identification q-axis AC current frequency set value */
#define PMSMID_CUR_DC_STEP                  (0.002f)      /*!< PMSM Ldq identification DC current variable step value */

#define STABLE_JUDGE_LIMIT                  (0.6f)        /*!< Signal stable judgement limit */

#define PMSMID_LDQ_PWMFREQ                  (11000UL)           /*!< PWM frequency in pmsm Ldq identification */
#if (PWM_MODE_SELECT == PWM_COUNT_UP_DOWN_MODE)
#define PMSMID_LDQ_PWMPRD        (((MOTOR_BUS_CLK / (PMSMID_LDQ_PWMFREQ * (PWM_PRSC + 1))) >> 1) - 1)       /*!< PWM period value in pmsm Ldq identification */
#else
#define PMSMID_LDQ_PWMPRD        ((MOTOR_BUS_CLK / (PMSMID_LDQ_PWMFREQ * (PWM_PRSC + 1))) - 1)              /*!< PWM period value in pmsm Ldq identification */
#endif

/* PMSM flux identify configuration */
#define PMSMID_FLUX_FRE                     (0.2f)        /*!< Flux identify set frequency of flux identification */
#define PMSMID_FLUX_CUR                     (0.366f)      /*!< Flux identify set current of flux identification, must less than Math_IQ(0.5) */
#define PMSMID_FLUX_IF_ACC                  0.0001f//(2UL / 32768.0f)               /*!< Flux identify IF run  acceleration */

/* ACIM identify configuration */
#define ACIMID_RR_CURRENT                   (0.5f)        /*!< Acim stall test current limit */
#define ACIMID_RR_FREQ                      (0.6f)        /*!< Acim stall test frequency limit */
#define ACIMID_LM_FREQ                      (0.6f)        /*!< Acim rotation test frequency limit */
#define ACIMID_RR_CURSTEP                   0.0003f              /*!< Acim stall test current step */
#define ACIMID_LM_FREQSTEP                  0.00015f               /*!< Acim rotation test frequency step */

#define ACIMID_CURRENT_THRESHOLD            (0.6f)        /*!< Acim identification current threshold to judge current state */
#define ACIMID_VOLTAGE_THRESHOLD            (0.8f)        /*!< Acim identification voltage threshold to judge voltage state */

/* Identify signal extraction configuration */
#define PARID_DATANUM                       (2048UL)            /*!< Motor identification data calculation number */
#define PARID_FILTER1COEFF1                 (PWM_FREQ / (PI2 * 100))               /*!< Motor identification signal filter-1 coefficient */
#define PARID_FILTER1COEFF2                 (PWM_FREQ / (PI2 * 160))              /*!< Motor identification signal filter-1 coefficient */
#define PARID_FILTER2COEFF1                 (PWM_FREQ / (PI2 * 100))            /*!< Motor identification signal filter-2 coefficient */
#define PARID_FILTER2COEFF2                 (PWM_FREQ / (PI2 * 160))            /*!< Motor identification signal filter-2 coefficient */

/*!
* @brief Dead time Compensation control define
*/
//#define DEAD_BAND_COMP                                        /*!< If defined DEAD_BAND_COMP will Enable dead time compensation */
#define CUTOFF_CUR_VAL                      (0.25f)       /*!< Dead band compensation current threshold */
#define MAX_COMP_FREQ                       (0.5f)        /*!< Dead band compensation speed threshold */

/*!
* @brief Over voltage supress at slow down stop mode.
*/
//#define OVER_VOL_SUPPRESS                 /*!< If defined OVER_VOL_SUPPRESS will enable over voltage suppression */

/*!
* @brief Narrow pulse elimination process.
*/
//#define NARROW_PULSE_PROCESS             /*!< If defined  NARROW_PULSE_PROCESS will enable narrow pulse elimination */

/*!
* @brief Energy save function configuration
*/
//#define SAVE_ENERGY                                           /*!< If defined SAVE_ENERGY will enable energy save function */
#define MAX_CUR_CHANGE                      (0.02f)       /*!< Max current change threshold*/
#define STEADY_CUR_CHANGE                   (0.01f)       /*!< Before enter energy save function, check current steady */
#define CUR_CHANGE_STEP                     (0.003f)      /*!< D axis current change step value */
#define ENERGY_SAVE_DEC                     (0.01f)       /*!< Current amplitude decrease threshold */
#define ENERGY_SAVE_INC                     (0.01f)       /*!< Current amplitude increase threshold */
#define RECHECK_CUR_CHANGE                  (0.05f)       /*!< Load change threshold to enter energy save again */
#define MAX_ID_TARGET                       (0.5f)        /*!< D axis current max value*/
#define MIN_ID_TARGET                       (-0.5f)       /*!< D axis current min value */
#define WAIT_TIME                           11                  /*!< Interval waite time value: 2^n *1ms, e.g.: this value is 11, then the time is 2^11 *1ms = 2048ms */
#define ENTER_ENERGY_SAVE_LOAD              (0.3f)        /*!< Enter energy save function load threshold value */

/*!
* @brief Sliding mode control(SMC) parameters configuration.
*/
/* SMC parameters configuration before close loop */
#define SMC_KSLIDE0                         (0.3f)        /*!< Open loop configuration sliding mode gain */
#define SMC_KSLF0                           (0.46f)               /*!< Open loop configuration SMC filter coefficient, the smaller the value, the more obvious the filtering effect, 0~32767 */
#define SMC_MAXERR0                         (0.8f)        /*!< Open loop configuration sliding mode boundary */
/* SMC parameters configuration after close loop */
#define SMC_KSLIDE1                         (0.7f)        /*!< Close loop configuration sliding mode gain */
#define SMC_KSLF1                           SMC_KSLF0           /*!< Close loop configuration sliding mode filter coefficient */
#define SMC_MAXERR1                         (0.8f)        /*!< Close loop configuration sliding mode boundary */

#define FULL_ORDER_SMC                                          /*!< Full order smc enable switch, only works when SMC_OBSERVE and FULL_ORDER_SMC are defined */
#define FULL_SMC_GAIN                       (0.3f)               /*!< Full order smc gain coefficients */
#define FULL_SMC_FILT                       DISABLE//ENABLE     /*!< Full order smc filter enable switch, ENABLE means enable filter feature, DISABLE means disable filter feature */
/*!
* @brief Flux observer parameters configuration.
*/
#define FLIT_COEFF                          (0.061f)                /*!< Flux observer LPF filter coefficient, the smaller the value, the more obvious the filtering effect, 0-32767 */
#define COMPCOF_UPLIMIT                     (5.0f)          /*!< Flux observer back emf compensation uplimit, Math_IQ(0)-Math_IQ(100) */

/*!
* @brief Mras observer parameters configuration.
*/
#define ALPHA_COEFF                         (0.061f)                /*!< Mras observer alpha coffcient, use this parameter calculate we,  0~32767 */
#define BETA_COEFF                          (0.46f)               /*!< Mras observer beta coffcient, use this parameter calculate correct we, 0~45875 */

/* Only in case of PLL_ANGLE_ESTIMATION is defined, this coefficient is used to set PI parameters of the pll */
#define COEFFPLL_SMC                        (0.091f)                /*!< Smc observer pll PI set coefficient, need to be set less than 16384 */
#define COEFFPLL_FLUX                       (0.091f)                /*!< Flux observer pll PI set coefficient, need to be set less than 16384 */

/*!
* @brief Sensorless pre-positioning parameters configuration.
*/
#define T_ALIGNMENT1                        (uint32_t)(1000)     /*!< First alignment time in ms */
#define T_ALIGNMENT2                        (uint32_t)(1000)    /*!< Second alignment time in ms */
#define ALIGNMENT_ANGLE_DEGREE              (270.0f)               /*!< Alignment angle in degree */
#define ALIGNMENT_ANGLE                     (ALIGNMENT_ANGLE_DEGREE / 360.0f)   /*!<Alignment Degrees in Q16 */
#define I_ALIGNMENT1                        OPEN_LOOP_CURRENT//(0.2f)        /*!< First alignment current */
#define I_ALIGNMENT2                        OPEN_LOOP_CURRENT   /*!< Second alignment current */
#define I_RAMP1_INIT                        (0)          /*!< Initial value of first alignment current ramp */
#define I_RAMP2_INIT                        I_ALIGNMENT1        /*!< Initial value of second alignment current ramp */
#define T_SLOPE1                            (T_ALIGNMENT1 >> 2) /*!< Duration of first alignment current ramp in ms */
#define T_SLOPE2                            (T_ALIGNMENT2 >> 5) /*!< Duration of second alignment current ramp in ms */

/*!
* @brief Initial position check configuration
*/
//#define PULSE_SELF_LEARN                                                     /*!< If not defined, will cancel pulse width learn process */
#define PWM_SELF_LEARN_PRD                  10000//(2100)                           /*!< If cancel pulse width learn process, set pulse inject width */
#define INIT_PULSE_PERIOD                   ((MOTOR_BUS_CLK / 20000) >> 1)   /*!< initial pwm period 20k */
#define SELF_LEARN_THRES                    (MAX_CURRENT * 0.3f * (ADC_RANGE * RSHUNT * OP_AMPLIFICATION_GAIN / VSVREF) + (ADC_RANGE >> 1))
#define SELF_LEARN_DUTY_STEP                (240)                            /*!< pwm width slef learn duty increase step */

#define INITPOS_VOLT_AMP                    (0.45f)        /*!< Initial position check injection rotation voltage amplitude */
#define INITPOS_VOLT_FREQ                   (3.0f)        /*!< Initial position check injection rotation voltage frequency */
#define INITPOS_PULSE_AMP                   (0.6f)        /*!< Initial position check injection pulse voltage amplitude */
#define INITPOS_PULSE_INJECT_TIME           1U                  /*!< Initial position check injection pulse voltage times */
#define INITPOS_PULSE_CLEAR_TIME            100U                /*!< Initial position check injection pulse voltage clear times */

/*!
* @brief High frequency injection parameters configuration.
*/
#define HFI_DEBUG_MODE                      FALSE               /*!< High frequency injection debug mode enable switch, FALSE means HFI run mode, TRUE means debug mode */
#define HFI_TO_STO                          ENABLE              /*!< High frequency injection switch to state observer(STO) enable switch,
                                                                *    DISABLE means can not switch to high speed strategy,
                                                                *    ENABLE means can switch to high speed strategy
                                                                */

#define HFI_FREQUENCY                       600                 /*!< High frequency injection electric frequency unit Hz */
#define HFI_AMPLITUDE                       (0.2f)        /*!< High frequency injection voltage amplitude */
#define HFI_PULSE_AMP                       (0.5f)        /*!< Pulse voltage amplitude */
#define HFI_PULSE_TIME                      3                   /*!< Pulse voltage inject time, 3 means 3 PWM interrupt period, LG:4 is better */
#define HFI_CLEAR_TIME                      30 * MS_UINT_INT    /*!< Current clear to zero time, 30 means 30ms*/
#define HFI_JUDGE_TIME1                     300 * MS_UINT_INT   /*!< High frequency injection first judge time, 300 means 300ms */
#define HFI_JUDGE_TIME2                     60 * MS_UINT_INT    /*!< High frequency injection second judge time, 60 means 60ms */
#define HFI_INIT_TIME                       60 * MS_UINT_INT    /*!< High frequency injection initial inject time, 60 means 60ms */
#define HFI_INIT_THETA                      (0.1667f)               /*!< High frequency injection initial theta, 10923 means 60 degree */

#define HFI_PLL_KP_DEFAULT                  (0.305f)               /*!< High frequency injection pll kp */
#define HFI_PLL_KI_DEFAULT                  (0.003f)                 /*!< High frequency injection pll ki */

#define HFI_INIT_FREQ                       (4.0f * POLE_PAIR_NUM / BASE_FREQ)    /*!< High frequency injection default initial frequence, 4.0 means 4*60=240rpm */
#define HFI_FREQ_HIGH                       (8.0f * POLE_PAIR_NUM / BASE_FREQ)    /*!< High frequency injection to observer frequence threshold, 8.0 means 8*60=480rpm */
#define HFI_FREQ_LOW                        (5.0f * POLE_PAIR_NUM / BASE_FREQ)    /*!< Observer to high frequency injection frequence threshold, 5.0 means 5*60=300rpm */
#define HFI_VD_FREQ_HIGH                    (12.0f * POLE_PAIR_NUM / BASE_FREQ)   /*!< High frequency injection voltage amplitude decrease frequence high threshold, 12.0 means 12*60=720rpm*/
#define HFI_VD_FREQ_LOW                     (8.0f * POLE_PAIR_NUM / BASE_FREQ)    /*!< High frequency injection voltage amplitude decrease frequence low threshold, 8.0 means 8*60=480rpm */
#define HFI_DIR_ERR_TIME                    100 * MS_UINT_INT                           /*!< High frequency injection run dir error judge time, 100 means 100ms*/
#define HFI_DIR_ERR_FREQ                    (0.009f)                              /*!< High frequency injection run dir error frequence judge threshold */

/*!
* @brief Two order band pass filter and low pass filter parameters design, user can refer to "ATC_AC78xx_Filter_Parameter_Setting" document.
*        Filter transfer function: y(n) = A0 * x(n) + A1 * x(n-1) + A2 * x(n-2) + B1 * y(n-1) + B2 * y(n-2).
*        Different filters can be obtained by  matching different coefficients A and B.
*/
/* HFI 2rd BPF COEFF 8k pwmfreq, 200-1000 passFreq */
#define BPF_A0                              (0.2452f)
#define BPF_A2                              (-0.2452f)
#define BPF_B1                              (1.4142f)
#define BPF_B2                              (-0.5095f)
/* HFI 2rd LPF COEFF 8k pwmfreq, 200Hz cutFreq */
#define LPF_A0                              (0.0055f)
#define LPF_A1                              (0.0111f)
#define LPF_A2                              (0.0055f)
#define LPF_B1                              (1.7786f)
#define LPF_B2                              (-0.8008f)

/* HFI 1rd LPF COEFF 8k pwmfreq, 100Hz cutFreq */
#define THETA_LPF_COEFF                     (0.073f)      /*!< High frequency injection deta theta lpf coefficient */
/* HFI 1rd LPF COEFF 8k pwmfreq, 200Hz cutFreq */
#define CUR_LPF_COEFF                       (0.135f)      /*!< High frequency injection current loop lpf coefficient */

/*!
* @brief Self learning of PID regulator parameters
*/
#define CURRENT_PID_SELF_LEARN              DISABLE             /*!< Current loop PID parameter self-learning, ENABLE to support and DISABLE to ban this feature */
#define SPEED_PID_SELF_LEARN                DISABLE             /*!< Speed loop PID parameter self-learning, ENABLE to support and DISABLE to ban this feature */

/*!
* @brief Control loop bandwidth select
*/
#define CURRENT_REGULATOR_BANDWIDTH         2500                /*!< Bandwidth of current regulator, Unit in rad/s */
#define SPEED_REGULATOR_BANDWIDTH           250                 /*!< Default bandwidth of speed regulator, Unit in rad/s */
#define SPEED_DAMP_CONST                    4                   /*!< Motor damp const, range is 1 ~ 16 */

/*!
* @brief Position loop control parameters
*/
#if ((defined ENCODER_SENSORS) || (defined ENCODER_HALL_SENSORS))
//#define POSITION_LOOP_CTRL                                                        /*!< If not defined, motor will run in speed/torque control */
#endif
#define REDUCTION_RATIO_CONST               1                                     /*!< Motor reduction ratio, range is 1 ~ 60 */
#define POSITION_ACTION_MAX_TIME            (2000)                                /*!< Max time cost in the slowest position gear action, Unit in ms */
#define POSITION_GEAR_DEFAULT               0                                     /*!< Motor position run slowest gear, range is 0 ~ 2 */
#define POSITION_ANGLE_BASE                 (100.0f * PULSE_PER_REVOLUTION)        /*!< Motor position run max angle range, Unit in encoder pulse count */
#define POSITION_TARGET_DEFAULT             (0.0 / POSITION_ANGLE_BASE)    /*!< Default target Motor position, User need set the default target angle in encoder pulse count unit at molecule,
                                                                                  *    be careful the molecule must set in floating format, in order to do Math function calculate;
                                                                                  */

/*!
* @brief Gains values for torque and flux control loop
*/
#define PID_POSITION_KP_DEFAULT             (0.9f)        /*!< position loop PID kp, the setting range is Math_IQ(0.0) ~ Math_IQ(4.0) */
#define PID_POSITION_KI_DEFAULT             (0.000f)      /*!< position loop PID ki, the setting range is Math_IQ(0.0) ~ Math_IQ(4.0) */
#define PID_POSITION_KD_DEFAULT             0                   /*!< position loop PID kd, the default value is 0 since it is not used in the regulator calcaution */
#define PID_POSITION_MAX_DEFAULT            (1.0f)             /*!< position loop PID max, the setting range is -32768 ~ 32767 */
#define PID_POSITION_MIN_DEFAULT            (-1.0f)              /*!< position loop PID min, the setting range is -32768 ~ 32767 */

/*!
* @brief Gains values for torque and flux control loop
*/
#define PID_TORQUE_KP_DEFAULT               (0.6f)        /*!< torque loop PID kp, the setting range is Math_IQ(0.0) ~ Math_IQ(4.0) */
#define PID_TORQUE_KI_DEFAULT               (0.009f)      /*!< torque loop PID ki, the setting range is Math_IQ(0.0) ~ Math_IQ(4.0) */
#define PID_TORQUE_KD_DEFAULT               0                   /*!< torque loop PID kd, the default value is 0 since it is not used in the regulator calcaution */
#define PID_TORQUE_MAX_DEFAULT              (1.0f)               /*!< torque loop PID max, the setting range is -32768 ~ 32767 */
#define PID_TORQUE_MIN_DEFAULT              (-1.0f)              /*!< torque loop PID min, the setting range is -32768 ~ 32767 */

#define PID_FLUX_KP_DEFAULT                 (0.6f)        /*!< flux loop PID kp, the setting range is Math_IQ(0.0) ~ Math_IQ(4.0) */
#define PID_FLUX_KI_DEFAULT                 (0.009f)      /*!< flux loop PID ki, the setting range is Math_IQ(0.0) ~ Math_IQ(4.0) */
#define PID_FLUX_KD_DEFAULT                 0                   /*!< flux loop PID kd, the default value is 0 since it is not used in the regulator calcaution */
#define PID_FLUX_MAX_DEFAULT                (1.0f)               /*!< flux loop PID max, the setting range is -32768 ~ 32767 */
#define PID_FLUX_MIN_DEFAULT                (-1.0f)              /*!< flux loop PID min, the setting range is -32768 ~ 32767 */

/*!
* @brief Gains values for speed control loop
*/
#define PID_SPEED_KP_DEFAULT                (0.9f)        /*!< speed loop PID kp, the setting range is Math_IQ(0.0) ~ Math_IQ(4.0) */
#define PID_SPEED_KI_DEFAULT                (0.009f)      /*!< speed loop PID ki, the setting range is Math_IQ(0.0) ~ Math_IQ(4.0) */
#define PID_SPEED_KD_DEFAULT                0                   /*!< speed loop PID kd, the default value is 0 since it is not used in the regulator calcaution */
#define PID_SPEED_MAX_DEFAULT               (1.0f)               /*!< speed loop PID max, the setting range is -32768 ~ 32767 */
#define PID_SPEED_MIN_DEFAULT               (-1.0f)              /*!< speed loop PID min, the setting range is -32768 ~ 32767 */

#define FEED_FORWARD_DECOUPLE                                 /*!< Acr current loop feed forward decouple enable switch, defined to supported the feature, or commented to disable the feature */
#define FEED_FORWARD_LIMIT                  (0.866f)      /*!< Feed forward decouple voltage uplimit, sqrt(3)/2 = 0.866 */

//#define COMPLEX_VECTOR_DECOUPLE                               /*!< Acr current loop complex vector decouple enable switch, defined to supported the feature, or commented to disable the feature */

//#define LOAD_TORQUE_ESTIMATE                                  /*!< Load torque estimate enable switch, defined to supported the feature, or commented to disable the feature  */
#define LOAD_ESTIMATE_COEFF                 (0.9155f)               /*!< Load torque estimate gain coefficient */
#define LOAD_COMP_LIMIT                     (0.99f)       /*!< Load torque compensation limit value */

/*!
* @brief Inertia online identify parameters configuration, there are two ways of inertia identify: MOTOR_INERTIA_IDENT_MRAS and MOTOR_INERTIA_IDENT_FFLS, only one can be used at the same time.
*/
//#define INERTIA_ONLINE_IDENT                                                      /*!< Inertia identify enable switch, defined to supported the feature, or commented to disable the feature */

#define INERTIA_IDENT_MRAS                                                          /*!< Inertia identify of MRAS enable switch */
//#define INERTIA_IDENT_FFLS                                                        /*!< Inertia identify of FFLS enable switch */

#define INERTIA_IDENT_GAIN_MRAS            10                                       /*!< Inertia identify gain coefficient of MRAS */

#define INERTIA_IDENT_FACTOR_FFLS          (0.9999f)                                   /*!< Inertia identify forget factor of FFLS */
#define INERTIA_IDENT_PK_INIT_FFLS         100000                                   /*!< Inertia identify Pk parameter initial value of FFLS, 10^4~10^6 */

#define INERTIA_IDENT_COF_INIT             500                                      /*!< Inertia identify coefficient initial value, INERTIA_COF_INIT = OPERATION_PERIOD / J */
#define INERTIA_IDENT_OPERATION_PERIOD     (0.01f)                                     /*!< Inertia identify operation period, unit:s */
#define INERTIA_IDENT_ERROR_THRESHOLD      (4e-6)                                     /*!< Inertia error threshold, if inertia error less than ERROR_THRESHOLD && judge time high than STEADY_JUDGE_TIME, than output steady Inertia */
#define INERTIA_IDENT_STEADY_TIME          (5.0f / INERTIA_IDENT_OPERATION_PERIOD)   /*!< Inertia error steady judge time, 5.0 means judge time is 5 seconds */
#define INERTIA_IDENT_OLD_TIME             (5.0f / INERTIA_IDENT_OPERATION_PERIOD)   /*!< Last period inertia refresh time, 5.0 means last period inertia refresh time is 5 seconds */

//#define ASR_VARPI_ENABLE                                      /*!< Asr speed loop variable PI enable switch, defined to supported the feature, or commented to disable the feature */

//#define QUICK_DESATURATION                                    /*!< Asr speed loop quick desaturation enable switch, defined to supported the feature, or commented to disable the feature */
#define FEEDBACK_GAIN                       (0.01f)       /*!< Feedback suppression PI regulator feedback gain coefficient */

/*!
* @brief Foc flux weakening control parameters configuration.
*/
//#define FLUX_WEAKENING                                        /*!< Flux weakening function enable switch, defined to supported the feature, or commented to disable the feature */
#define FLUXWEAKE_VOLTAGE_REF               1000                /*!< voltage reference, tenth of a percent */
#define FLUX_WEAKE_KP                       (0.5f)        /*!< Default Kp gain */
#define FLUX_WEAKE_KI                       (0.015f)                 /*!< Default Ki gain */
#define IDMIN                               (-0.999f)     /*!< Flux weakening pid input min limit */
#define IDDEMAG                             (-0.5f)       /*!< Demagnetization current */
#define ISMAX                               (0.999f)      /*!< Motor nominal current */
#define MAXMODULE                           (0.95f)       /*!< Circle limitation maximum allowed module */

/*!
* @brief Maximum Torque Per Ampere strategy parameters configuration.
*/
//#define MTPA_ENABLE                     /*!< Max torque per ample feature enable switch, defined to supported the feature, or commented to disable the feature */

#endif  /* end of MOTOR_ONE_TYPE */


#if (defined MOTOR_TWO_TYPE)

/*!
* @brief This macro define determines whether to execute the parameters access to flash.
*/
#define DATA_ACCESS_FLASH_ENABLE_M2

/*!
* @brief There are two main control modes for three phase brushless motor(include MOTOR_BLDC,MOTOR_SPMSM...in motor_parameters_define.h), which are six step square wave mode and foc sine wave mode.
*        Six step square wave mode are configured in bldc_parameters_define.h; Foc sine wave mode are configured in drive_parameters_define.h.
*        If user choose foc sine wave mode, should undefine SIX_STEP_SQUARE_WAVE in bldc_parameters_define.h, and define FOC_SINE_WAVE in this file.
*        There are four foc sine wave control sub modes, which are sensorless control, hall sensor control, encoder control and encoder_hall control.
*        After define FOC_SINE_WAVE, user can continue choose the sub modes by define SENSORLESS,HALL_SENSORS...and so on, and please undefine the other three sub mode meanwhile.
*        IF user choose six step square wave mode, make sure to undefine FOC_SINE_WAVE and all the four sub modes.
*/
#define FOC_SINE_WAVE_M2

#define FOC_PMSM_M2
//#define FOC_ACIM_M2

#if (defined FOC_PMSM_M2)
#define SENSORLESS_M2
//#define HALL_SENSORS_M2
//#define ENCODER_SENSORS_M2
//#define ENCODER_HALL_SENSORS_M2

#elif (defined FOC_ACIM_M2)
#define SENSORLESS_ACIM_M2
//#define SENSOR_ACIM_M2

#else
#endif

/*!
* @brief If need to apply V/f control, the FOC_SINE_WAVE macro definition must be shield and turn on VF_CONTROL macro definition.
*/
//#define VF_CONTROL_M2

/*!
* @brief There are three startup methods for sensorless control: VF startup, IF startup and Direct startup.
*        When using high frequency injection sensorless mode, all of the three startup methods need to be shielded.
*/
//#define VF_STARTUP_M2
#define IF_STARTUP_M2
//#define DIRECT_STARTUP_M2

/*!< High frequence injection control mode when in low speed, need to close IF_STARTUP, VF_STARTUP, DIRECT_STARTUP and CLOSE_TO_OPEN_SWITCH before use */
//#define HIGH_FREQUENCE_INJECTION_M2

/*!
* @brief There are three sensorless control modes: namely the sliding mode control, flux observer mode and mras observer mode.
*        They are only suitable for medium and high speed applications.
*/
//#define SMC_OBSERVE_M2
#define FLUX_OBSERVE_M2
//#define MRAS_OBSERVE_M2

/*!
* @brief There are two sensorless angle obtain ways: namely the ATAN_ANGLE_ESTIMATION way and PLL_ANGLE_ESTIMATION way.
*/
//#define ATAN_ANGLE_ESTIMATION_M2
#define PLL_ANGLE_ESTIMATION_M2

/*!
* @brief There are two ways to pre-position the motor rotor: ALIGNMENT_ONCE, ALIGNMENT_TWICE.
*        In sensorless application, one of the two ways must be turned on.
*/
#define ALIGNMENT_ONCE_M2
//#define ALIGNMENT_TWICE_M2

/*!
* @brief If PULSE_INJECTION macro is defined, the initial position can be detect when the motor is stopped.
*/
//#define PULSE_INJECTION_M2
#define PULSE_INJECTION_TIMES_M2   2UL         /*!< Pulse injection times, When it is greater than 1, multiple pulse injection will be performed, the setting range is 1 ~ 5 */

/*!
* @brief If INIT_POS_DETEC macro is defined, use the rotation high frequency voltage injection method to check rotor position. Cannot be used at the same time with PULSE_INJECTION definition.
*/
//#define INIT_POS_DETEC_M2                  /*!< Initial position check with injecting rotation high frequency voltage */

/*!
* Inject DC current into the d axis, manually rotate the motor rotor, derermine the number
* of motor pole pairs according to the position of the resistance point.
*/
//#define MOTOR_POLES_OBTAIN_M2  /*!< Get the motor poles through the alignemnt method, close PULSE_INJECTION before use */

/*!
* There are two ways to obtain the sensorless input voltage: use voltage reconstruction and use svpwm voltage.
* Just need to open or close VOLTAGE_RESTRUCT to choose. Open the VOLTAGE_RESTRUCT means using the voltage reconstruction, close means using svpwm voltage.
*/
//#define VOLTAGE_RESTRUCT_M2

/*!
* @brief Sensorless operation mode, including open loop operation and closed loop operation.
*/
//#define OPEN_LOOP_M2
#define CLOSE_LOOP_M2

/*!
* @brief Over modulation control
*/
#define OVER_MODULATION_M2                     0UL                   /*!< Set 1 will enable over moduation. Set 0 will disable over modulation */

/*!
* @brief Svpwm base on zero sequence component, be effective while OVER_MODULATION is setting 0, only working within linear modulation region.
*/
#define ZERO_SVPWM_M2                          0UL                   /*!< Set 1 will reduce SVPWM time consumption than Set 0 */

/*!
* @brief SVPWM output control, chose SVPWM_SEGMENT_TYPE is SEVEN_SEGMENT or FIVE_SEGMENT.
*/
#define SVPWM_SEGMENT_TYPE_M2                  SEVEN_SEGMENT

/*!
* @brief Hall sensor order and install error identify.
*/
#define ORDER_ERROR_COMP_MODE_M2               1                   /*!< 0:force degree compensation 1:disperse degree compensation */
#define HALL_INSTALL_TYPE_M2                   0                   /*!< 0:120 degree hall install 1:60 degree hall install */

#define HALL_SENSOR_SELF_LEARN_M2              DISABLE             /*!< Hall self-learn. If this macro definition is opened, will perform the Hall angle self-learning function */
#define HALL_SELF_LEARN_MODE_M2                2                   /*!< Hall self learn mode, 1: learn order and theta; 2: learn order and theta and sector width */
#define HALL_SWAP_SECTORS_M2                   3                   /*!< Hall run swap state setors, set range is 1~3 */
#define HALL_SWAP_POS_ANGLE_M2                 30                  /*!< Init positive position while hall run swap state, set range is 0~60 */
#define HALL_SWAP_NEG_ANGLE_M2                 30                  /*!< Init negative position while hall run swap state, set range is 0~60 */
#define HALL_SWAP_TRQ_COFF_M2                  100                 /*!< Torque boost at low speed, set range is 0~100 */
#define HALL_SELF_LEARN_INTERVAL_M2            500                 /*!< Hall self-learn time interval */
#define HALL_SELF_LEARN_CUR_M2                 (0.5f)              /*!< Hall self-learn current value */
#define HALL_SELF_LEARN_ORDER_M2               15                  /*!< Hall self-learn hall order */
#define HALL_SELF_LEARN_THETA_M2               (3640.0f / 65536)   /*!< Hall self-learn hall installation error */

/*!
* Sensorless calculation frequency function location selection, ELEC_FREQUENCE_HIGH means to calculate in interrupt,
* ELEC_FREQUENCE_LOW means to calculate in lms task, only used in ATAN_ANGLE_ESTIMATION mode.
*/
//#define ELEC_FREQUENCE_HIGH_M2
#define ELEC_FREQUENCE_LOW_M2

#define ATAN_SPEED_FILT_H_M2                   (0.9861f)   /*!< Low pass filter coefficient used for ELEC_FREQUENCE_HIGH and
                                                                *    ATAN_ANGLE_ESTIMATION, the larger coefficient, the heavier filtering.
                                                                */
#define ATAN_SPEED_FILT_L_M2                   (0.9661f)   /*!< Low pass filter coefficient used for ELEC_FREQUENCE_LOW and
                                                                *    ATAN_ANGLE_ESTIMATION, the larger coefficient,t he heavier filtering.
                                                                */

/* M-T speed measure configuration */
#if ((defined ENCODER_SENSORS_M2) || (defined SENSOR_ACIM_M2))
#define MT_SPEED_MEASURE_M2
#endif

/*!
* @brief Speed adjust method, only one method can be chosed.
*/
#define SPEED_ADJUST_KEY_M2                                        /*!< If defined SPEED_ADJUST_KEY, use key adjust speed */
//#define SPEED_ADJUST_KNOB_M2                                     /*!< If defined SPEED_ADJUST_KNOB, use knob adjust speed */

#define MOTOR_CONTROL_MODE_M2                  SPEED_CONTROL_MODE  /*!< Motor control mode selection */

/*!
* @brief Torque control mode with speed limit.
*/
#define TORQUE_MODE_IQ_ADD_STEP_M2             (0.1f / 1000)     /*!< iq target command increase 3000 in 1000ms */
#define TORQUE_MODE_IQ_CMD_MAX_M2              (0.95f)       /*!< iq target command max value */
#define TORQUE_MODE_IQ_CMD_MIN_M2              (0.1f)        /*!< iq target command min value for initial iq default value */

#define VDQ_CIRCLE_LIMIT_CTRL_M2                                   /*!< Voltage output limit enable switch. If this macro definition is open, voltage output limit in FOC is supported */
#define VDQ_LIMIT_VALUE_M2                     (1.0f)        /*!< Voltage output limit value, 1.0 means the max amplitude of output voltage vector is 1.0 times of bus voltage */

/*!
* @brief Configuration of speed ramp.
*/
#define SPEED_CHANGE_STEP_M2                   (2.0f / BASE_FREQ_M2)                                /*!< Speed change step, 2.0 means increase 2Hz for each calculation cycle */
#define MAX_SPEED_VALUE_M2                     (0.9f)                                            /*!< Speed upper limit, 0.8 means max limit of motor running speed is 0.8 * MOTOR_MAX_SPEED_RPM */
#define MIN_SPEED_VALUE_M2                     (OPEN_LOOP_SPEED_HZ_M2 * POLE_PAIR_NUM_M2 / BASE_FREQ_M2) /*!< Speed lower limit */
#define STOP_SPEED_VALUE_M2                    (STOP_SPEED_HZ_M2 * POLE_PAIR_NUM_M2 / BASE_FREQ_M2)      /*!< Speed cutoff value at motor stop */

/*!
* @brief Configuration of acceleration and deceleration speed.
*/
#define ACCELERATION_RPS_M2                    ((600.0f * 10) / (MOTOR_MAX_SPEED_RPM_M2 * 100)) /*!< Acceleration of motor speed up in unit RPM/s, motor mechanical rotation speed increase per second.
                                                                                                *    600.0: 600rpm increase per second, must be float type. Can be modified to change the acceleration;
                                                                                                *    MOTOR_MAX_SPEED_RPM: motor speed max value, can not be modified;
                                                                                                *    100: 1s = 100 * (10ms), the speed increase every 10ms, can not be modified.
                                                                                                */
#define DECELERATION_RPS_M2                    ((600.0f * 10) / (MOTOR_MAX_SPEED_RPM_M2 * 100)) /*!< Deceleration of motor speed up in unit RPM/s, motor mechanical rotation speed decrease per second.
                                                                                                *    600.0: 600rpm increase per second, must be float type. Can be modified to change the deceleration;
                                                                                                *    MOTOR_MAX_SPEED_RPM: motor speed max value, can not be modified;
                                                                                                *    100: 1s = 100 * (10ms), the speed decrease every 10ms, can not be modified.
                                                                                                */

/*!
* @brief Parameter configuration of FOC open loop control.
*/
#define OPEN_LOOP_SPEED_HZ_M2                  (10.0f)                /*!< Final forced synchronize mechanical FREQ in Hz */
#define MAX_SYNC_SPEED_THRES_M2                (1.08f)       /*!< Final forced synchronize mechanical FREQ upper limit */
#define MIN_SYNC_SPEED_THRES_M2                (0.92f)       /*!< Final forced synchronize mechanical FREQ lower limit */
#define OPEN_LOOP_TIME_SYN_M2                  10                  /*!< Reach openloop target speed synchronization times */

#define CUR_START_SUB_M2                       5                   /*!< Current reduce step */

#define SPD_SMOOTH_CUR_M2                      (0.1f)                /*!< Smooth the sensorless speed switching process */

#define DEFAULT_TARGET_FREQ_START_M2           (OPEN_LOOP_SPEED_HZ_M2 * POLE_PAIR_NUM_M2 / BASE_FREQ_M2)    /*!< Default target frequency for sensorless mode */

#define STOP_SPEED_HZ_M2                       (2.0f)                 /*!< Mechanical frequency cutoff value in Hz at motor stop */

#define OPEN_LOOP_CURRENT_M2                   (0.2f)        /*!< Three phase current amplitude in Q15 format if in IF_STARTUP mode */
#define OPEN_LOOP_VOLTAGE_M2                   (0.4f)        /*!< Three phase voltage amplitude in Q15 format if in VF_STARTUP mode */

/*!
* @brief Sensorless foc close loop to open loop switch when in low speed, need to close HIGH_FREQUENCE_INJECTION before use.
*/
//#define CLOSE_TO_OPEN_SWITCH_M2
#define SPEED_DELAY_ZONE_M2                     (2.0f * POLE_PAIR_NUM_M2 / BASE_FREQ_M2)    /*!< Close and open loop switch speed delay zone, need to be set less than MIN_SPEED_VALUE */

/*!
* @brief Parameter configuration of FOC open loop start-up curve.
*/
#define CURVE_TYPE_M2                          CURVE_TYPE_STRAIGHT /*!< select curve type */

#define TIME_DURATION1_M2                      (uint32_t)1000U     /*!< 1000ms = 1s, time duration value of 1st segment, setted by tools */
#define TIME_DURATION2_M2                      (uint32_t)1000U     /*!< 1000ms = 1s, time duration value of 2nd segment */
#define TIME_DURATION3_M2                      (uint32_t)1000U     /*!< 1000ms = 1s, time duration value of 3rd segment */
#define TIME_DURATION4_M2                      (uint32_t)1000U     /*!< 1000ms = 1s, time duration value of 4th segment */
#define TIME_DURATION5_M2                      (uint32_t)1000U     /*!< 1000ms = 1s, time duration value of 5th segment */

#define CUSTOMER_CURVE_POINTS_M2               1U                   /*!< curve segment number, configed by host tool */

#define FREQ_REF_INITVALUE_M2                  (0.0f / BASE_FREQ_M2)    /*!< initial state: 0Hz */
#define CUR_REF_INITVALUE_M2                   OPEN_LOOP_CURRENT_M2   /*!< initial current, unit: pu */
#define VOLT_REF_INITVALUE_M2                  (0.1f)        /*!< initial voltage, unit: pu */

/*!
* If select CURVE_TYPE_STRAIGHT, FREQ_REF_VALUE_1 is the target frequency of open loop;
* If select 5 points CURVE_TYPE_CUSTOMER, FREQ_REF_VALUE_5 is the final target frequency of open loop.
*/
#define FREQ_REF_VALUE_1_M2                    (OPEN_LOOP_SPEED_HZ_M2 * POLE_PAIR_NUM_M2 / BASE_FREQ_M2)  /*!< set the 1st point frequency value */
#define FREQ_REF_VALUE_2_M2                    (8.0f / BASE_FREQ_M2)    /*!< set the 2nd point frequency value */
#define FREQ_REF_VALUE_3_M2                    (10.0f / BASE_FREQ_M2)   /*!< set the 3rd point frequency value */
#define FREQ_REF_VALUE_4_M2                    (12.0f / BASE_FREQ_M2)   /*!< set the 4th point frequency value */
#define FREQ_REF_VALUE_5_M2                    MIN_SPEED_VALUE_M2             /*!< set the 5th point frequency value */

#define CUR_REF_VALUE_1_M2                     OPEN_LOOP_CURRENT_M2           /*!< set the 1st point current value */
#define CUR_REF_VALUE_2_M2                     (0.3f)                /*!< set the 2nd point current value */
#define CUR_REF_VALUE_3_M2                     (0.15f)               /*!< set the 3rd point current value */
#define CUR_REF_VALUE_4_M2                     (0.2f)                /*!< set the 4th point current value */
#define CUR_REF_VALUE_5_M2                     OPEN_LOOP_CURRENT_M2           /*!< set the 5th point current value */

#define VOLT_REF_VALUE_1_M2                    (0.15f)               /*!< set the 1st point voltage value */
#define VOLT_REF_VALUE_2_M2                    (0.1f)                /*!< set the 2nd point voltage value */
#define VOLT_REF_VALUE_3_M2                    (0.11f)               /*!< set the 3rd point voltage value */
#define VOLT_REF_VALUE_4_M2                    (0.125f)              /*!< set the 4th point voltage value */
#define VOLT_REF_VALUE_5_M2                    (0.15f)               /*!< set the 5th point voltage value */

/*!
* @brief S-curve speed ramp definition. S-curve will be usde to generate speed ramp with different accelerations at different stages.
*/
//#define SCURVE_SPEED_RAMP_M2
#define ACC_START_SEG_RATIO_M2                 (0.3f)        /*!< The time proportion of the start segment(acceleration increase) of S-curve, m value */
#define ACC_END_SEG_RATIO_M2                   (0.3f)        /*!< The time proportion of the end segment(acceleration reduction) of S-curve, n value */
#define DEC_START_SEG_RATIO_M2                 (0.3f)        /*!< The time proportion of the start segment(deceleration increase) of S-curve, m value */
#define DEC_END_SEG_RATIO_M2                   (0.3f)        /*!< The time proportion of the start segment(deceleration reduction) of S-curve, n value */

#define MOTOR_STOP_MODE_M2                     FREE_RUN_STOP       /*!< Motor stop mode */

#define MOTOR_DIR_M2                           FORWARD_ROTATE      /*!< Motor direction 1: foward rotate; -1: reverse rotate */

#define SPEED_FIFO_DEPTH_M2                    64                  /*!< Depth of the FIFO used to average speed use in lib */

//#define TAIL_HEAD_WIND_M2

#define MOTOR_MOTION_BEMF_AMP_THRESHOLD_M2     50                /*!< Initial speed judgment back emf ad sample value amplitude threshold, higher than the threshold means the motor initial speed state is in motion, 0-4096 */
#define HIGH_REVERSE_THRESHOLD_M2              (-0.15f)          /*!< High reverse threshold, higher than the threshold means the motor initial speed state is HIGH_REVERSE. */
#define STATIC_THRESHOLD_M2                    (0.01f)           /*!< Static threshold, Lower than the threshold means the motor initial speed state is STATIC. */
#define HIGH_FORWARD_THRESHOLD_M2              (0.05f)           /*!< High forward threshold, higher than the threshold means the motor initial speed state is HIGH_FORWARD. */

/*!
* @brief Short brake configuration.
*/
#define SHORT_BRAKE_CTRL_M2                    0                    /*!< 0:disable 1:before startup brake 2:after stop brake 3:before startup and after stop, both brake */
#define SHORT_BRAKE_CUR_RATIO_M2               (1.0f)               /*!< Short brake max current pu value, set range is 0~100 */
#define SHORT_BRAKE_UDC_MAX_M2                 35                   /*!< Short brake max Udc value */
#define SHORT_BRAKE_TIMES_M2                   800                  /*!< Short brake finished wait time uint:ms, set range is 0~8000 */
#define SHORT_QUIT_CUR_M2                      (0.1f)               /*!< Quit short brake function current threshold */
#define SHORT_TIME_M2                          5000                 /*!< Single R sample shortbrake time  uint: ms */

/*!
* @brief Electric brake parameters configuration.
*/
//#define ELEC_BRAKE_ENABLE_M2                                     /*!< Electric brake enable switch, if this macro definition is opened, the electric brake function is supported */
#define ENTER_BRAKE_DEPTH_M2                   (0.00152f)               /*!< Enter electric brake min depth 0~32767 */
#define EXIT_BRAKE_DEPTH_M2                    (0.00137f)               /*!< Exit electric brake max depth */
#define ENTER_ELEC_BRAKE_FREQ_M2               (30.0f / BASE_FREQ_M2)   /*!< Enter electric brake min frequence */
#define EXIT_ELEC_BRAKE_FREQ_M2                (20.0f / BASE_FREQ_M2)   /*!< Exit electric brake max frequence */
#define ISQ_ADD_STEP_M2                        (0.0003f)                /*!< Isqref add step */
#define ISQ_DEC_STEP_M2                        (0.0003f)                /*!< Isqref dec step */
#define ELEC_BRAKE_CUR_RATIO_M2                (0.5f)                   /*!< Max current higher than 50% * 32768 pwmoff */
#define ELEC_BRAKE_UDC_MAX_M2                  15                       /*!< Dc voltage higher than 15V  pwmoff ,15 can adjust according to dc voltage*/
#define ELEC_BRAKE_DEPTH_M2                    0                        /*!< Input electric brake depth */
#define ELEC_BRAKE_MAX_DEPTH_M2                (0.999f)                 /*!< Input electric brake max depth */

/*!
* @brief Power and bus current estimate parameters configuration.
*/
#define POWER_DCCUR_EST_ENABLE_M2                                   /*!< Power and bus current estimate enable switch, if this macro definition is opened, the input power and DC bus current estimated function is supported */
#define PWM_DELAY_COF_M2                       (0.31f)              /*!< Power estimate pwm delay coffe */
#define UQCOMP_M2                              (0.018f)             /*!< Power estimate uq compensation */

/*!
* @brief Parameter configuration.
*/
#define PARAM_IDENTIFY_SEL_M2                  (0x07UL)            /*!< bit0: rs identify. bit1: Ld Lq identify. bit2: flux identify. bit3: inertia identify */

/* Stator resistance identification phase select */
#define RS_ID_PHASE_SEL_M2                     (1UL)                /*!< Select motor stator phase order to identify stator resistance, 1: One phase; 2: Two phase; 3: Three phase */
/* The following two current setting, 0.4 means the amplitude of the injection current is 0.4 * MAX_CURRENT. Excessive setting will lead to motor heating */
#define RS_ID_FIRST_INJECT_M2                  (0.3f)               /*!< Rs identify first inject current threshold */
#define RS_ID_SECOND_INJECT_M2                 (0.6f)               /*!< Rs identify second inject current threshold */
/* The following three time settings affect the duration of current injection, excessive setting will lead to motor heating */
#define RS_STABLE_DELAY_M2                     (1000UL)             /*!< Unit: ms, wait for stable after changing the amplitude of the injection current */
#define RS_SETTING_DELAY_M2                    (500UL)              /*!< Unit: ms, wait for changing the amplitude of the injection current */
#define RS_SAMPLE_DELAY_M2                     (500UL)              /*!< Unit: ms, wait for sampling and saving current and voltage */

/* PMSM Ldq identify configuration */
#define PMSM_LDQ_METHOD_SEL_M2                 PMSM_LDQ_METHOD_HFS /*!< PMSM Ldq identification method select */

/* PMSM Ldq method HFP */
#define PMSMID_LDQ_INIT_PRD_M2                 (600UL)              /*!< Ldq identify initial pwm period value. Generally, it does not need to be modified*/
#define PMSMID_LDQ_CUR_LIMIT_M2                (0.4f)               /*!< Ldq identify max current limit*/

#define PMSMID_ALIGN_TIME1_M2                  T_ALIGNMENT1_M2      /*!< First alignment time of Ldq identification in ms */
#define PMSMID_ALIGN_TIME2_M2                  T_ALIGNMENT2_M2      /*!< Second alignment time of Ldq identification in ms */
#define PMSMID_ALIGN_CUR_M2                    I_ALIGNMENT1_M2      /*!< Alignment current of Ldq identification */
#define PMSMID_ALIGN_WAIT_M2                   (10UL)               /*!< Alignment done and wait time of Ldq identification in ms */

/* PMSM Ldq method HFS */
#define PMSMID_CUR_DC_SET_M2                   (0.3f)           /*!< PMSM Ldq identification DC current set value */
#define PMSMID_CUR_AC_SETD_M2                  (0.3f)           /*!< PMSM Ldq identification d-axis AC current set value */
#define PMSMID_CUR_AC_SETQ_M2                  (0.3f)           /*!< PMSM Ldq identification q-axis AC current set value */
#define PMSMID_CUR_FREQ_SETD_M2                (3.0f)           /*!< PMSM Ldq identification d-axis AC current frequency set value */
#define PMSMID_CUR_FREQ_SETQ_M2                (3.0f)           /*!< PMSM Ldq identification q-axis AC current frequency set value */
#define PMSMID_CUR_DC_STEP_M2                  (0.002f)         /*!< PMSM Ldq identification DC current variable step value */

#define STABLE_JUDGE_LIMIT_M2                  (0.6f)           /*!< Signal stable judgement limit */

#define PMSMID_LDQ_PWMFREQ_M2                  (11000UL)           /*!< PWM frequency in pmsm Ldq identification */
#if (PWM_MODE_SELECT == PWM_COUNT_UP_DOWN_MODE)
#define PMSMID_LDQ_PWMPRD_M2        (((MOTOR_BUS_CLK / (PMSMID_LDQ_PWMFREQ_M2 * (PWM_PRSC + 1))) >> 1) - 1)       /*!< PWM period value in pmsm Ldq identification */
#else
#define PMSMID_LDQ_PWMPRD_M2        ((MOTOR_BUS_CLK / (PMSMID_LDQ_PWMFREQ_M2 * (PWM_PRSC + 1))) - 1)              /*!< PWM period value in pmsm Ldq identification */
#endif

/* PMSM flux identify configuration */
#define PMSMID_FLUX_FRE_M2                     (0.2f)       /*!< Flux identify set frequency of flux identification */
#define PMSMID_FLUX_CUR_M2                     (0.25f)      /*!< Flux identify set current of flux identification, must less than Math_IQ(0.5) */
#define PMSMID_FLUX_IF_ACC_M2                  (0.0001f)    /*!< Flux identify IF run  acceleration */

/* ACIM identify configuration */
#define ACIMID_RR_CURRENT_M2                   (0.5f)       /*!< Acim stall test current limit */
#define ACIMID_RR_FREQ_M2                      (0.6f)       /*!< Acim stall test frequency limit */
#define ACIMID_LM_FREQ_M2                      (0.6f)       /*!< Acim rotation test frequency limit */
#define ACIMID_RR_CURSTEP_M2                   (0.0003f)    /*!< Acim stall test current step */
#define ACIMID_LM_FREQSTEP_M2                  (0.00015f)   /*!< Acim rotation test frequency step */

#define ACIMID_CURRENT_THRESHOLD_M2            (0.6f)       /*!< Acim identification current threshold to judge current state */
#define ACIMID_VOLTAGE_THRESHOLD_M2            (0.8f)       /*!< Acim identification voltage threshold to judge voltage state */

/* Identify signal extraction configuration */
#define PARID_DATANUM_M2                       (2048UL)     /*!< Motor identification data calculation number */
#define PARID_FILTER1COEFF1_M2                 (PWM_FREQ / (PI2 * 100))     /*!< Motor identification signal filter-1 coefficient */
#define PARID_FILTER1COEFF2_M2                 (PWM_FREQ / (PI2 * 160))     /*!< Motor identification signal filter-1 coefficient */
#define PARID_FILTER2COEFF1_M2                 (PWM_FREQ / (PI2 * 100))     /*!< Motor identification signal filter-2 coefficient */
#define PARID_FILTER2COEFF2_M2                 (PWM_FREQ / (PI2 * 160))     /*!< Motor identification signal filter-2 coefficient */

/*!
* @brief Dead time Compensation control define
*/
//#define DEAD_BAND_COMP_M2                                        /*!< If defined DEAD_BAND_COMP will Enable dead time compensation */
#define CUTOFF_CUR_VAL_M2                      (0.25f)       /*!< Dead band compensation current threshold */
#define MAX_COMP_FREQ_M2                       (0.5f)        /*!< Dead band compensation speed threshold */

/*!
* @brief Over voltage supress at slow down stop mode.
*/
//#define OVER_VOL_SUPPRESS_M2                 /*!< If defined OVER_VOL_SUPPRESS will enable over voltage suppression */

/*!
* @brief Narrow pulse elimination process.
*/
//#define NARROW_PULSE_PROCESS_M2             /*!< If defined  NARROW_PULSE_PROCESS will enable narrow pulse elimination */

/*!
* @brief Energy save function configuration
*/
//#define SAVE_ENERGY_M2                                           /*!< If defined SAVE_ENERGY will enable energy save function */
#define MAX_CUR_CHANGE_M2                      (0.02f)       /*!< Max current change threshold*/
#define STEADY_CUR_CHANGE_M2                   (0.01f)       /*!< Before enter energy save function, check current steady */
#define CUR_CHANGE_STEP_M2                     (0.003f)      /*!< D axis current change step value */
#define ENERGY_SAVE_DEC_M2                     (0.01f)       /*!< Current amplitude decrease threshold */
#define ENERGY_SAVE_INC_M2                     (0.01f)       /*!< Current amplitude increase threshold */
#define RECHECK_CUR_CHANGE_M2                  (0.05f)       /*!< Load change threshold to enter energy save again */
#define MAX_ID_TARGET_M2                       (0.5f)        /*!< D axis current max value*/
#define MIN_ID_TARGET_M2                       (-0.5f)       /*!< D axis current min value */
#define WAIT_TIME_M2                           (11)          /*!< Interval waite time value: 2^n *1ms, e.g.: this value is 11, then the time is 2^11 *1ms = 2048ms */
#define ENTER_ENERGY_SAVE_LOAD_M2              (0.3f)        /*!< Enter energy save function load threshold value */

/*!
* @brief Sliding mode control(SMC) parameters configuration.
*/
/* SMC parameters configuration before close loop */
#define SMC_KSLIDE0_M2                         (0.3f)       /*!< Open loop configuration sliding mode gain */
#define SMC_KSLF0_M2                           (0.46f)      /*!< Open loop configuration SMC filter coefficient, the smaller the value, the more obvious the filtering effect, 0~32767 */
#define SMC_MAXERR0_M2                         (0.8f)       /*!< Open loop configuration sliding mode boundary */
/* SMC parameters configuration after close loop */
#define SMC_KSLIDE1_M2                         (0.7f)           /*!< Close loop configuration sliding mode gain */
#define SMC_KSLF1_M2                           SMC_KSLF0_M2     /*!< Close loop configuration sliding mode filter coefficient */
#define SMC_MAXERR1_M2                         (0.8f)           /*!< Close loop configuration sliding mode boundary */

#define FULL_ORDER_SMC                                      /*!< Full order smc enable switch, only works when SMC_OBSERVE and FULL_ORDER_SMC are defined */
#define FULL_SMC_GAIN_M2                       (0.3f)       /*!< Full order smc gain coefficients */
#define FULL_SMC_FILT_M2                       DISABLE      /*!< Full order smc filter enable switch, ENABLE means enable filter feature, DISABLE means disable filter feature */
/*!
* @brief Flux observer parameters configuration.
*/
#define FLIT_COEFF_M2                          (0.061f)     /*!< Flux observer LPF filter coefficient, the smaller the value, the more obvious the filtering effect, 0-32767 */
#define COMPCOF_UPLIMIT_M2                     (5.0f)       /*!< Flux observer back emf compensation uplimit, Math_IQ(0)-Math_IQ(100) */

/*!
* @brief Mras observer parameters configuration.
*/
#define ALPHA_COEFF_M2                         (0.061f)     /*!< Mras observer alpha coffcient, use this parameter calculate we,  0~32767 */
#define BETA_COEFF_M2                          (0.46f)      /*!< Mras observer beta coffcient, use this parameter calculate correct we, 0~45875, decreases with the increase of PWM frequency */

/* Only in case of PLL_ANGLE_ESTIMATION is defined, this coefficient is used to set PI parameters of the pll */
#define COEFFPLL_SMC_M2                        (0.091f)     /*!< Smc observer pll PI set coefficient, need to be set less than 16384 */
#define COEFFPLL_FLUX_M2                       (0.091f)     /*!< Flux observer pll PI set coefficient, need to be set less than 16384 */

/*!
* @brief Sensorless pre-positioning parameters configuration.
*/
#define T_ALIGNMENT1_M2                        (uint32_t)(1000)     /*!< First alignment time in ms */
#define T_ALIGNMENT2_M2                        (uint32_t)(1000)     /*!< Second alignment time in ms */
#define ALIGNMENT_ANGLE_DEGREE_M2              (270.0f)             /*!< Alignment angle in degree */
#define ALIGNMENT_ANGLE_M2                     (ALIGNMENT_ANGLE_DEGREE_M2 / 360.0f)   /*!<Alignment Degrees in Q16 */
#define I_ALIGNMENT1_M2                        (0.1f)               /*!< First alignment current */
#define I_ALIGNMENT2_M2                        OPEN_LOOP_CURRENT_M2     /*!< Second alignment current */
#define I_RAMP1_INIT_M2                        (0)                      /*!< Initial value of first alignment current ramp */
#define I_RAMP2_INIT_M2                        I_ALIGNMENT1_M2          /*!< Initial value of second alignment current ramp */
#define T_SLOPE1_M2                            (T_ALIGNMENT1_M2 >> 2)   /*!< Duration of first alignment current ramp in ms */
#define T_SLOPE2_M2                            (T_ALIGNMENT2_M2 >> 5)   /*!< Duration of second alignment current ramp in ms */

/*!
* @brief Initial position check configuration
*/
//#define PULSE_SELF_LEARN_M2                                                     /*!< If not defined, will cancel pulse width learn process */
#define PWM_SELF_LEARN_PRD_M2                  (2100)                           /*!< If cancel pulse width learn process, set pulse inject width */
#define INIT_PULSE_PERIOD_M2                   ((MOTOR_BUS_CLK / 20000) >> 1)   /*!< initial pwm period 20k */
#define SELF_LEARN_THRES_M2                    (MAX_CURRENT_M2 * 0.3f * (ADC_RANGE * RSHUNT * OP_AMPLIFICATION_GAIN / VSVREF) + (ADC_RANGE >> 1))
#define SELF_LEARN_DUTY_STEP_M2                (240U)                            /*!< pwm width slef learn duty increase step */

#define INITPOS_VOLT_AMP_M2                    (0.45f)      /*!< Initial position check injection rotation voltage amplitude */
#define INITPOS_VOLT_FREQ_M2                   (3.0f)       /*!< Initial position check injection rotation voltage frequency */
#define INITPOS_PULSE_AMP_M2                   (0.6f)       /*!< Initial position check injection pulse voltage amplitude */
#define INITPOS_PULSE_INJECT_TIME_M2           (1U)         /*!< Initial position check injection pulse voltage times */
#define INITPOS_PULSE_CLEAR_TIME_M2            (100U)       /*!< Initial position check injection pulse voltage clear times */

/*!
* @brief High frequency injection parameters configuration.
*/
#define HFI_DEBUG_MODE_M2                      FALSE        /*!< High frequency injection debug mode enable switch, FALSE means HFI run mode, TRUE means debug mode */
#define HFI_TO_STO_M2                          ENABLE       /*!< High frequency injection switch to state observer(STO) enable switch,
                                                            *    DISABLE means can not switch to high speed strategy,
                                                            *    ENABLE means can switch to high speed strategy
                                                            */

#define HFI_FREQUENCY_M2                       (600U)       /*!< High frequency injection electric frequency unit Hz */
#define HFI_AMPLITUDE_M2                       (0.2f)       /*!< High frequency injection voltage amplitude */
#define HFI_PULSE_AMP_M2                       (0.5f)       /*!< Pulse voltage amplitude */
#define HFI_PULSE_TIME_M2                      (3U)                 /*!< Pulse voltage inject time, 3 means 3 PWM interrupt period, LG:4 is better */
#define HFI_CLEAR_TIME_M2                      (30U * MS_UINT_INT)  /*!< Current clear to zero time, 30 means 30ms*/
#define HFI_JUDGE_TIME1_M2                     (300U * MS_UINT_INT) /*!< High frequency injection first judge time, 300 means 300ms */
#define HFI_JUDGE_TIME2_M2                     (60U * MS_UINT_INT)  /*!< High frequency injection second judge time, 60 means 60ms */
#define HFI_INIT_TIME_M2                       (60U * MS_UINT_INT)  /*!< High frequency injection initial inject time, 60 means 60ms */
#define HFI_INIT_THETA_M2                      (0.1667f)            /*!< High frequency injection initial theta, 10923 means 60 degree */

#define HFI_PLL_KP_DEFAULT_M2                  (0.305f)             /*!< High frequency injection pll kp */
#define HFI_PLL_KI_DEFAULT_M2                  (0.003f)             /*!< High frequency injection pll ki */

#define HFI_INIT_FREQ_M2                       (4.0f * POLE_PAIR_NUM_M2 / BASE_FREQ_M2)     /*!< High frequency injection default initial frequence, 4.0 means 4*60=240rpm */
#define HFI_FREQ_HIGH_M2                       (8.0f * POLE_PAIR_NUM_M2 / BASE_FREQ_M2)     /*!< High frequency injection to observer frequence threshold, 8.0 means 8*60=480rpm */
#define HFI_FREQ_LOW_M2                        (5.0f * POLE_PAIR_NUM_M2 / BASE_FREQ_M2)     /*!< Observer to high frequency injection frequence threshold, 5.0 means 5*60=300rpm */
#define HFI_VD_FREQ_HIGH_M2                    (12.0f * POLE_PAIR_NUM_M2 / BASE_FREQ_M2)    /*!< High frequency injection voltage amplitude decrease frequence high threshold, 12.0 means 12*60=720rpm*/
#define HFI_VD_FREQ_LOW_M2                     (8.0f * POLE_PAIR_NUM_M2 / BASE_FREQ_M2)     /*!< High frequency injection voltage amplitude decrease frequence low threshold, 8.0 means 8*60=480rpm */
#define HFI_DIR_ERR_TIME_M2                    (100U * MS_UINT_INT)                         /*!< High frequency injection run dir error judge time, 100 means 100ms*/
#define HFI_DIR_ERR_FREQ_M2                    (0.009f)                                     /*!< High frequency injection run dir error frequence judge threshold */

/*!
* @brief Two order band pass filter and low pass filter parameters design, user can refer to "ATC_AC78xx_Filter_Parameter_Setting" document.
*        Filter transfer function: y(n) = A0 * x(n) + A1 * x(n-1) + A2 * x(n-2) + B1 * y(n-1) + B2 * y(n-2).
*        Different filters can be obtained by  matching different coefficients A and B.
*/
/* HFI 2rd BPF COEFF 8k pwmfreq, 200-1000 passFreq */
#define BPF_A0_M2                              (0.2452f)
#define BPF_A2_M2                              (-0.2452f)
#define BPF_B1_M2                              (1.4142f)
#define BPF_B2_M2                              (-0.5095f)
/* HFI 2rd LPF COEFF 8k pwmfreq, 200Hz cutFreq */
#define LPF_A0_M2                              (0.0055f)
#define LPF_A1_M2                              (0.0111f)
#define LPF_A2_M2                              (0.0055f)
#define LPF_B1_M2                              (1.7786f)
#define LPF_B2_M2                              (-0.8008f)

/* HFI 1rd LPF COEFF 8k pwmfreq, 100Hz cutFreq */
#define THETA_LPF_COEFF_M2                     (0.073f)      /*!< High frequency injection deta theta lpf coefficient */
/* HFI 1rd LPF COEFF 8k pwmfreq, 200Hz cutFreq */
#define CUR_LPF_COEFF_M2                       (0.135f)      /*!< High frequency injection current loop lpf coefficient */

/*!
* @brief Self learning of PID regulator parameters
*/
#define CURRENT_PID_SELF_LEARN_M2              DISABLE             /*!< Current loop PID parameter self-learning, ENABLE to support and DISABLE to ban this feature */
#define SPEED_PID_SELF_LEARN_M2                DISABLE             /*!< Speed loop PID parameter self-learning, ENABLE to support and DISABLE to ban this feature */

/*!
* @brief Control loop bandwidth select
*/
#define CURRENT_REGULATOR_BANDWIDTH_M2         (2500U)                /*!< Bandwidth of current regulator, Unit in rad/s */
#define SPEED_REGULATOR_BANDWIDTH_M2           (250U)                 /*!< Default bandwidth of speed regulator, Unit in rad/s */
#define SPEED_DAMP_CONST_M2                    (4U)                   /*!< Motor damp const, range is 1 ~ 16 */

/*!
* @brief Position loop control parameters
*/
#if ((defined ENCODER_SENSORS_M2) || (defined ENCODER_HALL_SENSORS_M2))
//#define POSITION_LOOP_CTRL_M2                                                        /*!< If not defined, motor will run in speed/torque control */
#endif
#define REDUCTION_RATIO_CONST_M2               (1U)                                 /*!< Motor reduction ratio, range is 1 ~ 60 */
#define POSITION_ACTION_MAX_TIME_M2            (2000U)                              /*!< Max time cost in the slowest position gear action, Unit in ms */
#define POSITION_GEAR_DEFAULT_M2               (0U)                                 /*!< Motor position run slowest gear, range is 0 ~ 2 */
#define POSITION_ANGLE_BASE_M2                 (100.0f * PULSE_PER_REVOLUTION_M2)   /*!< Motor position run max angle range, Unit in encoder pulse count */
#define POSITION_TARGET_DEFAULT_M2             (0.0f / POSITION_ANGLE_BASE_M2)      /*!< Default target Motor position, User need set the default target angle in encoder pulse count unit at molecule,
                                                                                    *    be careful the molecule must set in floating format, in order to do Math function calculate;
                                                                                    */

/*!
* @brief Gains values for torque and flux control loop
*/
#define PID_POSITION_KP_DEFAULT_M2             (0.9f)        /*!< position loop PID kp, the setting range is Math_IQ(0.0) ~ Math_IQ(4.0) */
#define PID_POSITION_KI_DEFAULT_M2             (0.000f)      /*!< position loop PID ki, the setting range is Math_IQ(0.0) ~ Math_IQ(4.0) */
#define PID_POSITION_KD_DEFAULT_M2             (0.0f)        /*!< position loop PID kd, the default value is 0 since it is not used in the regulator calcaution */
#define PID_POSITION_MAX_DEFAULT_M2            (1.0f)        /*!< position loop PID max, the setting range is -32768 ~ 32767 */
#define PID_POSITION_MIN_DEFAULT_M2            (-1.0f)       /*!< position loop PID min, the setting range is -32768 ~ 32767 */

/*!
* @brief Gains values for torque and flux control loop
*/
#define PID_TORQUE_KP_DEFAULT_M2               (0.6f)        /*!< torque loop PID kp, the setting range is Math_IQ(0.0) ~ Math_IQ(4.0) */
#define PID_TORQUE_KI_DEFAULT_M2               (0.009f)      /*!< torque loop PID ki, the setting range is Math_IQ(0.0) ~ Math_IQ(4.0) */
#define PID_TORQUE_KD_DEFAULT_M2               (0.0f)        /*!< torque loop PID kd, the default value is 0 since it is not used in the regulator calcaution */
#define PID_TORQUE_MAX_DEFAULT_M2              (1.0f)        /*!< torque loop PID max, the setting range is -32768 ~ 32767 */
#define PID_TORQUE_MIN_DEFAULT_M2              (-1.0f)       /*!< torque loop PID min, the setting range is -32768 ~ 32767 */

#define PID_FLUX_KP_DEFAULT_M2                 (0.6f)        /*!< flux loop PID kp, the setting range is Math_IQ(0.0) ~ Math_IQ(4.0) */
#define PID_FLUX_KI_DEFAULT_M2                 (0.009f)      /*!< flux loop PID ki, the setting range is Math_IQ(0.0) ~ Math_IQ(4.0) */
#define PID_FLUX_KD_DEFAULT_M2                 (0.0f)        /*!< flux loop PID kd, the default value is 0 since it is not used in the regulator calcaution */
#define PID_FLUX_MAX_DEFAULT_M2                (1.0f)        /*!< flux loop PID max, the setting range is -32768 ~ 32767 */
#define PID_FLUX_MIN_DEFAULT_M2                (-1.0f)       /*!< flux loop PID min, the setting range is -32768 ~ 32767 */

/*!
* @brief Gains values for speed control loop
*/
#define PID_SPEED_KP_DEFAULT_M2                (0.9f)       /*!< speed loop PID kp, the setting range is Math_IQ(0.0) ~ Math_IQ(4.0) */
#define PID_SPEED_KI_DEFAULT_M2                (0.009f)     /*!< speed loop PID ki, the setting range is Math_IQ(0.0) ~ Math_IQ(4.0) */
#define PID_SPEED_KD_DEFAULT_M2                (0.0f)       /*!< speed loop PID kd, the default value is 0 since it is not used in the regulator calcaution */
#define PID_SPEED_MAX_DEFAULT_M2               (1.0f)       /*!< speed loop PID max, the setting range is -32768 ~ 32767 */
#define PID_SPEED_MIN_DEFAULT_M2               (-1.0f)      /*!< speed loop PID min, the setting range is -32768 ~ 32767 */

#define FEED_FORWARD_DECOUPLE_M2                            /*!< Acr current loop feed forward decouple enable switch, defined to supported the feature, or commented to disable the feature */
#define FEED_FORWARD_LIMIT_M2                  (0.866f)     /*!< Feed forward decouple voltage uplimit, sqrt(3)/2 = 0.866 */

//#define COMPLEX_VECTOR_DECOUPLE_M2                         /*!< Acr current loop complex vector decouple enable switch, defined to supported the feature, or commented to disable the feature */

//#define LOAD_TORQUE_ESTIMATE_M2                            /*!< Load torque estimate enable switch, defined to supported the feature, or commented to disable the feature  */
#define LOAD_ESTIMATE_COEFF_M2                 (0.9155f)    /*!< Load torque estimate gain coefficient */
#define LOAD_COMP_LIMIT_M2                     (0.99f)      /*!< Load torque compensation limit value */

/*!
* @brief Inertia online identify parameters configuration, there are two ways of inertia identify: MOTOR_INERTIA_IDENT_MRAS and MOTOR_INERTIA_IDENT_FFLS, only one can be used at the same time.
*/
//#define INERTIA_ONLINE_IDENT_M2                           /*!< Inertia identify enable switch, defined to supported the feature, or commented to disable the feature */

#define INERTIA_IDENT_MRAS_M2                               /*!< Inertia identify of MRAS enable switch */
//#define INERTIA_IDENT_FFLS_M2                             /*!< Inertia identify of FFLS enable switch */

#define INERTIA_IDENT_GAIN_MRAS_M2            (10U)         /*!< Inertia identify gain coefficient of MRAS */

#define INERTIA_IDENT_FACTOR_FFLS_M2          (0.9999f)     /*!< Inertia identify forget factor of FFLS */
#define INERTIA_IDENT_PK_INIT_FFLS_M2         (100000U)     /*!< Inertia identify Pk parameter initial value of FFLS, 10^4~10^6 */

#define INERTIA_IDENT_COF_INIT_M2             (500U)        /*!< Inertia identify coefficient initial value, INERTIA_COF_INIT = OPERATION_PERIOD / J */
#define INERTIA_IDENT_OPERATION_PERIOD_M2     (0.01f)       /*!< Inertia identify operation period, unit:s */
#define INERTIA_IDENT_ERROR_THRESHOLD_M2      (4e-6)        /*!< Inertia error threshold, if inertia error less than ERROR_THRESHOLD && judge time high than STEADY_JUDGE_TIME, than output steady Inertia */
#define INERTIA_IDENT_STEADY_TIME_M2          (5.0f / INERTIA_IDENT_OPERATION_PERIOD_M2)    /*!< Inertia error steady judge time, 5.0 means judge time is 5 seconds */
#define INERTIA_IDENT_OLD_TIME_M2             (5.0f / INERTIA_IDENT_OPERATION_PERIOD_M2)    /*!< Last period inertia refresh time, 5.0 means last period inertia refresh time is 5 seconds */

//#define ASR_VARPI_ENABLE_M2                                   /*!< Asr speed loop variable PI enable switch, defined to supported the feature, or commented to disable the feature */

//#define QUICK_DESATURATION_M2                                 /*!< Asr speed loop quick desaturation enable switch, defined to supported the feature, or commented to disable the feature */
#define FEEDBACK_GAIN_M2                       (0.01f)      /*!< Feedback suppression PI regulator feedback gain coefficient */

/*!
* @brief Foc flux weakening control parameters configuration.
*/
//#define FLUX_WEAKENING_M2                                 /*!< Flux weakening function enable switch, defined to supported the feature, or commented to disable the feature */
#define FLUXWEAKE_VOLTAGE_REF_M2               (1000U)      /*!< voltage reference, tenth of a percent */
#define FLUX_WEAKE_KP_M2                       (0.5f)       /*!< Default Kp gain */
#define FLUX_WEAKE_KI_M2                       (0.015f)     /*!< Default Ki gain */
#define IDMIN_M2                               (-0.999f)    /*!< Flux weakening pid input min limit */
#define IDDEMAG_M2                             (-0.5f)      /*!< Demagnetization current */
#define ISMAX_M2                               (0.999f)     /*!< Motor nominal current */
#define MAXMODULE_M2                           (0.95f)      /*!< Circle limitation maximum allowed module */

/*!
* @brief Maximum Torque Per Ampere strategy parameters configuration.
*/
//#define MTPA_ENABLE_M2                     /*!< Max torque per ample feature enable switch, defined to supported the feature, or commented to disable the feature */

#endif  /* end of MOTOR_TWO_TYPE */


/*!
* @brief Debug configuration.
*/
/* Using PWM or SPI to display debug information */
#define DAC_FUNCTIONALITY                                       /*!< DAC debug output enable. If this macro definition is open, DAC debugging is supported */
#define DAC_PWM                             ENABLE              /*!< DAC debug output using PWM module, ENABLE to support and DISABLE to ban this feature */
#define DAC_SPI                             DISABLE             /*!< DAC debug output using SPI module, ENABLE to support and DISABLE to ban this feature */
#define DEBUG_DAC_CH0                       ENABLE              /*!< DAC debug output channel 0, ENABLE to support and DISABLE to ban this feature */
#define DEBUG_DAC_CH1                       ENABLE              /*!< DAC debug output channel 1, ENABLE to support and DISABLE to ban this feature */
#define DEBUG_DAC_CH2                       DISABLE
#define DEBUG_DAC_CH3                       DISABLE

#define DEBUG_DAC_CH0_M2                    DISABLE              /*!< DAC debug output channel 0, ENABLE to support and DISABLE to ban this feature */
#define DEBUG_DAC_CH1_M2                    DISABLE              /*!< DAC debug output channel 1, ENABLE to support and DISABLE to ban this feature */
#define DEBUG_DAC_CH2_M2                    ENABLE
#define DEBUG_DAC_CH3_M2                    ENABLE

#define DUBUG_DAC_CH0_PARAMID               157                 /*!< DAC debug output channel 0 parameter ID */
#define DUBUG_DAC_CH1_PARAMID               158                 /*!< DAC debug output channel 1 parameter ID */

#define DUBUG_DAC_CH2_PARAMID               157                 /*!< DAC debug output channel 0 parameter ID */
#define DUBUG_DAC_CH3_PARAMID               158                 /*!< DAC debug output channel 1 parameter ID */

/* Using GPIO pin to measure code execution time */
#define USE_GPIO_CALCULATE_FOC_RUNTIME


#define TRUE                                (1U)
#define FALSE                               (0U)

/* ===========================================  Typedef  ============================================ */

#endif /* __DRIVE_PARAMETERS_DEFINE_H__ */

/* =============================================  EOF  ============================================== */
