/* Copyright Statement:
*
* This software/firmware and related documentation ("AutoChips Software") are
* protected under relevant copyright laws. The information contained herein is
* confidential and proprietary to AutoChips Inc. and/or its licensors. Without
* the prior written permission of AutoChips inc. and/or its licensors, any
* reproduction, modification, use or disclosure of AutoChips Software, and
* information contained herein, in whole or in part, shall be strictly
* prohibited.
*
* AutoChips Inc. (C) 2020. All rights reserved.
*
* BY OPENING THIS FILE, RECEIVER HEREBY UNEQUIVOCALLY ACKNOWLEDGES AND AGREES
* THAT THE SOFTWARE/FIRMWARE AND ITS DOCUMENTATIONS ("AUTOCHIPS SOFTWARE")
* RECEIVED FROM AUTOCHIPS AND/OR ITS REPRESENTATIVES ARE PROVIDED TO RECEIVER
* ON AN "AS-IS" BASIS ONLY. AUTOCHIPS EXPRESSLY DISCLAIMS ANY AND ALL
* WARRANTIES, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
* WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
* NONINFRINGEMENT. NEITHER DOES AUTOCHIPS PROVIDE ANY WARRANTY WHATSOEVER WITH
* RESPECT TO THE SOFTWARE OF ANY THIRD PARTY WHICH MAY BE USED BY,
* INCORPORATED IN, OR SUPPLIED WITH THE AUTOCHIPS SOFTWARE, AND RECEIVER AGREES
* TO LOOK ONLY TO SUCH THIRD PARTY FOR ANY WARRANTY CLAIM RELATING THERETO.
* RECEIVER EXPRESSLY ACKNOWLEDGES THAT IT IS RECEIVER'S SOLE RESPONSIBILITY TO
* OBTAIN FROM ANY THIRD PARTY ALL PROPER LICENSES CONTAINED IN AUTOCHIPS
* SOFTWARE. AUTOCHIPS SHALL ALSO NOT BE RESPONSIBLE FOR ANY AUTOCHIPS SOFTWARE
* RELEASES MADE TO RECEIVER'S SPECIFICATION OR TO CONFORM TO A PARTICULAR
* STANDARD OR OPEN FORUM. RECEIVER'S SOLE AND EXCLUSIVE REMEDY AND AUTOCHIPS'S
* ENTIRE AND CUMULATIVE LIABILITY WITH RESPECT TO THE AUTOCHIPS SOFTWARE
* RELEASED HEREUNDER WILL BE, AT AUTOCHIPS'S OPTION, TO REVISE OR REPLACE THE
* AUTOCHIPS SOFTWARE AT ISSUE, OR REFUND ANY SOFTWARE LICENSE FEES OR SERVICE
* CHARGE PAID BY RECEIVER TO AUTOCHIPS FOR SUCH AUTOCHIPS SOFTWARE AT ISSUE.
*/

/*!
* @file bldc_parameters_init.c
*
* @brief This file provides common structure parameters initialization.
*
*/

/* ===========================================  Includes  =========================================== */
#include "bldc_parameters_init.h"
#include "bldc_parameters_define.h"
#include "bldc_mcu_config.h"

#if defined(__CC_ARM)
// Suppress warning message: extended constant initialiser used
#pragma diag_suppress 1296
#elif defined(__ICCARM__)
#elif defined(__GNUC__)
#endif


/* ============================================  Define  ============================================ */

/* ===========================================  Typedef  ============================================ */

/* ==========================================  Variables  =========================================== */
#if (defined MOTOR_ONE_TYPE)
/*!
* @brief Motor parameters.
*/
BLDC_MOTOR                 defBldcMotorParam =
{
    .statorRs               = STATOR_RS,                        /*!< Motor stator resistance value */
    .statorLs               = LS,                               /*!< Motor stator inductance value */
    .ld                     = LD,                               /*!< Motor D-axis inductance value */
    .lq                     = LQ,                               /*!< Motor Q-axis inductance value */
    .rotorRs                = ROTOR_RS,                         /*!< Motor rotor resistance value */
    .lm                     = LM,                               /*!< Motor mutual inductance value */
    .lleak                  = 0,                                /*!< Motor leakage inductance value */

    .jm                     = (uint32_t)(INERTIA + 0.5f),       /*!< Motor inertia value */
    .polePairs              = POLE_PAIR_NUM,                    /*!< Motor pole pairs value */
    .encoderLines           = PULSE_PER_REVOLUTION,             /*!< encoder pulse lines per motor mechanical cycle */
    .bEmf                   = VOLTAGE_CONSTANT,                 /*!< Motor back EMF constant  */
    .maxCur                 = MAX_CURRENT,                      /*!< Max stator current value */
    .ratedVoltage           = NORMAL_BUS_VOLTAGE,               /*!< DC bus voltage of inverter system */
    .maxSpeed               = MOTOR_MAX_SPEED_RPM,              /*!< Motor running speed max value */
    .maxFreq                = BASE_FREQ,                        /*!< Motor current frequency max value */

    .maxTorque              = MAX_TORQUE,                       /*!< Motor electromagnetic torque max value */
    .maxPower               = RATE_POWER,                       /*!< Motor rated power */
};

/*!
* @brief BLDC PID regulator configuration parameters.
*/
BLDC_PID_CFG                defBldcPid =
{
    .currentKp              = BLDC_CURRENT_KP_DEFAULT,           /*!< current loop PID kp */
    .currentKi              = BLDC_CURRENT_KI_DEFAULT,           /*!< current loop PID ki */
    .currentKd              = BLDC_CURRENT_KD_DEFAULT,           /*!< current loop PID kd */
    .maxCurrent             = BLDC_CURRENT_MAX_DEFAULT,          /*!< current loop PID max */
    .minCurrent             = BLDC_CURRENT_MIN_DEFAULT,          /*!< current loop PID min */

    .speedKp                = BLDC_SPEED_KP_DEFAULT,             /*!< speed loop PID kp */
    .speedKi                = BLDC_SPEED_KI_DEFAULT,             /*!< speed loop PID ki */
    .speedKd                = BLDC_SPEED_KD_DEFAULT,             /*!< speed loop PID kd */
    .maxSpeed               = BLDC_SPEED_MAX_DEFAULT,            /*!< speed loop PID max */
    .minSpeed               = BLDC_SPEED_MIN_DEFAULT,            /*!< speed loop PID min */

};

/*!
* @brief BLDC control configuration parameters.
*/
BLDC_VARS_CFG                defBldcVars =
{
    .motorDir               = MOTOR_DIR,                        /*!< Motor rotation direction */
    #if (defined BLDC_HALL)
    .initDuty               = MIN_DUTY_WITH_HALL,               /*!< initial hall startup duty */
    .maxDuty                = MAX_DUTY_WITH_HALL,               /*!< max duty */
    .minDuty                = MIN_DUTY_WITH_HALL,               /*!< min duty */
    #elif (defined BLDC_SENSORLESS)
    .initDuty               = MIN_DUTY_PREPOSITION,             /*!< initial sensorless startup duty */
    .maxDuty                = MAX_DUTY_SENSORLESS,              /*!< max duty */
    .minDuty                = MIN_DUTY_SENSORLESS,              /*!< min duty */
    #else
    #endif
    .startupStepDuty        = STARTUP_DUTY_STEP,                /*!< startup step duty */
    .dragDuty               = DRAG_DUTY,                        /*!< drag duty */

    .forceCommutateTime     = FORCE_COMMUTATE_TIME,             /*!< Force change phase time while can not detect zero crossing point */
    .firstDragTime          = FIRST_STEP_DRAG_TIME,             /*!< First drag time after turn in close loop */
    .firstCommutateTime     = FIRST_COMMUTATE_TIME,             /*!< First change phase time after turn in close loop */

    .speedDefaultTarget     = DEFAULT_TARGET_FREQ_START,        /*!< Default speed command */
    .speedStep              = SPEED_CHANGE_STEP,                /*!< Step speed target change value */
    .speedMaxLimit          = MAX_SPEED_VALUE,                  /*!< Max limit of speed command */
    .speedMinLimit          = MIN_SPEED_VALUE,                  /*!< Min limit of speed command */
    .motorAcceleration      = ACCELERATION_RPS,                 /*!< Acceleration of motor speed up */
    .motorDeceleration      = DECELERATION_RPS,                 /*!< Deceleration of motor speed reduction */

    .pwmId                  = PWM_INDEX(PWM_MODULE),            /*!< Motor pwm ID */
    .pwmChannelU            = PWM_U_CHANNEL,                    /*!< PWM channnel used by phase U */
    .pwmChannelV            = PWM_V_CHANNEL,                    /*!< PWM channnel used by phase V */
    .pwmChannelW            = PWM_W_CHANNEL,                    /*!< PWM channnel used by phase W */
    .pwmChannelT            = PWM_T_CHANNEL,                    /*!< PWM channnel used by Others */

    .baseFreq               = BASE_FREQ,                        /*!< Motor frequency reference base value */

    .msUnitInt              = MS_UINT_INT,                      /*!< Millisecond unit */
    .halfPwmPeriod          = (PWM_PERIOD_VALUE >> 1),          /*!< PWM half period */
    .pwmPeriod              = PWM_PERIOD_VALUE,                 /*!< PWM period */
    .pwmDeadTime            = PWM_DEAD_TIME,                    /*!< PWM dead time */

    .bemfNeutralCoef        = BEMF_NEUTRAL_THRESHOLD_COEFF,     /*!< Bemf neutral point threshold coefficient */

    .sysBusClk              = MOTOR_BUS_CLK,                    /*!< MCU system bus clock */
    .pwdtPrescal            = PWDT_CLK_PSC,                     /*!< PWDT clock prescaler */
};

/*!
* @brief BLDC sensorless startup Bemf configuration parameters.
*/
BLDC_BEMF_CFG               defBemfVars =
{
    .prepositionDutyMin     = MIN_DUTY_PREPOSITION,              /*!< Min duty while preposition in BLDC sensorless startup */
    .prepositionDutyMax     = MAX_DUTY_PREPOSITION,              /*!< Max duty while preposition in BLDC sensorless startup */
    .prepositionDutyStep    = STEP_DUTY_PREPOSITION,             /*!< Step duty once while preposition in BLDC sensorless startup */
    .prepositionTimeMin     = MIN_PREPOSITION_DELAY,             /*!< Min time while preposition in BLDC sensorless startup(unit:ms) */
    .prepositionTimeMax     = INIT_PREPOSITION_DELAY,            /*!< Max time while preposition in BLDC sensorless startup(unit:ms) */
    .prepositionTimeStep    = INIT_PREPOSITION_DELAY_STEP,       /*!< Step time once while preposition in BLDC sensorless startup(unit:ms) */
    .kickDutyMax            = MAX_DUTY_SENSORLESS_START,         /*!< Max duty while open kick in BLDC sensorless startup */
    .kickDutyStep           = STARTUP_DUTY_STEP,                /*!< Step duty once while open kick in BLDC sensorless startup */
    .kickTimeMin            = BLDC_SENSORLESS_MIN_DELAY_TIME,    /*!< Min time while open kick in BLDC sensorless startup(unit:ms) */
    .kickTimeMax            = BLDC_SENSORLESS_INIT_DELAY_TIME,   /*!< Max time while open kick in BLDC sensorless startup(unit:ms) */
    .kickTimeStep           = BLDC_SENSORLESS_STEP_DELAY_TIME,   /*!< Step time once while open kick in BLDC sensorless startup(unit:ms) */
};
#endif

#if (defined MOTOR_TWO_TYPE)
/*!
* @brief Motor2 parameters.
*/
BLDC_MOTOR                 defBldcMotorParam_M2 =
{
    .statorRs               = STATOR_RS_M2,                        /*!< Motor stator resistance value */
    .statorLs               = LS_M2,                               /*!< Motor stator inductance value */
    .ld                     = LD_M2,                               /*!< Motor D-axis inductance value */
    .lq                     = LQ_M2,                               /*!< Motor Q-axis inductance value */
    .rotorRs                = ROTOR_RS_M2,                         /*!< Motor rotor resistance value */
    .lm                     = LM_M2,                               /*!< Motor mutual inductance value */
    .lleak                  = 0,                                /*!< Motor leakage inductance value */

    .jm                     = (uint32_t)(INERTIA_M2 + 0.5f),       /*!< Motor inertia value */
    .polePairs              = POLE_PAIR_NUM_M2,                    /*!< Motor pole pairs value */
    .encoderLines           = PULSE_PER_REVOLUTION_M2,             /*!< encoder pulse lines per motor mechanical cycle */
    .bEmf                   = VOLTAGE_CONSTANT_M2,                 /*!< Motor back EMF constant  */
    .maxCur                 = MAX_CURRENT_M2,                      /*!< Max stator current value */
    .ratedVoltage           = NORMAL_BUS_VOLTAGE,               /*!< DC bus voltage of inverter system */
    .maxSpeed               = MOTOR_MAX_SPEED_RPM_M2,              /*!< Motor running speed max value */
    .maxFreq                = BASE_FREQ_M2,                        /*!< Motor current frequency max value */

    .maxTorque              = MAX_TORQUE_M2,                       /*!< Motor electromagnetic torque max value */
    .maxPower               = RATE_POWER_M2,                       /*!< Motor rated power */
};

/*!
* @brief BLDC motor2 PID regulator configuration parameters.
*/
BLDC_PID_CFG                defBldcPid_M2 =
{
    .currentKp              = BLDC_CURRENT_KP_DEFAULT_M2,           /*!< current loop PID kp */
    .currentKi              = BLDC_CURRENT_KI_DEFAULT_M2,           /*!< current loop PID ki */
    .currentKd              = BLDC_CURRENT_KD_DEFAULT_M2,           /*!< current loop PID kd */
    .maxCurrent             = BLDC_CURRENT_MAX_DEFAULT_M2,          /*!< current loop PID max */
    .minCurrent             = BLDC_CURRENT_MIN_DEFAULT_M2,          /*!< current loop PID min */

    .speedKp                = BLDC_SPEED_KP_DEFAULT_M2,             /*!< speed loop PID kp */
    .speedKi                = BLDC_SPEED_KI_DEFAULT_M2,             /*!< speed loop PID ki */
    .speedKd                = BLDC_SPEED_KD_DEFAULT_M2,             /*!< speed loop PID kd */
    .maxSpeed               = BLDC_SPEED_MAX_DEFAULT_M2,            /*!< speed loop PID max */
    .minSpeed               = BLDC_SPEED_MIN_DEFAULT_M2,            /*!< speed loop PID min */
};

/*!
* @brief BLDC motor2 control configuration parameters.
*/
BLDC_VARS_CFG                defBldcVars_M2 =
{
    .motorDir               = MOTOR_DIR_M2,                     /*!< Motor rotation direction */
    #if (defined BLDC_HALL_M2)
    .initDuty               = MIN_DUTY_WITH_HALL_M2,            /*!< initial hall startup duty */
    .maxDuty                = MAX_DUTY_WITH_HALL_M2,            /*!< max duty */
    .minDuty                = MIN_DUTY_WITH_HALL_M2,            /*!< min duty */
    #elif (defined BLDC_SENSORLESS_M2)
    .initDuty               = MIN_DUTY_PREPOSITION_M2,          /*!< initial sensorless startup duty */
    .maxDuty                = MAX_DUTY_SENSORLESS_M2,           /*!< max duty */
    .minDuty                = MIN_DUTY_SENSORLESS_M2,           /*!< min duty */
    #else
    #endif
    .startupStepDuty        = STARTUP_DUTY_STEP_M2,             /*!< startup step duty */
    .dragDuty               = DRAG_DUTY_M2,                     /*!< drag duty */

    .forceCommutateTime     = FORCE_COMMUTATE_TIME_M2,          /*!< Force change phase time while can not detect zero crossing point */
    .firstDragTime          = FIRST_STEP_DRAG_TIME_M2,          /*!< First drag time after turn in close loop */
    .firstCommutateTime     = FIRST_COMMUTATE_TIME_M2,          /*!< First change phase time after turn in close loop */

    .speedDefaultTarget     = DEFAULT_TARGET_FREQ_START_M2,     /*!< Default speed command */
    .speedStep              = SPEED_CHANGE_STEP_M2,             /*!< Step speed target change value */
    .speedMaxLimit          = MAX_SPEED_VALUE_M2,               /*!< Max limit of speed command */
    .speedMinLimit          = MIN_SPEED_VALUE_M2,               /*!< Min limit of speed command */
    .motorAcceleration      = ACCELERATION_RPS_M2,              /*!< Acceleration of motor speed up */
    .motorDeceleration      = DECELERATION_RPS_M2,              /*!< Deceleration of motor speed reduction */

    .pwmId                  = PWM_INDEX(PWM_MODULE_M2),         /*!< Motor pwm ID */
    .pwmChannelU            = PWM_U_CHANNEL_M2,                 /*!< PWM channnel used by phase U */
    .pwmChannelV            = PWM_V_CHANNEL_M2,                 /*!< PWM channnel used by phase V */
    .pwmChannelW            = PWM_W_CHANNEL_M2,                 /*!< PWM channnel used by phase W */
    .pwmChannelT            = PWM_T_CHANNEL_M2,                 /*!< PWM channnel used by Others */

    .baseFreq               = BASE_FREQ_M2,                     /*!< Motor frequency reference base value */

    .msUnitInt              = MS_UINT_INT,                      /*!< Millisecond unit */
    .halfPwmPeriod          = (PWM_PERIOD_VALUE >> 1),          /*!< PWM half period */
    .pwmPeriod              = PWM_PERIOD_VALUE,                 /*!< PWM period */
    .pwmDeadTime            = PWM_DEAD_TIME,                    /*!< PWM dead time */

    .bemfNeutralCoef        = BEMF_NEUTRAL_THRESHOLD_COEFF,     /*!< Bemf neutral point threshold coefficient */

    .sysBusClk              = MOTOR_BUS_CLK,                    /*!< MCU system bus clock */
    .pwdtPrescal            = PWDT_CLK_PSC_M2,                  /*!< PWDT clock prescaler */
};

/*!
* @brief BLDC motor2 sensorless startup Bemf configuration parameters.
*/
BLDC_BEMF_CFG               defBemfVars_M2 =
{
    .prepositionDutyMin     = MIN_DUTY_PREPOSITION_M2,           /*!< Min duty while preposition in BLDC sensorless startup */
    .prepositionDutyMax     = MAX_DUTY_PREPOSITION_M2,           /*!< Max duty while preposition in BLDC sensorless startup */
    .prepositionDutyStep    = STEP_DUTY_PREPOSITION_M2,          /*!< Step duty once while preposition in BLDC sensorless startup */
    .prepositionTimeMin     = MIN_PREPOSITION_DELAY_M2,          /*!< Min time while preposition in BLDC sensorless startup(unit:ms) */
    .prepositionTimeMax     = INIT_PREPOSITION_DELAY_M2,         /*!< Max time while preposition in BLDC sensorless startup(unit:ms) */
    .prepositionTimeStep    = INIT_PREPOSITION_DELAY_STEP_M2,    /*!< Step time once while preposition in BLDC sensorless startup(unit:ms) */
    .kickDutyMax            = MAX_DUTY_SENSORLESS_START_M2,      /*!< Max duty while open kick in BLDC sensorless startup */
    .kickDutyStep           = STARTUP_DUTY_STEP_M2,              /*!< Step duty once while open kick in BLDC sensorless startup */
    .kickTimeMin            = BLDC_SENSORLESS_MIN_DELAY_TIME_M2, /*!< Min time while open kick in BLDC sensorless startup(unit:ms) */
    .kickTimeMax            = BLDC_SENSORLESS_INIT_DELAY_TIME_M2,/*!< Max time while open kick in BLDC sensorless startup(unit:ms) */
    .kickTimeStep           = BLDC_SENSORLESS_STEP_DELAY_TIME_M2,/*!< Step time once while open kick in BLDC sensorless startup(unit:ms) */
};

#endif

/*!
* @brief System hardware parameters.
*/
BLDC_HW                    defBldcHwBoard =
{
    .pwmPolarity            = PHASE_UVW_POLARITY,               /*!< Three phase polarity of inverter */
    .hwDeadTime             = HW_DEAD_TIME_NS,                  /*!< Hardware dead time of power device in inverter */
    .swDeadTime             = SW_DEAD_TIME_NS,                  /*!< Configurable software dead time of power device in inverter */
    .maxVoltage             = MAX_BUS_VOLTAGE,                  /*!< Max DC Bus voltage */
    .normalVoltage          = NORMAL_BUS_VOLTAGE,               /*!< Normal DC Bus voltage */
    .minVoltage             = MIN_BUS_VOLTAGE,                  /*!< Min DC Bus voltage */
    .vbusAttenuateVal       = VBUS_ATTENUATE_FACTOR,            /*!< Voltage sampling circuit attenuation factor */

    .maxCurrent             = HW_MAX_CURRENT,                   /*!< Inverter hardware max current */
    .rShunt                 = RSHUNT,                           /*!< Sampling resistance value */
    .opGainVal              = OP_AMPLIFICATION_GAIN,            /*!< Operation amplifier gain in current sampling circuit */
    .icsGainVal             = ICS_GAIN,                         /*!< Current sensor gain */
    .adcRange               = ADC_RANGE,                        /*!< ADC range */
    .adcRefV                = VSVREF,                           /*!< ADC reference voltage */
    .overCurrentPolarity    = (uint16_t)OVERCUR_FEEDBACK_POLARITY,  /*!< Hardware over current fault polarity */
};

/*!
* @brief Eflash access configuration parameters.
*/
DATA_ACCESS_FLASH_CFG       defEflashAccessCfg =
{
#if (defined DATA_ACCESS_FLASH_ENABLE)
    .accessEnable          = ENABLE,                            /*!< Enable Eflash access */
#else
    .accessEnable          = DISABLE,                           /*!< Disable Eflash access */
#endif
};

/* ====================================  Functions declaration  ===================================== */

/* ======================================  Functions define  ======================================== */

/* =============================================  EOF  ============================================== */
