/* Copyright Statement:
*
* This software/firmware and related documentation ("AutoChips Software") are
* protected under relevant copyright laws. The information contained herein is
* confidential and proprietary to AutoChips Inc. and/or its licensors. Without
* the prior written permission of AutoChips inc. and/or its licensors, any
* reproduction, modification, use or disclosure of AutoChips Software, and
* information contained herein, in whole or in part, shall be strictly
* prohibited.
*
* AutoChips Inc. (C) 2020. All rights reserved.
*
* BY OPENING THIS FILE, RECEIVER HEREBY UNEQUIVOCALLY ACKNOWLEDGES AND AGREES
* THAT THE SOFTWARE/FIRMWARE AND ITS DOCUMENTATIONS ("AUTOCHIPS SOFTWARE")
* RECEIVED FROM AUTOCHIPS AND/OR ITS REPRESENTATIVES ARE PROVIDED TO RECEIVER
* ON AN "AS-IS" BASIS ONLY. AUTOCHIPS EXPRESSLY DISCLAIMS ANY AND ALL
* WARRANTIES, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
* WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
* NONINFRINGEMENT. NEITHER DOES AUTOCHIPS PROVIDE ANY WARRANTY WHATSOEVER WITH
* RESPECT TO THE SOFTWARE OF ANY THIRD PARTY WHICH MAY BE USED BY,
* INCORPORATED IN, OR SUPPLIED WITH THE AUTOCHIPS SOFTWARE, AND RECEIVER AGREES
* TO LOOK ONLY TO SUCH THIRD PARTY FOR ANY WARRANTY CLAIM RELATING THERETO.
* RECEIVER EXPRESSLY ACKNOWLEDGES THAT IT IS RECEIVER'S SOLE RESPONSIBILITY TO
* OBTAIN FROM ANY THIRD PARTY ALL PROPER LICENSES CONTAINED IN AUTOCHIPS
* SOFTWARE. AUTOCHIPS SHALL ALSO NOT BE RESPONSIBLE FOR ANY AUTOCHIPS SOFTWARE
* RELEASES MADE TO RECEIVER'S SPECIFICATION OR TO CONFORM TO A PARTICULAR
* STANDARD OR OPEN FORUM. RECEIVER'S SOLE AND EXCLUSIVE REMEDY AND AUTOCHIPS'S
* ENTIRE AND CUMULATIVE LIABILITY WITH RESPECT TO THE AUTOCHIPS SOFTWARE
* RELEASED HEREUNDER WILL BE, AT AUTOCHIPS'S OPTION, TO REVISE OR REPLACE THE
* AUTOCHIPS SOFTWARE AT ISSUE, OR REFUND ANY SOFTWARE LICENSE FEES OR SERVICE
* CHARGE PAID BY RECEIVER TO AUTOCHIPS FOR SUCH AUTOCHIPS SOFTWARE AT ISSUE.
*/


#ifndef __GLOBAL_VAR_H__
#define __GLOBAL_VAR_H__
/*!
* @file globalvar.h
*
* @brief This file provides the declaration of global variables.
*
*/

/* ===========================================  Includes  =========================================== */
#include "parameters_config.h"
#include "maxtorque_per_ampere.h"
#include "protector.h"
#include "foc_data_access_eflash.h"
#include "param_identify_api.h"

/* ============================================  Define  ============================================ */

/* ===========================================  Typedef  ============================================ */

/* ==========================================  Variables  =========================================== */
extern uint8_t g_hallSelfLearnFinishFlag;                   /*!< Hall sensor self learn finish flag */
extern uint8_t g_hallSelfLearnFinishFlag_M2;                /*!< Hall sensor self learn finish flag */

extern HW_BOARD                     *pHWBoardParam;         /*!< Hardware parameters of motor controller */

/*!
* Global variables of Motor1
*/
#if (defined MOTOR_ONE_TYPE)
/*!
* ADC sampling
*/
extern ADC_SAMPLE                   g_adcSampleReal;        /*!< Phase Sample Current and Voltage ADC Result */
extern ADC_SAMPLE_COF               g_adcSampleCof;         /*!< Voltage and current sampling factor */
extern ADC_TRANSFORM                g_adcTransform;         /*!< Phase Sample Current and Voltage transform to Q format */
extern ADC_PARAM                    g_adcParam;

/*!
* Motor protector
*/
extern PROTECTOR                    g_protector;            /*!< Protection related structure */

/*!
* FOC
*/
extern STC_PARAM                    g_STC;                  /*!< Speed and torque control parameters*/

/*!
* Motor parameters
*/
extern MOTOR_PARAM                  *pMotorParam;           /*!< Motor parameters measurement*/
extern MOTOR_PARAM_PU               g_motorParamPu;         /*!< Motor parameters Per Unit */

extern FOC_VARS_CTRL                g_focVarsCtrl;          /*!< FOC control parameters structure */
extern FOC_VARS                     g_focVars;              /*!< FOC parameters structure */

extern ACR_CURPI_COFIN              g_acrCurIdCofIn;        /*!< Current loop d axis PID parameter input structure */
extern ACR_CURPI_COFIN              g_acrCurIqCofIn;        /*!< Current loop q axis PID parameter input structure */

extern ACR_UDQDCP_COF               g_udqDcpCof;            /*!< Feedforward decoupled parameter input structure */
extern ACR_UDQDCP_OUT               g_udqDcpOut;            /*!< Feedforward decoupled output structure */
extern ACR_UDQDCP_IN                g_udqDcpIn;             /*!< Feedforward decoupled intput structure */

extern TRQ_LOAD_EST                 g_loadTorqueEst;        /*!< Load torque estimate structure */

extern PWM_GEN_OUT                  g_pwmValue;             /*!< Pwm generate output structure */

extern CUR_RESTRUCT_DATA            g_curRestruct;          /*!< Current restruct function data */
extern PWM_SHIFT_DATA               g_pwmShift;             /*!< Pwm shift sub-function data */

extern ASR_SPEEDPI_COFIN            g_asrSpeedIqCofIn;      /*!< Speed loop PID parameter input structure */

extern SMC_ESTIMATOR                g_smcEstimator;         /*!< Sliding mode control structure */
extern THETA_COMPENSATOR            g_thetaComp;            /*!< Angle compensation structure */
extern SM_OBSERVER                  g_smo;

extern FLUX_OBSERVER_CTRL           g_fluxObserverCtrl;     /*!< Flux Observer control structure */
extern FLUX_OBSERVER                g_fluxObserver;         /*!< Flux Observer control and configuration structure */
extern MRAS_OBSERVER                g_mrasObserver;         /*!< Mras Observer structure */
extern MRAS                         g_mras;

extern HALL_SENSOR                  g_hall;                 /*!< Hall sensor assembly struct */
extern HALL_SENSOR_DATA             g_hallSensor;           /*!< Hall sensor calculate rotor position structure */
extern HALL_LEARN_DATA              g_hallSelfLearn;        /*!< Hall sensor self learn data */

#if (defined ENCODER_SENSORS)
extern ENCODER_TYPE                 g_encoderSensor;        /*!< Encoder sensor structure in ENCODER FOC mode */
#if (defined MT_SPEED_MEASURE)
extern ENCODER_SPEED_MEASURE        g_encoderSpeed;         /*!< Encoder speed measure structure in ENCODER FOC mode */
#endif
#endif

#if (defined ENCODER_HALL_SENSORS)
extern ENCODER_TYPE                 g_encoderSensor;        /*!< Encoder sensor structure in ENCODER FOC mode */
extern ENCODER_HALL_SENSOR          g_encoderHallSensor;    /*!< Encoder_Hall sensor data struct */
#endif

#if (defined PULSE_INJECTION)
extern PULSE_INJECT_DATA            g_pulseInject;          /*!< Initial position data struct */
#endif

#if (defined INIT_POS_DETEC)
extern INIT_POSITION_POLARITY_CHECK g_initPosPolar;         /*!< Initial position check with rotation voltage injection polarity check data struct */
extern INITPOS_DETEC_STRUCT         g_initPosDetecStruct;   /*!< Initial position check with rotation voltage injection struct */
extern INITPOS_DETEC_PARAM          g_initPosDetecParam;    /*!< Initial position check with rotation voltage injection parameters struct */
extern FFT_PARAM                    g_initPosDetecFft;      /*!< Initial position check with rotation voltage injection FFT data struct */
#endif

extern FLUXWEAK_PARAM               g_fluxWeak;             /*!< fluxWeak parameter structure */
extern MTPA_THETA_COMPENSATOR       g_thetaMtpaComp;        /*!< MTPA Angle compensation structure */

extern HFI_INIT_POS_DETECT          g_initPosDet;           /*!< Hfi Initial position detect structure */
extern HFI_BPF_COEFF                g_bpfCoeff;             /*!< Hfi Butterworth BPF structure */
extern HFI_TWO_ORDER_LPF            g_lpfCoeff;             /*!< Hfi Two Order Butterworth LPF structure */
extern HFI_TYPE                     g_hfinj;                /*!< Hfi related structure */
extern HFI_CTRL                     g_hfiCtrl;              /*!< Hfi control structure, including INIT_POS_DETECT,BPF_COEFF,TWO_ORDER_LPF and HIGH_FREQ_INJ_TYPE */

extern TORQUE_COMMAND               g_torqueCommand;        /*!< Torque command parameters structure */

/*!
* Speed control
*/
extern SPEED_RAMP_HANDLE            g_speedCommand;         /*!< Speed command ramp structure */

extern SCURVE_DATA                  g_sCurveData;           /*!< S curve speed ramp structure */
extern SCURVE                       g_sCurve;               /*!< S curve speed ramp structure */

extern ASR_SPEEDVARPI               g_asrSpeedVarPI;        /*!< Speed loop variable PI parameters structure */

extern ESTIMATED_SPEED              g_rotationalSpeed;      /*!< Motor speed calculation structure */

/*!
* Position control
*/
extern POSITION_RAMP_HANDLE         g_positionCommand;      /*!< Position command structure */

/*!
* Open loop start up
*/
extern STARTUP_CTRL                 g_startUpCtrl;          /*!< Startup control parameters structure */
extern START_UP                     g_startUp;              /*!< Startup structure */
extern CURVE_REFVALUE               g_startUpCurveSeg1;     /*!< Open loop startup curve segment 1 parameters */
extern CURVE_REFVALUE               g_startUpCurveSeg2;     /*!< Open loop startup curve segment 2 parameters */
extern CURVE_REFVALUE               g_startUpCurveSeg3;     /*!< Open loop startup curve segment 3 parameters */
extern CURVE_REFVALUE               g_startUpCurveSeg4;     /*!< Open loop startup curve segment 4 parameters */
extern CURVE_REFVALUE               g_startUpCurveSeg5;     /*!< Open loop startup curve segment 5 parameters */
extern CURVE_IN                     g_startUpCurveIn;       /*!< Open loop startup curve input parameters */
extern CURVE_OUT                    g_startUpCurveOut;      /*!< Open loop startup curve output parameters */
extern CURVE_TIME_DURATION          g_startUpCurveTime;     /*!< Open loop startup curve time parameters structure */
extern STARTUP_CURVE                g_startUpCurve;         /*!< Open loop startup curve structure */

/*!
* Feature functions
*/
extern SHORT_BRAKE_DATA             g_shortBrake;           /*!< Short brake function data struct */
extern ELEC_BRAKE_DATA              g_elecBrake;            /*!< electric brake function data struct */

extern SHORT_BRAKE                  g_sBrake;               /*!< Short brake structure */
extern ELEC_BRAKE                   g_eBrake;               /*!< Electric brake structure */

extern PARAM_IDENTIFY_STRUCT        g_paramIdentify;        /*!< Parameter identify struct */
extern PARAM_IDENTIFY_MAIN_CTRL     g_paramIdentifyCtrl;    /*!< Identify motor parameter data struct */
extern PARAM_RS_IDENTIFY_DATA       g_paramRsIdentify;      /*!< Identify Rs data struct */
extern PARAM_LDQ_IDENTIFY_DATA      g_paramLdqIdentify;     /*!< Identify Ld Lq data struct */
extern PARAM_FLUX_IDENTIFY_DATA     g_paramFluxIdentify;    /*!< Identify flux data struct*/
extern PARAM_INERTIA_IDENTIFY_DATA  g_paramInertiaIdentify; /*!< Identify inertia data struct */

extern ACIM_PARAM_IDENTIFY          g_acimParamId;          /*!< Acim parameters identification data struct */
extern SIGNAL_PHASE_SHIFT           g_sigPhaseShift;        /*<! Signal phase shifter data struct */

extern PMSM_LDQID_PARAM             g_pmsmLdqId;            /*!< PMSM DQ inductance identification struct */
extern PMSM_LDQID_SWDATA            g_pmsmLdqSwParam;       /*!< PMSM DQ inductance identification data struct */
extern FFT_PARAM                    g_fftParam;             /*!< PMSM DQ inductance identification FFT data struct */

extern INERTIA_IDENTIFY_STRUCT      g_inertiaIdentify;      /*!< Identify inertia data struct */

extern OVER_VOLTAGE_SUPPRESS        g_overVoltSupp;         /*!< Bus over voltage suppress data */

extern POWER_EST                    g_powerEst;             /*!< Power estimate structure */
extern DCCUR_EST                    g_dcCurEst;             /*!< Dc current estimate structure */

extern DEAD_BAND_COMP_STRUCT        g_deadCompCtrl;         /*!< Dead band compensation data stauct */

extern ENERGY_SAVE                  g_energySave;           /*!< Enegy save function data */

extern INITSPEED_JUDGE              g_initSpeedJudge;       /*!< Speed judge function data */

/*!
* Induction motor FOC
*/
//#if (defined FOC_ACIM)
//extern ACIM_FOC_CTRL                g_acimFocCtrl;          /*!< Induction motor FOC control structure */
//extern ACIM_FLUX_PARAM              g_acimFlux;             /*!< Induction motor FOC control flux parameters structure */
//extern ACIM_SPEED_PARAM             g_acimSpeed;            /*!< Induction motor FOC control speed parameters structure */
//extern ACIM_CURRENT_PARAM           g_acimCur;              /*!< Induction motor FOC control current parameters structure */
//extern ACIM_VOLTAGE_PARAM           g_acimVolt;             /*!< Induction motor FOC control voltage parameters structure */
//extern ACIM_MOTOR_PARAM             g_aciMotor;             /*!< Induction motor FOC control motor parameters structure */
//#endif

/*!
* PID
*/
/* Acr_d */
extern PID_REGULATOR_IN             g_acrCurIdPIIn;         /*!< Current loop d axis PID input structure */
extern PID_REGULATOR_COF            g_acrCurIdCof;          /*!< Current loop d axis PID parameter structure */
extern PID_REGULATOR_OUT            g_acrCurIdPIOut;        /*!< Current loop d axis PID output structure */
/* Acr_q */
extern PID_REGULATOR_IN             g_acrCurIqPIIn;         /*!< Current loop q axis PID input structure */
extern PID_REGULATOR_COF            g_acrCurIqCof;          /*!< Current loop q axis PID parameter structure */
extern PID_REGULATOR_OUT            g_acrCurIqPIOut;        /*!< Current loop q axis PID output structure */
/* Asr */
extern PID_REGULATOR_IN             g_asrSpeedPIIn;         /*!< Speed loop PID input structure */
extern PID_REGULATOR_COF            g_asrSpeedCof;          /*!< Speed loop PID parameter structure */
extern PID_REGULATOR_OUT            g_asrSpeedPIOut;        /*!< Speed loop PID output structure */

/* Apr */
extern PID_REGULATOR_IN             g_aprPositionPIIn;      /*!< Position loop PID input structure */
extern PID_REGULATOR_COF            g_aprPositionCof;       /*!< Position loop PID parameter structure */
extern PID_REGULATOR_OUT            g_aprPositionPIOut;     /*!< Position loop PID output structure */

/* Speed estimated */
extern PID_REGULATOR_IN             g_asrSpeedEstPIIn;      /*!< PLL method speed loop estimation PID input structure */
extern PID_REGULATOR_COF            g_asrSpeedEstCof;       /*!< Speed est pll PID parameter structure */
extern PID_REGULATOR_OUT            g_asrSpeedEstPIOut;     /*!< PLL method speed loop estimation PID output structure */
/* flux weakening */
extern PID_REGULATOR_IN             g_fluxWeakPIIn;         /*!< fluxWeak PID input structure */
extern PID_REGULATOR_COF            g_fluxWeakPICof;        /*!< fluxWeak PID parameter structure */
extern PID_REGULATOR_OUT            g_fluxWeakPIOut;        /*!< fluxWeak PID output structure */

/* Hfi */
extern PID_REGULATOR_IN             g_asrHfiEstPIIn;        /*!< PLL method Hfi estimation PID input structure */
extern PID_REGULATOR_COF            g_asrHfiEstCof;         /*!< PLL method Hfi estimation PID parameter structure */
extern PID_REGULATOR_OUT            g_asrHfiEstPIOut;       /*!< PLL method Hfi estimation PID output structure */

/* Udc suppress */
extern PID_REGULATOR_IN             g_udcLimitPIIn;         /*!< Bus over voltage suppress PID input structure */
extern PID_REGULATOR_OUT            g_udcLimitPIOut;        /*!< Bus over voltage suppress PID output structure */
extern PID_REGULATOR_COF            g_udcLimitPICof;        /*!< Bus over voltage suppress PID parameter structure */

/* Rs identification ACR */
extern PID_REGULATOR_IN             g_acrAlfaRsIdPIIn;      /*!< Rs identification ACR alpha axis component PID input structure */
extern PID_REGULATOR_OUT            g_acrAlfaRsIdPIOut;     /*!< Rs identification ACR alpha axis component PID output structure */
extern PID_REGULATOR_COF            g_acrAlfaRsIdPICof;     /*!< Rs identification ACR alpha axis component PID parameter structure */

extern PID_REGULATOR_IN             g_acrBetaRsIdPIIn;      /*!< Rs identification ACR beta axis component PID input structure */
extern PID_REGULATOR_OUT            g_acrBetaRsIdPIOut;     /*!< Rs identification ACR beta axis component PID output structure */
extern PID_REGULATOR_COF            g_acrBetaRsIdPICof;     /*!< Rs identification ACR beta axis component PID parameter structure */

/* PID structure */
extern PID_REGULATOR                g_pidAcrD;              /*!< D_asxis current loop (flux) PID regulator structure */
extern PID_REGULATOR                g_pidAcrQ;              /*!< Q_asxis current loop (torque) PID regulator structure */
extern PID_REGULATOR                g_pidAsr;               /*!< Speed loop PID regulator structure */

extern PID_REGULATOR                g_pidApr;               /*!< Position loop PID regulator structure */

extern PID_REGULATOR                g_pidPll;               /*!< PLL speed estimated */
extern PID_REGULATOR                g_pidFw;                /*!< Flux weakening */

extern PID_REGULATOR                g_pidHfi;               /*!< High frequency injection */

extern PID_REGULATOR                g_pidUdcLimit;          /*!< Bus over voltage suppress */

extern PID_REGULATOR                g_pidAcrAlfaRsId;       /*!< Alpha_asxis current loop for Rs identification PID regulator structure */
extern PID_REGULATOR                g_pidAcrBetaRsId;       /*!< Beta_asxis current loop for Rs identification PID regulator structure */

extern PARAM_IDENTIFY_BUFFER        g_paramIdentifyBuffer;  /*!< Buffer structure of parameter identification */
extern HALL_SELFLEARN_BUFFER        g_hallSelfLearnBuffer;  /*!< Buffer structure of hall sensor self learn result */

/*!
* Configuration
*/
extern FOC_VARS_CFG                 *pFocVarsCfg;           /*!< FOC configuration parameters */
extern TRANSFORM_CFG                *pTransformCfg;         /*!< ADC sampling transform configuration parameters */
extern FLUX_OBSERVER_CFG            *pFluxObserveCfg;       /*!< Flux observer configuration parameters */
extern PLL_CFG                      *pPllCfg;               /*!< PLL speed estimated configuration parameters */
extern SMC_CFG                      *pSmoObserveCfg;        /*!< Sliding mode configuration parameters*/
extern MRAS_OBSERVER_CFG            *pMrasObserveCfg;       /*!< Mras Observer configuration parameters */

extern CUR_RESTRUCT_CFG             *pCurRestructCfg;       /*!< Current resonstruction configuration parameters */
extern SVPWM_CFG                    *pSvModulationCfg;      /*!< SVPWM configuration parameters */
extern DEADCOMP_CFG                 *pDeadCompCfg;          /*!< SVPWM dead time compensation configuration parameters */

extern HFI_CFG                      *pHighFreqInjCfg;       /*!< High frequency injection configuration parameters */
extern FLUXWEAK_CFG                 *pFluxWeakCfg;          /*!< Flux weakening configuration parameters */

extern START_UP_CFG                 *pStartUpCfg;           /*!< FOC start up configuration parameters */
extern PULSE_INJ_CFG                *pPulseInjCfg;          /*!< FOC pulse injection configuration parameters */

#if (defined INIT_POS_DETEC)
extern INITPOS_DETEC_CFG            *pInitPosDetecCfg;      /*!< Rotation voltage injection initial position check configuration parameters */
#endif

extern SHORT_BRAKE_CFG              *pShortBrakeCfg;        /*!< Short brake configuration parameters */
extern ENERGY_SAVE_CFG              *pEnergySaveCfg;        /*!< Energy save configuration parameters */
extern ELECBRAKE_CFG                *pElecBrakeCfg;         /*!< Electromagnetic brake configuration parameters */

extern POWEREST_CFG                 *pPowerEstCfg;          /*!< Power estimated configuration parameters */

extern PID_CFG                      *pPidCfg;               /*!< PID regulator configuration parameters */

extern LOAD_ESTIMATE_CFG            *pLoadTorqueCfg;        /*!< Load torque estimate configuration parameters */

extern INERTIA_IDENTIFY_CFG         *pInertiaIdentCfg;      /*!< Inertia identify configuration parameters */

extern SCURVE_CFG                   *pSCurveCfg;            /*!< Speed S-Curve configuration parameters */

#if (defined DATA_ACCESS_FLASH_ENABLE)
extern DATA_ACCESS_FLASH           g_dataAccessFlash;                      /*!< Variables access flash operation parameters structure */
extern DATA_ACCESS_FLASH_CFG        *pEflashCfg;            /*!< Eflash access configuration parameters */
#endif

#endif  /* end of MOTOR_ONE_TYPE */

/*!
* Global variables of Motor2
*/
#if (defined MOTOR_TWO_TYPE)

extern ADC_SAMPLE                   g_adcSampleReal_M2;        /*!< Phase Sample Current and Voltage ADC Result */
extern ADC_SAMPLE_COF               g_adcSampleCof_M2;         /*!< Voltage and current sampling factor */
extern ADC_TRANSFORM                g_adcTransform_M2;         /*!< Phase Sample Current and Voltage transform to Q format */
extern ADC_PARAM                    g_adcParam_M2;

/*!
* Motor protector
*/
extern PROTECTOR                    g_protector_M2;            /*!< Protection related structure */

/*!
* FOC
*/
extern STC_PARAM                    g_STC_M2;                  /*!< Speed and torque control parameters*/

extern MOTOR_PARAM                  *pMotorParam_M2;                           /*!< Motor parameters measurement*/
extern MOTOR_PARAM_PU               g_motorParamPu_M2;                         /*!< Motor parameters Per Unit */

extern FOC_VARS_CTRL                g_focVarsCtrl_M2;          /*!< FOC control parameters structure */
extern FOC_VARS                     g_focVars_M2;              /*!< FOC parameters structure */

extern ACR_CURPI_COFIN              g_acrCurIdCofIn_M2;        /*!< Current loop d axis PID parameter input structure */
extern ACR_CURPI_COFIN              g_acrCurIqCofIn_M2;        /*!< Current loop q axis PID parameter input structure */

extern ACR_UDQDCP_COF               g_udqDcpCof_M2;            /*!< Feedforward decoupled parameter input structure */
extern ACR_UDQDCP_OUT               g_udqDcpOut_M2;            /*!< Feedforward decoupled output structure */
extern ACR_UDQDCP_IN                g_udqDcpIn_M2;             /*!< Feedforward decoupled intput structure */

extern TRQ_LOAD_EST                 g_loadTorqueEst_M2;        /*!< Load torque estimate structure */

extern PWM_GEN_OUT                  g_pwmValue_M2;             /*!< Pwm generate output structure */

extern CUR_RESTRUCT_DATA            g_curRestruct_M2;          /*!< Current restruct function data */
extern PWM_SHIFT_DATA               g_pwmShift_M2;             /*!< Pwm shift sub-function data */

extern ASR_SPEEDPI_COFIN            g_asrSpeedIqCofIn_M2;      /*!< Speed loop PID parameter input structure */

extern SMC_ESTIMATOR                g_smcEstimator_M2;         /*!< Sliding mode control structure */
extern THETA_COMPENSATOR            g_thetaComp_M2;            /*!< Angle compensation structure */
extern SM_OBSERVER                  g_smo_M2;

extern FLUX_OBSERVER_CTRL           g_fluxObserverCtrl_M2;     /*!< Flux Observer control structure */
extern FLUX_OBSERVER                g_fluxObserver_M2;         /*!< Flux Observer control and configuration structure */
extern MRAS_OBSERVER                g_mrasObserver_M2;         /*!< Mras Observer structure */
extern MRAS                         g_mras_M2;

extern HALL_SENSOR                  g_hall_M2;                 /*!< Hall sensor assembly struct */
extern HALL_SENSOR_DATA             g_hallSensor_M2;           /*!< Hall sensor calculate rotor position structure */
extern HALL_LEARN_DATA              g_hallSelfLearn_M2;        /*!< Hall sensor self learn data */

#if (defined ENCODER_SENSORS)
extern ENCODER_TYPE                 g_encoderSensor_M2;        /*!< Encoder sensor structure in ENCODER FOC mode */
#if (defined MT_SPEED_MEASURE)
extern ENCODER_SPEED_MEASURE        g_encoderSpeed_M2;         /*!< Encoder speed measure structure in ENCODER FOC mode */
#endif
#endif

#if (defined ENCODER_HALL_SENSORS)
extern ENCODER_TYPE                 g_encoderSensor_M2;        /*!< Encoder sensor structure in ENCODER FOC mode */
extern ENCODER_HALL_SENSOR          g_encoderHallSensor_M2;    /*!< Encoder_Hall sensor data struct */
#endif

#if (defined PULSE_INJECTION)
extern PULSE_INJECT_DATA            g_pulseInject_M2;          /*!< Initial position data struct */
#endif

#if (defined INIT_POS_DETEC)
extern INIT_POSITION_POLARITY_CHECK g_initPosPolar_M2;      /*!< Initial position check with rotation voltage injection polarity check data struct */
extern INITPOS_DETEC_STRUCT         g_initPosDetecStruct_M2;    /*!< Initial position check with rotation voltage injection struct */
extern INITPOS_DETEC_PARAM          g_initPosDetecParam_M2;     /*!< Initial position check with rotation voltage injection parameters struct */
extern FFT_PARAM                    g_initPosDetecFft_M2;       /*!< Initial position check with rotation voltage injection FFT data struct */
#endif

extern FLUXWEAK_PARAM               g_fluxWeak_M2;             /*!< fluxWeak parameter structure */
extern MTPA_THETA_COMPENSATOR       g_thetaMtpaComp_M2;        /*!< MTPA Angle compensation structure */

extern HFI_INIT_POS_DETECT          g_initPosDet_M2;           /*!< Hfi Initial position detect structure */
extern HFI_BPF_COEFF                g_bpfCoeff_M2;             /*!< Hfi Butterworth BPF structure */
extern HFI_TWO_ORDER_LPF            g_lpfCoeff_M2;             /*!< Hfi Two Order Butterworth LPF structure */
extern HFI_TYPE                     g_hfinj_M2;                /*!< Hfi related structure */
extern HFI_CTRL                     g_hfiCtrl_M2;              /*!< Hfi control structure, including INIT_POS_DETECT,BPF_COEFF,TWO_ORDER_LPF and HIGH_FREQ_INJ_TYPE */

extern TORQUE_COMMAND               g_torqueCommand_M2;        /*!< Torque command parameters structure */

/*!
* Speed control
*/
extern SPEED_RAMP_HANDLE            g_speedCommand_M2;         /*!< Speed command ramp structure */

extern SCURVE_DATA                  g_sCurveData_M2;           /*!< S curve speed ramp calculation structure */
extern SCURVE                       g_sCurve_M2;               /*!< S curve speed ramp structure */

extern ASR_SPEEDVARPI               g_asrSpeedVarPI_M2;        /*!< Speed loop variable PI parameters structure */

extern ESTIMATED_SPEED              g_rotationalSpeed_M2;      /*!< Motor speed calculation structure */

/*!
* Position control
*/
extern POSITION_RAMP_HANDLE         g_positionCommand_M2;      /*!< Position command structure */

/*!
* Open loop start up
*/
extern STARTUP_CTRL                 g_startUpCtrl_M2;          /*!< Startup control parameters structure */
extern START_UP                     g_startUp_M2;              /*!< Startup structure */
extern CURVE_REFVALUE               g_startUpCurveSeg1_M2;     /*!< Open loop startup curve segment 1 parameters */
extern CURVE_REFVALUE               g_startUpCurveSeg2_M2;     /*!< Open loop startup curve segment 2 parameters */
extern CURVE_REFVALUE               g_startUpCurveSeg3_M2;     /*!< Open loop startup curve segment 3 parameters */
extern CURVE_REFVALUE               g_startUpCurveSeg4_M2;     /*!< Open loop startup curve segment 4 parameters */
extern CURVE_REFVALUE               g_startUpCurveSeg5_M2;     /*!< Open loop startup curve segment 5 parameters */
extern CURVE_IN                     g_startUpCurveIn_M2;       /*!< Open loop startup curve input parameters */
extern CURVE_OUT                    g_startUpCurveOut_M2;      /*!< Open loop startup curve output parameters */
extern CURVE_TIME_DURATION          g_startUpCurveTime_M2;     /*!< Open loop startup curve time parameters structure */
extern STARTUP_CURVE                g_startUpCurve_M2;         /*!< Open loop startup curve structure */

/*!
* Feature functions
*/
extern SHORT_BRAKE_DATA             g_shortBrake_M2;           /*!< Short brake function data struct */
extern ELEC_BRAKE_DATA              g_elecBrake_M2;            /*!< Electric brake function data struct */

extern SHORT_BRAKE                  g_sBrake_M2;               /*!< Short brake structure */
extern ELEC_BRAKE                   g_eBrake_M2;               /*!< Electric brake structure */

extern PARAM_IDENTIFY_STRUCT        g_paramIdentify_M2;        /*!< Parameter identify struct */
extern PARAM_IDENTIFY_MAIN_CTRL     g_paramIdentifyCtrl_M2;    /*!< Identify motor parameter data struct */
extern PARAM_RS_IDENTIFY_DATA       g_paramRsIdentify_M2;      /*!< Identify Rs data struct */
extern PARAM_LDQ_IDENTIFY_DATA      g_paramLdqIdentify_M2;     /*!< Identify Ld Lq data struct */
extern PARAM_FLUX_IDENTIFY_DATA     g_paramFluxIdentify_M2;    /*!< Identify flux data struct*/
extern PARAM_INERTIA_IDENTIFY_DATA  g_paramInertiaIdentify_M2; /*!< Identify inertia data struct */

extern ACIM_PARAM_IDENTIFY          g_acimParamId_M2;          /*!< Acim parameters identification data struct */
extern SIGNAL_PHASE_SHIFT           g_sigPhaseShift_M2;        /*<! Signal phase shifter data struct */

extern PMSM_LDQID_PARAM             g_pmsmLdqId_M2;            /*!< PMSM DQ inductance identification struct */
extern PMSM_LDQID_SWDATA            g_pmsmLdqSwParam_M2;       /*!< PMSM DQ inductance identification data struct */
extern FFT_PARAM                    g_fftParam_M2;             /*!< PMSM DQ inductance identification FFT data struct */

extern INERTIA_IDENTIFY_STRUCT      g_inertiaIdentify_M2;      /*!< Identify inertia data struct */

extern OVER_VOLTAGE_SUPPRESS        g_overVoltSupp_M2;         /*!< Bus over voltage suppress data */

extern POWER_EST                    g_powerEst_M2;             /*!< Power estimate structure */
extern DCCUR_EST                    g_dcCurEst_M2;             /*!< Dc current estimate structure */

extern DEAD_BAND_COMP_STRUCT        g_deadCompCtrl_M2;         /*!< Dead band compensation data stauct */

extern ENERGY_SAVE                  g_energySave_M2;           /*!< Enegy save function data */

extern INITSPEED_JUDGE              g_initSpeedJudge_M2;       /*!< Speed judge function data */

/*!
* Induction motor FOC
*/
//#if (defined FOC_ACIM)
//extern ACIM_FOC_CTRL                g_acimFocCtrlMotor2;          /*!< Induction motor FOC control structure */
//extern ACIM_FLUX_PARAM              g_acimFluxMotor2;             /*!< Induction motor FOC control flux parameters structure */
//extern ACIM_SPEED_PARAM             g_acimSpeedMotor2;            /*!< Induction motor FOC control speed parameters structure */
//extern ACIM_CURRENT_PARAM           g_acimCurMotor2;              /*!< Induction motor FOC control current parameters structure */
//extern ACIM_VOLTAGE_PARAM           g_acimVoltMotor2;             /*!< Induction motor FOC control voltage parameters structure */
//extern ACIM_MOTOR_PARAM             g_aciMotorMotor2;             /*!< Induction motor FOC control motor parameters structure */
//#endif

/*!
* PID
*/
/* Acr_d */
extern PID_REGULATOR_IN             g_acrCurIdPIIn_M2;         /*!< Current loop d axis PID input structure */
extern PID_REGULATOR_COF            g_acrCurIdCof_M2;          /*!< Current loop d axis PID parameter structure */
extern PID_REGULATOR_OUT            g_acrCurIdPIOut_M2;        /*!< Current loop d axis PID output structure */
/* Acr_q */
extern PID_REGULATOR_IN             g_acrCurIqPIIn_M2;         /*!< Current loop q axis PID input structure */
extern PID_REGULATOR_COF            g_acrCurIqCof_M2;          /*!< Current loop q axis PID parameter structure */
extern PID_REGULATOR_OUT            g_acrCurIqPIOut_M2;        /*!< Current loop q axis PID output structure */
/* Asr */
extern PID_REGULATOR_IN             g_asrSpeedPIIn_M2;         /*!< Speed loop PID input structure */
extern PID_REGULATOR_COF            g_asrSpeedCof_M2;          /*!< Speed loop PID parameter structure */
extern PID_REGULATOR_OUT            g_asrSpeedPIOut_M2;        /*!< Speed loop PID output structure */

/* Apr */
extern PID_REGULATOR_IN             g_aprPositionPIIn_M2;      /*!< Position loop PID input structure */
extern PID_REGULATOR_COF            g_aprPositionCof_M2;       /*!< Position loop PID parameter structure */
extern PID_REGULATOR_OUT            g_aprPositionPIOut_M2;     /*!< Position loop PID output structure */

/* Speed estimated */
extern PID_REGULATOR_IN             g_asrSpeedEstPIIn_M2;      /*!< PLL method speed loop estimation PID input structure */
extern PID_REGULATOR_COF            g_asrSpeedEstCof_M2;       /*!< Speed est pll PID parameter structure */
extern PID_REGULATOR_OUT            g_asrSpeedEstPIOut_M2;     /*!< PLL method speed loop estimation PID output structure */
/* flux weakening */
extern PID_REGULATOR_IN             g_fluxWeakPIIn_M2;         /*!< fluxWeak PID input structure */
extern PID_REGULATOR_COF            g_fluxWeakPICof_M2;        /*!< fluxWeak PID parameter structure */
extern PID_REGULATOR_OUT            g_fluxWeakPIOut_M2;        /*!< fluxWeak PID output structure */

/* Hfi */
extern PID_REGULATOR_IN             g_asrHfiEstPIIn_M2;        /*!< PLL method Hfi estimation PID input structure */
extern PID_REGULATOR_COF            g_asrHfiEstCof_M2;         /*!< PLL method Hfi estimation PID parameter structure */
extern PID_REGULATOR_OUT            g_asrHfiEstPIOut_M2;       /*!< PLL method Hfi estimation PID output structure */

/* Udc suppress */
extern PID_REGULATOR_IN             g_udcLimitPIIn_M2;         /*!< Bus over voltage suppress PID input structure */
extern PID_REGULATOR_OUT            g_udcLimitPIOut_M2;        /*!< Bus over voltage suppress PID output structure */
extern PID_REGULATOR_COF            g_udcLimitPICof_M2;        /*!< Bus over voltage suppress PID parameter structure */

/* Rs identification ACR */
extern PID_REGULATOR_IN             g_acrAlfaRsIdPIIn_M2;      /*!< Rs identification ACR alpha axis component PID input structure */
extern PID_REGULATOR_OUT            g_acrAlfaRsIdPIOut_M2;     /*!< Rs identification ACR alpha axis component PID output structure */
extern PID_REGULATOR_COF            g_acrAlfaRsIdPICof_M2;     /*!< Rs identification ACR alpha axis component PID parameter structure */

extern PID_REGULATOR_IN             g_acrBetaRsIdPIIn_M2;      /*!< Rs identification ACR beta axis component PID input structure */
extern PID_REGULATOR_OUT            g_acrBetaRsIdPIOut_M2;     /*!< Rs identification ACR beta axis component PID output structure */
extern PID_REGULATOR_COF            g_acrBetaRsIdPICof_M2;     /*!< Rs identification ACR beta axis component PID parameter structure */

/* PID structure */
extern PID_REGULATOR                g_pidAcrD_M2;              /*!< D_asxis current loop (flux) PID regulator structure */
extern PID_REGULATOR                g_pidAcrQ_M2;              /*!< Q_asxis current loop (torque) PID regulator structure */
extern PID_REGULATOR                g_pidAsr_M2;               /*!< Speed loop PID regulator structure */

extern PID_REGULATOR                g_pidApr_M2;               /*!< Position loop PID regulator structure */

extern PID_REGULATOR                g_pidPll_M2;               /*!< PLL speed estimated */
extern PID_REGULATOR                g_pidFw_M2;                /*!< Flux weakening */

extern PID_REGULATOR                g_pidHfi_M2;               /*!< High frequency injection */

extern PID_REGULATOR                g_pidUdcLimit_M2;          /*!< Bus over voltage suppress */

extern PID_REGULATOR                g_pidAcrAlfaRsId_M2;       /*!< Alpha_asxis current loop for Rs identification PID regulator structure */
extern PID_REGULATOR                g_pidAcrBetaRsId_M2;       /*!< Beta_asxis current loop for Rs identification PID regulator structure */

extern PARAM_IDENTIFY_BUFFER        g_paramIdentifyBuffer_M2;  /*!< Buffer structure of parameter identification */
extern HALL_SELFLEARN_BUFFER        g_hallSelfLearnBuffer_M2;  /*!< Buffer structure of hall sensor self learn result */

/*!
* Configuration
*/
extern FOC_VARS_CFG                 *pFocVarsCfg_M2;           /*!< FOC configuration parameters */
extern TRANSFORM_CFG                *pTransformCfg_M2;         /*!< ADC sampling transform configuration parameters */
extern FLUX_OBSERVER_CFG            *pFluxObserveCfg_M2;       /*!< Flux observer configuration parameters */
extern PLL_CFG                      *pPllCfg_M2;               /*!< PLL speed estimated configuration parameters */
extern SMC_CFG                      *pSmoObserveCfg_M2;        /*!< Sliding mode configuration parameters*/
extern MRAS_OBSERVER_CFG            *pMrasObserveCfg_M2;       /*!< Mras Observer configuration parameters */

extern CUR_RESTRUCT_CFG             *pCurRestructCfg_M2;       /*!< Current resonstruction configuration parameters */
extern SVPWM_CFG                    *pSvModulationCfg_M2;      /*!< SVPWM over modulation configuration parameters */
extern DEADCOMP_CFG                 *pDeadCompCfg_M2;          /*!< SVPWM dead time compensation configuration parameters */

extern HFI_CFG                      *pHighFreqInjCfg_M2;       /*!< High frequency injection configuration parameters */
extern FLUXWEAK_CFG                 *pFluxWeakCfg_M2;          /*!< Flux weakening configuration parameters */

extern START_UP_CFG                 *pStartUpCfg_M2;           /*!< FOC start up configuration parameters */
extern PULSE_INJ_CFG                *pPulseInjCfg_M2;          /*!< FOC pulse injection configuration parameters */
#if (defined INIT_POS_DETEC)
extern INITPOS_DETEC_CFG            *pInitPosDetecCfg_M2;      /*!< Rotation voltage injection initial position check configuration parameters */
#endif

extern SHORT_BRAKE_CFG              *pShortBrakeCfg_M2;        /*!< Short brake configuration parameters */
extern ENERGY_SAVE_CFG              *pEnergySaveCfg_M2;        /*!< Energy save configuration parameters */
extern ELECBRAKE_CFG                *pElecBrakeCfg_M2;         /*!< Electromagnetic brake configuration parameters */

extern POWEREST_CFG                 *pPowerEstCfg_M2;          /*!< Power estimated configuration parameters */

extern PID_CFG                      *pPidCfg_M2;               /*!< PID regulator configuration parameters */

extern LOAD_ESTIMATE_CFG            *pLoadTorqueCfg_M2;        /*!< Load torque estimate configuration parameters */

extern INERTIA_IDENTIFY_CFG         *pInertiaIdentCfg_M2;      /*!< Inertia identify configuration parameters */

extern SCURVE_CFG                   *pSCurveCfg_M2;            /*!< Speed S-Curve configuration parameters */

#if (defined DATA_ACCESS_FLASH_ENABLE)
extern DATA_ACCESS_FLASH           g_dataAccessFlash_M2;                      /*!< Variables access flash operation parameters structure */
extern DATA_ACCESS_FLASH_CFG        *pEflashCfg_M2;            /*!< Eflash access configuration parameters */
#endif

#endif  /* end of MOTOR_TWO_TYPE */

/* ====================================  Functions declaration  ===================================== */

/* ======================================  Functions define  ======================================== */

#endif /* __GLOBAL_VAR_H__ */

/* =============================================  EOF  ============================================== */
