/* Copyright Statement:
*
* This software/firmware and related documentation ("AutoChips Software") are
* protected under relevant copyright laws. The information contained herein is
* confidential and proprietary to AutoChips Inc. and/or its licensors. Without
* the prior written permission of AutoChips inc. and/or its licensors, any
* reproduction, modification, use or disclosure of AutoChips Software, and
* information contained herein, in whole or in part, shall be strictly
* prohibited.
*
* AutoChips Inc. (C) 2020. All rights reserved.
*
* BY OPENING THIS FILE, RECEIVER HEREBY UNEQUIVOCALLY ACKNOWLEDGES AND AGREES
* THAT THE SOFTWARE/FIRMWARE AND ITS DOCUMENTATIONS ("AUTOCHIPS SOFTWARE")
* RECEIVED FROM AUTOCHIPS AND/OR ITS REPRESENTATIVES ARE PROVIDED TO RECEIVER
* ON AN "AS-IS" BASIS ONLY. AUTOCHIPS EXPRESSLY DISCLAIMS ANY AND ALL
* WARRANTIES, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
* WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
* NONINFRINGEMENT. NEITHER DOES AUTOCHIPS PROVIDE ANY WARRANTY WHATSOEVER WITH
* RESPECT TO THE SOFTWARE OF ANY THIRD PARTY WHICH MAY BE USED BY,
* INCORPORATED IN, OR SUPPLIED WITH THE AUTOCHIPS SOFTWARE, AND RECEIVER AGREES
* TO LOOK ONLY TO SUCH THIRD PARTY FOR ANY WARRANTY CLAIM RELATING THERETO.
* RECEIVER EXPRESSLY ACKNOWLEDGES THAT IT IS RECEIVER'S SOLE RESPONSIBILITY TO
* OBTAIN FROM ANY THIRD PARTY ALL PROPER LICENSES CONTAINED IN AUTOCHIPS
* SOFTWARE. AUTOCHIPS SHALL ALSO NOT BE RESPONSIBLE FOR ANY AUTOCHIPS SOFTWARE
* RELEASES MADE TO RECEIVER'S SPECIFICATION OR TO CONFORM TO A PARTICULAR
* STANDARD OR OPEN FORUM. RECEIVER'S SOLE AND EXCLUSIVE REMEDY AND AUTOCHIPS'S
* ENTIRE AND CUMULATIVE LIABILITY WITH RESPECT TO THE AUTOCHIPS SOFTWARE
* RELEASED HEREUNDER WILL BE, AT AUTOCHIPS'S OPTION, TO REVISE OR REPLACE THE
* AUTOCHIPS SOFTWARE AT ISSUE, OR REFUND ANY SOFTWARE LICENSE FEES OR SERVICE
* CHARGE PAID BY RECEIVER TO AUTOCHIPS FOR SUCH AUTOCHIPS SOFTWARE AT ISSUE.
*/

/*!
* @file foc_app.c
*
* @brief This file provides FOC control parameters initialization, state machine and millisecond time base functions.
*
*/

/* ===========================================  Includes  =========================================== */
#include "gpio_drv.h"
#include "foc_app.h"
#include "foc_isr.h"
#include "globalvar.h"
#include "pulse_inject_api.h"
#include "motorctrl_api.h"
#include "foc_ui_uart.h"
#include "vf_control.h"

/* ============================================  Define  ============================================ */

/* ===========================================  Typedef  ============================================ */

/* ==========================================  Variables  =========================================== */
status_t g_flashOperationStatus = STATUS_UNSUPPORTED;       /*!< Return value of eflash operation status */
uint16_t g_flashTaskFinish = 0;                             /*!< Finish tag of flash task, used to uart communication */

status_t g_flashOperationStatus_M2 = STATUS_UNSUPPORTED;    /*!< Return value of eflash operation status */
uint16_t g_flashTaskFinish_M2 = 0;                          /*!< Finish tag of flash task, used to uart communication */

/* ====================================  Functions declaration  ===================================== */

/* ======================================  Functions define  ======================================== */
/*!
* @brief Common parameters initialization of Motor_1 and Motor_2.
*
* @param[in] none
* @return none
*/
void CommonParamInit(void)
{
    pHWBoardParam = &defHwBoard;
}

/*!
* @brief Motor vector control global variable parameters initialization.
*
* @param[in] STC: pointer to STC_PARAM structure
* @return none
*/
void STC_ParamInit(STC_PARAM *STC)
{
#if (defined MOTOR_ONE_TYPE)

    pMotorParam = &defMotorParam;

    g_adcParam.adCoef = &g_adcSampleCof;
    g_adcParam.adSample = &g_adcSampleReal;
    g_adcParam.adTrans = &g_adcTransform;

    pFocVarsCfg = &defFocVars;
    pTransformCfg = &defTransform;

    pStartUpCfg = &defStartUp;

#if (defined PULSE_INJECTION)
    pPulseInjCfg = &defPulseInj;
#endif
    pShortBrakeCfg = &defShortBrake;
#if (defined SAVE_ENERGY)
    pEnergySaveCfg = &defEnergySave;
#endif
    pCurRestructCfg = &defCurRestruct;
    pSvModulationCfg = &defSvModulation;
    pDeadCompCfg = &defDeadComp;
    pPidCfg = &defPID;

    pHighFreqInjCfg = &defHighFreqInj;
    g_hfiCtrl.bpf = &g_bpfCoeff;
    g_hfiCtrl.lpf = &g_lpfCoeff;
    g_hfiCtrl.initPos = &g_initPosDet;
    g_hfiCtrl.hfi = &g_hfinj;

    pPowerEstCfg = &defPowerEst;
    pFluxObserveCfg = &defFluxObserver;
    pMrasObserveCfg = &defMrasObserver;
    pPllCfg = &defPll;
    pSmoObserveCfg = &defSlideMode;
    pElecBrakeCfg = &defElecBrake;
    pFluxWeakCfg = &defFluxweak;
    pSCurveCfg = &defSCurve;

    pLoadTorqueCfg = &defLoadTorque;

    pInertiaIdentCfg = &defInertiaIdentify;

    STC_motorParamPuCalc(pMotorParam, pFocVarsCfg, &g_motorParamPu);
    ADC_SampleCofCalc(&g_motorParamPu, pHWBoardParam, &g_adcSampleCof);

    g_focVars.pCfg = pFocVarsCfg;
    g_focVars.pCtrl = &g_focVarsCtrl;

    STC->pFOCVars = &g_focVars;
    STC->pMotor = pMotorParam;
    STC->pMotorPu = &g_motorParamPu;

    STC->smc = &g_smo;
    STC->smc->comp = &g_thetaComp;
    STC->smc->smoCfg = pSmoObserveCfg;
    STC->smc->smoEst = &g_smcEstimator;
    STC->flux = &g_fluxObserver;
    STC->flux->pCfg = pFluxObserveCfg;
    STC->flux->pCtrl = &g_fluxObserverCtrl;
    STC->mras = &g_mras;
    STC->mras->mrasCfg = pMrasObserveCfg;
    STC->mras->mrasCtrl = &g_mrasObserver;

    g_fluxObserver.pCfg = pFluxObserveCfg;
    g_fluxObserver.pCtrl = &g_fluxObserverCtrl;

    g_startUp.pStCfg = pStartUpCfg;
    g_startUp.pStCtrl = &g_startUpCtrl;

    /* PID structure */
    g_pidAcrD.pPidInput = &g_acrCurIdPIIn;
    g_pidAcrD.pPidCoef = &g_acrCurIdCof;
    g_pidAcrD.pPidOutput = &g_acrCurIdPIOut;

    g_pidAcrQ.pPidInput = &g_acrCurIqPIIn;
    g_pidAcrQ.pPidCoef = &g_acrCurIqCof;
    g_pidAcrQ.pPidOutput = &g_acrCurIqPIOut;

    g_pidAsr.pPidInput = &g_asrSpeedPIIn;
    g_pidAsr.pPidCoef = &g_asrSpeedCof;
    g_pidAsr.pPidOutput = &g_asrSpeedPIOut;

#if (defined POSITION_LOOP_CTRL)
    g_pidApr.pPidInput = &g_aprPositionPIIn;
    g_pidApr.pPidCoef = &g_aprPositionCof;
    g_pidApr.pPidOutput = &g_aprPositionPIOut;
#endif

    g_pidPll.pPidInput = &g_asrSpeedEstPIIn;
    g_pidPll.pPidCoef = &g_asrSpeedEstCof;
    g_pidPll.pPidOutput = &g_asrSpeedEstPIOut;

    g_pidFw.pPidInput = &g_fluxWeakPIIn;
    g_pidFw.pPidCoef = &g_fluxWeakPICof;
    g_pidFw.pPidOutput = &g_fluxWeakPIOut;

    g_pidHfi.pPidInput = &g_asrHfiEstPIIn;
    g_pidHfi.pPidCoef = &g_asrHfiEstCof;
    g_pidHfi.pPidOutput = &g_asrHfiEstPIOut;

    g_pidUdcLimit.pPidInput = &g_udcLimitPIIn;
    g_pidUdcLimit.pPidCoef = &g_udcLimitPICof;
    g_pidUdcLimit.pPidOutput = &g_udcLimitPIOut;

    /* ACR for Rs identification */
    g_pidAcrAlfaRsId.pPidCoef = &g_acrAlfaRsIdPICof;
    g_pidAcrAlfaRsId.pPidInput = &g_acrAlfaRsIdPIIn;
    g_pidAcrAlfaRsId.pPidOutput = &g_acrAlfaRsIdPIOut;
    g_pidAcrBetaRsId.pPidCoef = &g_acrBetaRsIdPICof;
    g_pidAcrBetaRsId.pPidInput = &g_acrBetaRsIdPIIn;
    g_pidAcrBetaRsId.pPidOutput = &g_acrBetaRsIdPIOut;

    g_paramIdentify.pParamIdentifyCfg = &defParamIdentify;
    g_paramIdentify.pParamIdentifyCtrl = &g_paramIdentifyCtrl;
    g_paramIdentify.pParamRsIdentify = &g_paramRsIdentify;
    g_paramIdentify.pParamLdqIdentify = &g_paramLdqIdentify;
    g_paramIdentify.pParamFluxIdentify = &g_paramFluxIdentify;
    g_paramIdentify.pParamInertiaIdentify = &g_paramInertiaIdentify;

    g_paramIdentify.pmsmLdqIdentify = &g_pmsmLdqId;
    g_paramIdentify.pmsmLdqIdentify->ldqIdParam = &g_pmsmLdqSwParam;
    g_paramIdentify.pmsmLdqIdentify->fftParam = &g_fftParam;

    g_hall.pHallCfg = &defHallSensor;
    g_hall.pHallSensor = &g_hallSensor;
    g_hall.pHallSelfLearn = &g_hallSelfLearn;

    g_speedCommand.speedCalc = &g_rotationalSpeed;

    g_startUpCurve.curveIn = &g_startUpCurveIn;
    g_startUpCurve.curveOut = &g_startUpCurveOut;
    g_startUpCurve.curveTime = &g_startUpCurveTime;
    g_startUpCurveIn.curveSeg1 = &g_startUpCurveSeg1;
    g_startUpCurveIn.curveSeg2 = &g_startUpCurveSeg2;
    g_startUpCurveIn.curveSeg3 = &g_startUpCurveSeg3;
    g_startUpCurveIn.curveSeg4 = &g_startUpCurveSeg4;
    g_startUpCurveIn.curveSeg5 = &g_startUpCurveSeg5;

    g_sCurve.scurveCtrl = &g_sCurveData;
    g_sCurve.scurveCfg = pSCurveCfg;

    g_sBrake.cfg = pShortBrakeCfg;
    g_sBrake.ctrl = &g_shortBrake;
    g_eBrake.cfg = pElecBrakeCfg;
    g_eBrake.ctrl = &g_elecBrake;

#if (defined INIT_POS_DETEC)
    pInitPosDetecCfg = &defInitPosDetec;
    g_initPosDetecStruct.fftParam = &g_initPosDetecFft;
    g_initPosDetecStruct.initPosDetec = &g_initPosDetecParam;
#endif

#if (defined DATA_ACCESS_FLASH_ENABLE)
    pEflashCfg = &defEflashAccessCfg;
#endif

//#if (defined FOC_ACIM)
//    g_acimFocCtrl.cur = &g_acimCur;
//    g_acimFocCtrl.flux = &g_acimFlux;
//    g_acimFocCtrl.mot = &g_aciMotor;
//    g_acimFocCtrl.speed = &g_acimSpeed;
//    g_acimFocCtrl.volt = &g_acimVolt;
//    g_paramIdentify.pAcimParamIdentify = &g_acimParamId;
//#endif

#endif
}

/*!
* @brief Motor vector control global variable parameters initialization.
*
* @param[in] STCMotor2: pointer to STC_PARAM structure
* @return none
*/
void STC_ParamInit_M2(STC_PARAM *STCMotor2)
{
#if (defined MOTOR_TWO_TYPE)
    g_adcParam_M2.adCoef = &g_adcSampleCof_M2;
    g_adcParam_M2.adSample = &g_adcSampleReal_M2;
    g_adcParam_M2.adTrans = &g_adcTransform_M2;

    pMotorParam_M2 = &defMotorParam_M2;

    pFocVarsCfg_M2 = &defFocVars_M2;
    pTransformCfg_M2 = &defTransform_M2;

    pStartUpCfg_M2 = &defStartUp_M2;

#if (defined PULSE_INJECTION_M2)
    pPulseInjCfg_M2 = &defPulseInj_M2;
#endif

    pShortBrakeCfg_M2 = &defShortBrake_M2;
#if (defined SAVE_ENERGY_M2)
    pEnergySaveCfg_M2 = &defEnergySave_M2;
#endif

    pCurRestructCfg_M2 = &defCurRestruct_M2;
    pSvModulationCfg_M2 = &defSvModulation_M2;
    pDeadCompCfg_M2 = &defDeadComp_M2;
    pPidCfg_M2 = &defPID_M2;

    pHighFreqInjCfg_M2 = &defHighFreqInj_M2;
    g_hfiCtrl_M2.bpf = &g_bpfCoeff_M2;
    g_hfiCtrl_M2.lpf = &g_lpfCoeff_M2;
    g_hfiCtrl_M2.initPos = &g_initPosDet_M2;
    g_hfiCtrl_M2.hfi = &g_hfinj_M2;

    pPowerEstCfg_M2 = &defPowerEst_M2;
    pFluxObserveCfg_M2 = &defFluxObserver_M2;
    pMrasObserveCfg_M2 = &defMrasObserver_M2;
    pPllCfg_M2 = &defPll_M2;
    pSmoObserveCfg_M2 = &defSlideMode_M2;
    pElecBrakeCfg_M2 = &defElecBrake_M2;
    pFluxWeakCfg_M2 = &defFluxweak_M2;
    pSCurveCfg_M2 = &defSCurve_M2;

    pLoadTorqueCfg_M2 = &defLoadTorque_M2;

    pInertiaIdentCfg_M2 = &defInertiaIdentify_M2;

    STC_motorParamPuCalc(pMotorParam_M2, pFocVarsCfg_M2, &g_motorParamPu_M2);
    ADC_SampleCofCalc(&g_motorParamPu_M2, pHWBoardParam, &g_adcSampleCof_M2);

    g_focVars_M2.pCfg = pFocVarsCfg_M2;
    g_focVars_M2.pCtrl = &g_focVarsCtrl_M2;

    STCMotor2->pFOCVars = &g_focVars_M2;
    STCMotor2->pMotor = pMotorParam_M2;
    STCMotor2->pMotorPu = &g_motorParamPu_M2;

    STCMotor2->smc = &g_smo_M2;
    STCMotor2->smc->comp = &g_thetaComp_M2;
    STCMotor2->smc->smoCfg = pSmoObserveCfg_M2;
    STCMotor2->smc->smoEst = &g_smcEstimator_M2;
    STCMotor2->flux = &g_fluxObserver_M2;
    STCMotor2->flux->pCfg = pFluxObserveCfg_M2;
    STCMotor2->flux->pCtrl = &g_fluxObserverCtrl_M2;
    STCMotor2->mras = &g_mras_M2;
    STCMotor2->mras->mrasCfg = pMrasObserveCfg_M2;
    STCMotor2->mras->mrasCtrl = &g_mrasObserver_M2;
    g_fluxObserver_M2.pCfg = pFluxObserveCfg_M2;
    g_fluxObserver_M2.pCtrl = &g_fluxObserverCtrl_M2;

    g_startUp_M2.pStCfg = pStartUpCfg_M2;
    g_startUp_M2.pStCtrl = &g_startUpCtrl_M2;

    /* PID structure */
    g_pidAcrD_M2.pPidInput = &g_acrCurIdPIIn_M2;
    g_pidAcrD_M2.pPidCoef = &g_acrCurIdCof_M2;
    g_pidAcrD_M2.pPidOutput = &g_acrCurIdPIOut_M2;

    g_pidAcrQ_M2.pPidInput = &g_acrCurIqPIIn_M2;
    g_pidAcrQ_M2.pPidCoef = &g_acrCurIqCof_M2;
    g_pidAcrQ_M2.pPidOutput = &g_acrCurIqPIOut_M2;

    g_pidAsr_M2.pPidInput = &g_asrSpeedPIIn_M2;
    g_pidAsr_M2.pPidCoef = &g_asrSpeedCof_M2;
    g_pidAsr_M2.pPidOutput = &g_asrSpeedPIOut_M2;

#if (defined POSITION_LOOP_CTRL_M2)
    g_pidApr_M2.pPidInput = &g_aprPositionPIIn_M2;
    g_pidApr_M2.pPidCoef = &g_aprPositionCof_M2;
    g_pidApr_M2.pPidOutput = &g_aprPositionPIOut_M2;
#endif

    g_pidPll_M2.pPidInput = &g_asrSpeedEstPIIn_M2;
    g_pidPll_M2.pPidCoef = &g_asrSpeedEstCof_M2;
    g_pidPll_M2.pPidOutput = &g_asrSpeedEstPIOut_M2;

    g_pidFw_M2.pPidInput = &g_fluxWeakPIIn_M2;
    g_pidFw_M2.pPidCoef = &g_fluxWeakPICof_M2;
    g_pidFw_M2.pPidOutput = &g_fluxWeakPIOut_M2;

    g_pidHfi_M2.pPidInput = &g_asrHfiEstPIIn_M2;
    g_pidHfi_M2.pPidCoef = &g_asrHfiEstCof_M2;
    g_pidHfi_M2.pPidOutput = &g_asrHfiEstPIOut_M2;

    g_pidUdcLimit_M2.pPidInput = &g_udcLimitPIIn_M2;
    g_pidUdcLimit_M2.pPidCoef = &g_udcLimitPICof_M2;
    g_pidUdcLimit_M2.pPidOutput = &g_udcLimitPIOut_M2;

    /* ACR for Rs identification */
    g_pidAcrAlfaRsId_M2.pPidCoef = &g_acrAlfaRsIdPICof_M2;
    g_pidAcrAlfaRsId_M2.pPidInput = &g_acrAlfaRsIdPIIn_M2;
    g_pidAcrAlfaRsId_M2.pPidOutput = &g_acrAlfaRsIdPIOut_M2;
    g_pidAcrBetaRsId_M2.pPidCoef = &g_acrBetaRsIdPICof_M2;
    g_pidAcrBetaRsId_M2.pPidInput = &g_acrBetaRsIdPIIn_M2;
    g_pidAcrBetaRsId_M2.pPidOutput = &g_acrBetaRsIdPIOut_M2;

    g_paramIdentify_M2.pParamIdentifyCfg = &defParamIdentify_M2;
    g_paramIdentify_M2.pParamIdentifyCtrl = &g_paramIdentifyCtrl_M2;
    g_paramIdentify_M2.pParamRsIdentify = &g_paramRsIdentify_M2;
    g_paramIdentify_M2.pParamLdqIdentify = &g_paramLdqIdentify_M2;
    g_paramIdentify_M2.pParamFluxIdentify = &g_paramFluxIdentify_M2;
    g_paramIdentify_M2.pParamInertiaIdentify = &g_paramInertiaIdentify_M2;

    g_paramIdentify_M2.pmsmLdqIdentify = &g_pmsmLdqId_M2;
    g_paramIdentify_M2.pmsmLdqIdentify->ldqIdParam = &g_pmsmLdqSwParam_M2;
    g_paramIdentify_M2.pmsmLdqIdentify->fftParam = &g_fftParam_M2;

    g_hall_M2.pHallCfg = &defHallSensor_M2;
    g_hall_M2.pHallSensor = &g_hallSensor_M2;
    g_hall_M2.pHallSelfLearn = &g_hallSelfLearn_M2;

    g_speedCommand_M2.speedCalc = &g_rotationalSpeed_M2;

    g_startUpCurve_M2.curveIn = &g_startUpCurveIn_M2;
    g_startUpCurve_M2.curveOut = &g_startUpCurveOut_M2;
    g_startUpCurve_M2.curveTime = &g_startUpCurveTime_M2;
    g_startUpCurveIn_M2.curveSeg1 = &g_startUpCurveSeg1_M2;
    g_startUpCurveIn_M2.curveSeg2 = &g_startUpCurveSeg2_M2;
    g_startUpCurveIn_M2.curveSeg3 = &g_startUpCurveSeg3_M2;
    g_startUpCurveIn_M2.curveSeg4 = &g_startUpCurveSeg4_M2;
    g_startUpCurveIn_M2.curveSeg5 = &g_startUpCurveSeg5_M2;

    g_sCurve_M2.scurveCtrl = &g_sCurveData_M2;
    g_sCurve_M2.scurveCfg = pSCurveCfg_M2;

    g_sBrake_M2.cfg = pShortBrakeCfg_M2;
    g_sBrake_M2.ctrl = &g_shortBrake_M2;
    g_eBrake_M2.cfg = pElecBrakeCfg_M2;
    g_eBrake_M2.ctrl = &g_elecBrake_M2;

#if (defined INIT_POS_DETEC_M2)
    pInitPosDetecCfg_M2 = &defInitPosDetec_M2;
    g_initPosDetecStruct_M2.fftParam = &g_initPosDetecFft_M2;
    g_initPosDetecStruct_M2.initPosDetec = &g_initPosDetecParam_M2;
#endif

#if (defined DATA_ACCESS_FLASH_ENABLE_M2)
    pEflashCfg_M2 = &defEflashAccessCfg_M2;
#endif
#endif
}

/*!
* @brief Foc control variables initialization, clear all the variables of FOC_VARS_CTRL to zero.
*
* @param[in] focVarsCtrl: pointer to FOC_VARS_CTRL structure
* @return none
*/
void FOC_CtrlVarsInit(FOC_VARS_CTRL *focVarsCtrl)
{
    memset(focVarsCtrl, 0, sizeof(*focVarsCtrl));
}

/*!
* @brief ACR parameters initialization of stator resistance identification.
*
* @param[in] pAcrAlfaRsId: pointer to PID_REGULATOR structure
* @param[in] pAcrBetaRsId: pointer to PID_REGULATOR structure
* @return none
*/
void StatorResistanceACRInit(PID_REGULATOR *pAcrAlfaRsId, PID_REGULATOR *pAcrBetaRsId)
{
    memset(pAcrAlfaRsId->pPidOutput, 0, sizeof(*(pAcrAlfaRsId->pPidOutput)));
    memset(pAcrBetaRsId->pPidOutput, 0, sizeof(*(pAcrBetaRsId->pPidOutput)));
    pAcrAlfaRsId->pPidCoef->kpPu = Math_IQ(0.5);
    pAcrAlfaRsId->pPidCoef->kiPu = Math_IQ(0.005);
    pAcrAlfaRsId->pPidInput->maxPu = 32767;
    pAcrAlfaRsId->pPidInput->minPu = -32768;

    pAcrBetaRsId->pPidCoef->kpPu = Math_IQ(0.5);
    pAcrBetaRsId->pPidCoef->kiPu = Math_IQ(0.005);
    pAcrBetaRsId->pPidInput->maxPu = 32767;
    pAcrBetaRsId->pPidInput->minPu = -32768;
}

/*!
* @brief Foc control parameters initialization, executed in the Stop state and the first Power on.
*
* @param[in] none
* @return none
*/
void FOC_Parameters_Initialize(void)
{
#if (defined MOTOR_ONE_TYPE)
    FOC_CtrlVarsInit(&g_focVarsCtrl);
    Clear_CurrentSample(&g_adcTransform);

    MC_SpeedTrqInit();
    ASR_SpeedCommandInit(pFocVarsCfg, &g_speedCommand, pTransformCfg);
#if (defined POSITION_LOOP_CTRL)
    MC_PositionInit(&g_pidApr, pPidCfg);
    APR_PositionCommandInit(pFocVarsCfg, &g_positionCommand);
#endif
#if (defined MT_SPEED_MEASURE)
    Encoder_MT_Init(&g_encoderSpeed);
#endif
    Asr_SpeedPLLInit(&g_pidPll, pFocVarsCfg, pPllCfg);

#if (defined SCURVE_SPEED_RAMP)
    SCurve_InitProcess(&g_sCurve);
#endif

    /* Sensorless FOC */
#if ((defined SENSORLESS) ||  (defined ENCODER_SENSORS))
    StartUp_Ctrl_Init(&g_startUp, &g_startUpCurve, &g_STC);
    /* Slide Mode observer */
    #if (defined SMC_OBSERVE)
    Smc_ParamInit(&g_motorParamPu, pFocVarsCfg, pSmoObserveCfg, &g_smcEstimator);
    /* Flux observer */
    #elif (defined FLUX_OBSERVE)
    FluxObserver_Init(&g_fluxObserver, &g_motorParamPu, pFocVarsCfg);
    #elif (defined MRAS_OBSERVE)
    Mras_Init(&g_mrasObserver, &g_motorParamPu, pMrasObserveCfg);
    #else
    #endif

    /* High frequency injection */
    #if (defined HIGH_FREQUENCE_INJECTION)
    Hfi_Init(&g_hfiCtrl, &g_pidHfi, pHighFreqInjCfg, pFocVarsCfg);
    #endif

    /* Rotor initial position identification by Pulse injection */
    #if (defined PULSE_INJECTION)
    PulseInject_Init(&g_pulseInject);
    #endif
#endif

//#if (defined FOC_ACIM)
//    Acim_FluxObserverInit(&g_acimFocCtrl, &g_STC);
//    StartUp_Ctrl_Init(&g_startUp, &g_startUpCurve, &g_STC);
//#endif

    /* Motor protection */
    Init_Protection(&g_protector);

    /* Is over load fall frequency */
#if (defined OVER_LOAD_PROTECT_ENABLE)
    Is_OverLoad_Init(&g_overLoadPro);
#endif

    /* Power decrease limit speed target command */
#if (defined POWER_DECREASE_PROTECT_ENABLE)
    PowerDecrease_init(&g_powerDecrease);
#endif

    /* Speed calculation parameters initialization */
    ASR_RotationSpeedInit(&g_speedCommand);

    /* Short circuit brake */
#if (SHORT_BRAKE_CTRL > 0)
    ShortBrake_Init(&g_shortBrake);
#endif
    /* Electric brake */
#if (defined ELEC_BRAKE_ENABLE)
    ElecBrake_Init(&g_elecBrake, pElecBrakeCfg);
#endif

    /* Power and DC current estimation */
#if (defined POWER_DCCUR_EST_ENABLE)
    PowerEst_Init(&g_powerEst);
    DcCurEst_Init(&g_dcCurEst);
#endif

    /* DC bus current limit */
#if (defined DCCUR_LIMIT_ENABLE)
    DcCurLimit_Init(&g_dcCurLimit);
#endif

    /* Power limit */
#if (defined POWER_LIMIT_ENABLE)
    PowerLimit_Init(&g_powerLimit);
#endif

    /* Flux weakening */
#if (defined FLUX_WEAKENING)
    Fluxweak_Init(&g_pidFw, &g_fluxWeak, pFluxWeakCfg);
#endif

    /* MTPA, maximum torque per ampere */
#if (defined MTPA_ENABLE)
    Mtpa_ParamInit(&g_thetaMtpaComp, &g_motorParamPu);
#endif
    /* Hall FOC */
#if (defined HALL_SENSORS)
    HallSensor_Init(GetHallStatus(PWM_MODULE_HALL_MOTOR1), &g_hall, &g_focVars);
#endif

    /* Motor parameters identification */
    if (g_paramIdentifyCtrl.initCtrl == 1)
    {
        ParamIdentify_Init(&g_paramIdentify);
//#if (defined FOC_PMSM)
//        PmsmId_LdqParamInit(&g_paramIdentify, &g_sigPhaseShift);
//#elif (defined FOC_ACIM)
//        AcimId_ParamInit(&g_paramIdentify, &g_sigPhaseShift);
//#endif
    }
    StatorResistanceACRInit(&g_pidAcrAlfaRsId, &g_pidAcrBetaRsId);

#if (defined TAIL_HEAD_WIND)
    InitSpeed_Judge_Init(&g_initSpeedJudge);
#endif

#ifdef OVER_VOL_SUPPRESS
    OverVoltageSuppressInit(&g_overVoltSupp, &g_pidUdcLimit, &defPID_M2, &g_STC);
#endif

#if (defined LOAD_TORQUE_ESTIMATE)
    TrqLoadEst_Init(&g_loadTorqueEst, pLoadTorqueCfg, &g_motorParamPu);
#endif

#if (defined INERTIA_ONLINE_IDENT)
    InertiaIdentify_Init(&g_inertiaIdentify, pInertiaIdentCfg, pMotorParam);
#endif

#if (defined INIT_POS_DETEC)
    InitPosDetec_Init(&g_initPosDetecStruct, &g_initPosPolar);
#endif

    g_hallSelfLearnFinishFlag = 0;

    g_flashTaskFinish = 0;
    g_flashOperationStatus = STATUS_UNSUPPORTED;
#endif
}

/*!
* @brief Foc control parameters initialization, executed in the Stop state and the first Power on.
*
* @param[in] none
* @return none
*/
void FOC_Parameters_Initialize_M2(void)
{
#if (defined MOTOR_TWO_TYPE)

    FOC_CtrlVarsInit(&g_focVarsCtrl_M2);
    Clear_CurrentSample(&g_adcTransform_M2);

    MC_SpeedTrqInit_M2();
    ASR_SpeedCommandInit(pFocVarsCfg_M2, &g_speedCommand_M2, pTransformCfg_M2);
#if (defined POSITION_LOOP_CTRL_M2)
    MC_PositionInit(&g_pidApr_M2, pPidCfg_M2);
    APR_PositionCommandInit_M2(pFocVarsCfg_M2, &g_positionCommand_M2);
#endif
#if (defined MT_SPEED_MEASURE_M2)
    Encoder_MT_Init_M2(&g_encoderSpeed_M2);
#endif
    Asr_SpeedPLLInit(&g_pidPll_M2, pFocVarsCfg_M2, pPllCfg_M2);

#if (defined SCURVE_SPEED_RAMP_M2)
    SCurve_InitProcess(&g_sCurve_M2);
#endif

    /* Sensorless FOC */
#if ((defined SENSORLESS_M2) ||  (defined ENCODER_SENSORS_M2))
    StartUp_Ctrl_Init_M2(&g_startUp_M2, &g_startUpCurve_M2, &g_STC_M2);
    /* Slide Mode observer */
    #if (defined SMC_OBSERVE_M2)
    Smc_ParamInit(&g_motorParamPu_M2, pFocVarsCfg_M2, pSmoObserveCfg_M2, &g_smcEstimator_M2);
    /* Flux observer */
    #elif (defined FLUX_OBSERVE_M2)
    FluxObserver_Init(&g_fluxObserver_M2, &g_motorParamPu_M2, pFocVarsCfg_M2);
    #elif (defined MRAS_OBSERVE_M2)
    Mras_Init(&g_mrasObserver_M2, &g_motorParamPu_M2, pMrasObserveCfg_M2);
    #else
    #endif

    /* High frequency injection */
    #if (defined HIGH_FREQUENCE_INJECTION_M2)
    Hfi_Init(&g_hfiCtrl_M2, &g_pidHfi_M2, pHighFreqInjCfg_M2, pFocVarsCfg_M2);
    #endif

    /* Rotor initial position identification by Pulse injection */
    #if (defined PULSE_INJECTION_M2)
    PulseInject_Init(&g_pulseInject_M2);
    #endif
#endif

//#if (defined FOC_ACIM_M2)
//    Acim_FluxObserverInit(&g_acimFocCtrl, &g_STC);
//    StartUp_Ctrl_Init(&g_startUp, &g_startUpCurve, &g_STC);
//#endif

    /* Motor protection */
    Init_Protection(&g_protector_M2);

    /* Is over load fall frequency */
#if (defined OVER_LOAD_PROTECT_ENABLE_M2)
    Is_OverLoad_Init(&g_overLoadPro_M2);
#endif

    /* Power decrease limit speed target command */
#if (defined POWER_DECREASE_PROTECT_ENABLE_M2)
    PowerDecrease_init(&g_powerDecrease_M2);
#endif

    /* Speed calculation parameters initialization */
    ASR_RotationSpeedInit_M2(&g_speedCommand_M2);

    /* Short circuit brake */
#if (SHORT_BRAKE_CTRL_M2 > 0)
    ShortBrake_Init(&g_shortBrake_M2);
#endif
    /* Electric brake */
#if (defined ELEC_BRAKE_ENABLE_M2)
    ElecBrake_Init(&g_elecBrake_M2, pElecBrakeCfg_M2);
#endif

    /* Power and DC current estimation */
#if (defined POWER_DCCUR_EST_ENABLE_M2)
    PowerEst_Init(&g_powerEst_M2);
    DcCurEst_Init(&g_dcCurEst_M2);
#endif

    /* DC bus current limit */
#if (defined DCCUR_LIMIT_ENABLE_M2)
    DcCurLimit_Init(&g_dcCurLimit_M2);
#endif

    /* Power limit */
#if (defined POWER_LIMIT_ENABLE_M2)
    PowerLimit_Init(&g_powerLimit_M2);
#endif

    /* Flux weakening */
#if (defined FLUX_WEAKENING_M2)
    Fluxweak_Init(&g_pidFw_M2, &g_fluxWeak_M2, pFluxWeakCfg_M2);
#endif

    /* MTPA, maximum torque per ampere */
#if (defined MTPA_ENABLE_M2)
    Mtpa_ParamInit(&g_thetaMtpaComp_M2, &g_motorParamPu_M2);
#endif
    /* Hall FOC */
#if (defined HALL_SENSORS_M2)
    HallSensor_Init(GetHallStatus(PWM_MODULE_HALL_MOTOR2), &g_hall_M2, &g_focVars_M2);
#endif

    /* Motor parameters identification */
    if (g_paramIdentifyCtrl_M2.initCtrl == 1)
    {
        ParamIdentify_Init(&g_paramIdentify_M2);
//#if (defined FOC_PMSM_M2)
//        PmsmId_LdqParamInit(&g_paramIdentify, &g_sigPhaseShift);
//#elif (defined FOC_ACIM_M2)
//        AcimId_ParamInit(&g_paramIdentify, &g_sigPhaseShift);
//#endif
    }
    StatorResistanceACRInit(&g_pidAcrAlfaRsId_M2, &g_pidAcrBetaRsId_M2);

#if (defined TAIL_HEAD_WIND_M2)
    InitSpeed_Judge_Init(&g_initSpeedJudge_M2);
#endif

#ifdef OVER_VOL_SUPPRESS_M2
    OverVoltageSuppressInit(&g_overVoltSupp_M2, &g_pidUdcLimit_M2, &defPID_M2, &g_STC_M2);
#endif

#if (defined LOAD_TORQUE_ESTIMATE_M2)
    TrqLoadEst_Init(&g_loadTorqueEst_M2, pLoadTorqueCfg_M2, &g_motorParamPu_M2);
#endif
#if (defined INIT_POS_DETEC_M2)
    InitPosDetec_Init(&g_initPosDetecStruct_M2, &g_initPosPolar_M2);
#endif

#if (defined INERTIA_ONLINE_IDENT_M2)
    InertiaIdentify_Init(&g_inertiaIdentify_M2, pInertiaIdentCfg_M2, pMotorParam_M2);
#endif

    g_hallSelfLearnFinishFlag_M2 = 0;

    g_flashTaskFinish_M2 = 0;
    g_flashOperationStatus_M2 = STATUS_UNSUPPORTED;
#endif
}

/*!
* @brief Foc control initialization, include control parameters and peripheral initialization.
*
* @param[in] none
* @return none
*/
void MC_Initialize(void)
{
#if (defined MOTOR_ONE_TYPE)
    #if ((defined HALL_SENSORS) || (defined ENCODER_HALL_SENSORS))
    FOC_Hall_Initialize();
    #endif
#endif
#if (defined MOTOR_TWO_TYPE)
    #if ((defined HALL_SENSORS_M2) || (defined ENCODER_HALL_SENSORS_M2))
    FOC_Hall_Initialize_M2();
    #endif
#endif

#if (defined MOTOR_ONE_TYPE)
    #if (defined FOC_SINE_WAVE)
    FOC_Parameters_Initialize();
    #elif (defined VF_CONTROL)
    Vf_ParamInit();
    #endif

#endif
#if (defined MOTOR_TWO_TYPE)
    #if (defined FOC_SINE_WAVE_M2)
    FOC_Parameters_Initialize_M2();
    #elif (defined VF_CONTROL_M2)
    Vf_ParamInit();
    #endif
#endif

    FOC_Peripheral_Initialize();
}

/*!
* @brief LED light turn on or off.
*
* @param[in] ledStatus: LED turn on or off
* @return none
*/
void LED1Display(uint8_t ledStatus)
{
    GPIO_DRV_WritePin(GPIOE, 0U, ledStatus);
}

/*!
* @brief LED light turn on or off.
*
* @param[in] ledStatus: LED turn on or off
* @return none
*/
void LED2Display(uint8_t ledStatus)
{
    GPIO_DRV_WritePin(GPIOE, 1U, ledStatus);
}

/*!
* @brief LED light turn on or off.
*
* @param[in] ledStatus: LED turn on or off
* @return none
*/
void LED3Display(uint8_t ledStatus)
{
    GPIO_DRV_WritePin(GPIOA, 10U, ledStatus);
}

/*!
* @brief Torque control with speed limit function.
*
* @param[in] torque: pointer to TORQUE_COMMAND structure
* @param[in] speed: pointer to SPEED_RAMP_HANDLE structure
* @param[in] asr: pointer to speed PID_REGULATOR structure
* @param[in] acr: pointer to q axis current PID_REGULATOR structure
* @return none
*/
void TorqueCtrlMode(TORQUE_COMMAND *torque, SPEED_RAMP_HANDLE *speed, PID_REGULATOR *asr, PID_REGULATOR *acr)
{
#if (defined MOTOR_ONE_TYPE)
    if (defFocVars.controlMode == SPEED_CONTROL_MODE)
    {
        torque->speedFlag = 1;
        if (torque->torqueFlag == 1)
        {
            torque->torqueFlag = 0;
            TroqueCmdSet(defPID.maxSpeed, asr->pPidInput);
            speed->speedTarget = asr->pPidInput->fbkPu;
            speed->speedTargetRamp = asr->pPidInput->fbkPu;
            asr->pPidOutput->uiPu = acr->pPidInput->refPu;
        }
        else
        {
        }
    }
    else if (defFocVars.controlMode == TORQUE_CONTROL_WITH_SPDLIMIT)
    {
        torque->torqueFlag = 1;
        if (torque->speedFlag == 0)
        {
            speed->speedTarget = torque->speedLimit * pFocVarsCfg->motorDir;
            torque->iqTarget = Math_Sat(torque->iqTarget, torque->iqMax, torque->iqMin);
            GIVE_SLOPE(torque->iqTarget, torque->iqTargetRamp, torque->iqChangeStep);
            torque->iqTargetRamp = Math_Sat(torque->iqTargetRamp, torque->iqMax, torque->iqMin);
            TroqueCmdSet(torque->iqTargetRamp, asr->pPidInput);
        }
        else
        {
            torque->speedFlag = 0;
            torque->iqTarget = asr->pPidOutput->outPu;
            torque->iqTargetRamp = asr->pPidOutput->outPu;
            acr->pPidInput->refPu = asr->pPidOutput->outPu;
        }
    }
    else
    {
    }
#endif
}

/*!
* @brief Torque control with speed limit function.
*
* @param[in] torque: pointer to TORQUE_COMMAND structure
* @param[in] speed: pointer to SPEED_RAMP_HANDLE structure
* @param[in] asr: pointer to speed PID_REGULATOR structure
* @param[in] acr: pointer to q axis current PID_REGULATOR structure
* @return none
*/
void TorqueCtrlMode_M2(TORQUE_COMMAND *torque, SPEED_RAMP_HANDLE *speed, PID_REGULATOR *asr, PID_REGULATOR *acr)
{
#if (defined MOTOR_TWO_TYPE)
    if (defFocVars_M2.controlMode == SPEED_CONTROL_MODE)
    {
        torque->speedFlag = 1;
        if (torque->torqueFlag == 1)
        {
            torque->torqueFlag = 0;
            TroqueCmdSet(defPID_M2.maxSpeed, asr->pPidInput);
            speed->speedTarget = asr->pPidInput->fbkPu;
            speed->speedTargetRamp = asr->pPidInput->fbkPu;
            asr->pPidOutput->uiPu = acr->pPidInput->refPu;
        }
        else
        {
        }
    }
    else if (defFocVars_M2.controlMode == TORQUE_CONTROL_WITH_SPDLIMIT)
    {
        torque->torqueFlag = 1;
        if (torque->speedFlag == 0)
        {
            speed->speedTarget = torque->speedLimit * pFocVarsCfg_M2->motorDir;
            torque->iqTarget = Math_Sat(torque->iqTarget, torque->iqMax, torque->iqMin);
            GIVE_SLOPE(torque->iqTarget, torque->iqTargetRamp, torque->iqChangeStep);
            torque->iqTargetRamp = Math_Sat(torque->iqTargetRamp, torque->iqMax, torque->iqMin);
            TroqueCmdSet(torque->iqTargetRamp, asr->pPidInput);
        }
        else
        {
            torque->speedFlag = 0;
            torque->iqTarget = asr->pPidOutput->outPu;
            torque->iqTargetRamp = asr->pPidOutput->outPu;
            acr->pPidInput->refPu = asr->pPidOutput->outPu;
        }
    }
    else
    {
    }
#endif
}

/*!
* @brief Inertia identify calculation function of PMSM.
*
* @param[in] none
* @return none
*/
void InertiaIdentify_Calc(void)
{
#if (defined MOTOR_ONE_TYPE)
#if (defined INERTIA_ONLINE_IDENT)
#if (defined SENSORLESS)
    if (g_startUpCtrl.startUpFlag >= 6)
    {
        #if (defined INERTIA_IDENT_MRAS)
        InertiaIdentify_MRAS(&g_inertiaIdentify, &g_motorParamPu, g_asrSpeedEstPIOut.outPu, g_focVarsCtrl.iqFdb);
        #else
        InertiaIdentify_FFLS(&g_inertiaIdentify, &g_motorParamPu, g_asrSpeedEstPIOut.outPu, g_focVarsCtrl.iqFdb);
        #endif  /* end of INERTIA_IDENT_MRAS */
    }
#elif (defined HALL_SENSORS)
    if (Get_McStatus() == RUN)
    {
        #if (defined INERTIA_IDENT_MRAS)
        InertiaIdentify_MRAS(&g_inertiaIdentify, &g_motorParamPu, g_hallSensor.estFre, g_focVarsCtrl.iqFdb);
        #else
        InertiaIdentify_FFLS(&g_inertiaIdentify, &g_motorParamPu, g_hallSensor.estFre, g_focVarsCtrl.iqFdb);
        #endif  /* end of INERTIA_IDENT_MRAS */
    }
#elif ((defined ENCODER_SENSORS) || (defined ENCODER_HALL_SENSORS))
    if (Get_McStatus() == RUN)
    {
        #if (defined INERTIA_IDENT_MRAS)
        InertiaIdentify_MRAS(&g_inertiaIdentify, &g_motorParamPu, g_rotationalSpeed.estSpeed, g_focVarsCtrl.iqFdb);
        #else
        InertiaIdentify_FFLS(&g_inertiaIdentify, &g_motorParamPu, g_rotationalSpeed.estSpeed, g_focVarsCtrl.iqFdb);
        #endif  /* end of INERTIA_IDENT_MRAS */
    }
#else
#endif  /* end of SENSORLESS */

#endif  /* end of INERTIA_ONLINE_IDENT */
#endif  /* end of MOTOR_ONE_TYPE */
}

/*!
* @brief Inertia identify calculation function of PMSM.
*
* @param[in] none
* @return none
*/
void InertiaIdentify_Calc_M2(void)
{
#if (defined MOTOR_TWO_TYPE)
#if (defined INERTIA_ONLINE_IDENT_M2)
#if (defined SENSORLESS_M2)
    if (g_startUpCtrl.startUpFlag >= 6)
    {
        #if (defined INERTIA_IDENT_MRAS_M2)
        InertiaIdentify_MRAS(&g_inertiaIdentify_M2, &g_motorParamPu_M2, g_asrSpeedEstPIOut_M2.outPu, g_focVarsCtrl_M2.iqFdb);
        #else
        InertiaIdentify_FFLS(&g_inertiaIdentify_M2, &g_motorParamPu_M2, g_asrSpeedEstPIOut_M2.outPu, g_focVarsCtrl_M2.iqFdb);
        #endif  /* end of INERTIA_IDENT_MRAS_M2 */
    }
#elif (defined HALL_SENSORS_M2)
    if (Get_McStatus_M2() == RUN)
    {
        #if (defined INERTIA_IDENT_MRAS_M2)
        InertiaIdentify_MRAS(&g_inertiaIdentify_M2, &g_motorParamPu_M2, g_hallSensor_M2.estFre, g_focVarsCtrl_M2.iqFdb);
        #else
        InertiaIdentify_FFLS(&g_inertiaIdentify_M2, &g_motorParamPu_M2, g_hallSensor_M2.estFre, g_focVarsCtrl_M2.iqFdb);
        #endif  /* end of INERTIA_IDENT_MRAS_M2 */
    }
#elif ((defined ENCODER_SENSORS_M2) || (defined ENCODER_HALL_SENSORS_M2))
    if (Get_McStatus_M2() == RUN)
    {
        #if (defined INERTIA_IDENT_MRAS_M2)
        InertiaIdentify_MRAS(&g_inertiaIdentify_M2, &g_motorParamPu_M2, g_rotationalSpeed_M2.estSpeed, g_focVarsCtrl_M2.iqFdb);
        #else
        InertiaIdentify_FFLS(&g_inertiaIdentify_M2, &g_motorParamPu_M2, g_rotationalSpeed_M2.estSpeed, g_focVarsCtrl_M2.iqFdb);
        #endif  /* end of INERTIA_IDENT_MRAS_M2 */
    }
#else
#endif  /* end of SENSORLESS_M2 */

#endif  /* end of INERTIA_ONLINE_IDENT_M2 */
#endif  /* end of MOTOR_TWO_TYPE */
}

/*!
* @brief The Main state machine, used to set the running state of the control system.
*
* @param[in] none
* @return none
*/
void MainStateMachine(void)
{
#if (defined MOTOR_ONE_TYPE)
    PullOut_McBufStatus();
    switch (Get_McStatus())
    {
    /* Power on, jump to STOP after completion */
    case POWER_ON:
        Stop_Motor(&g_startUpCtrl, &g_focVars);

        FOC_Parameters_Initialize();
        ACR_TorqueCommandInit(&g_torqueCommand);
        ASR_SpeedCommandSet(pFocVarsCfg, &g_speedCommand);
        /* must behind adc init, check current sampling offset*/
        Current_Sensor_Check(&g_adcSampleReal, &g_protector);

#if (defined DATA_ACCESS_FLASH_ENABLE)
        DisableInterrupts;
        g_flashOperationStatus = Eflash_DataAccess_DefaultSave(&g_dataAccessFlash, g_VarAccessArray);
        if (g_flashOperationStatus == STATUS_SUCCESS)
        {
            Eflash_DataAccess_PowerOnRecovery(&g_dataAccessFlash, g_VarAccessArray);
            Eflash_DataAccessEflashInit(&g_dataAccessFlash);
            Set_McStatus(IDLE);
            EnableInterrupts;
        }
#else
        Set_McStatus(IDLE);
#endif
    break;

    /* Idle, system standby */
    case IDLE:
        Current_Sensor_Check(&g_adcSampleReal, &g_protector);
        FOC_Parameters_Initialize();

        #if (defined DATA_ACCESS_FLASH_ENABLE)
        if (g_dataAccessFlash.operateEnable == 1)
        {
            Set_McStatus(FLASH_TASK);
        }
        #endif

        #if (defined STARTUP_CHECK_RESTART_ENABLE)
        Foc_RestartInit(&g_restart);
        #else
        #endif

        #if (defined SENSORLESS)
            #if (defined HIGH_FREQUENCE_INJECTION)
            ASR_SetSpeedTargetRamp_Pu(&g_speedCommand, (HFI_INIT_FREQ * pFocVarsCfg->motorDir));
            #else
            ASR_SetSpeedTargetRamp_Pu(&g_speedCommand, (Math_IQ(OPEN_LOOP_SPEED_HZ * POLE_PAIR_NUM / BASE_FREQ) * pFocVarsCfg->motorDir));
            #endif
        #endif
    break;

    /* Start brake, Short circuit brake */
    case START_BRAKE:
        ShortBrake_RunCaseBrake(&g_sBrake, &g_focVars);
        if (g_shortBrake.subStatus == (uint8_t)CONFIG_RESET)
        {
            PWM_Disable(PWM_MODULE_MOTOR1);
            ShortBrake_Init(&g_shortBrake);
            Set_McStatus(START);
        }
    break;

    /* Start, according to the configuration, different startup schemes are implemented */
    case START:
//#if (defined FOC_PMSM)
#if ((defined HALL_SENSORS) || (defined ENCODER_HALL_SENSORS))
        ASR_SetSpeedTargetRamp_Pu(&g_speedCommand, (Math_IQ(0.05) * pFocVarsCfg->motorDir));

        Set_McStatus(RUN);

        StartUp_Motor(&g_startUpCtrl, &g_focVars);
#endif

#if ((defined SENSORLESS) || (defined ENCODER_SENSORS))
        if (g_startUp.pStCtrl->alignState)
        {
            Set_McStatus(RUN);
            StartUp_Motor(&g_startUpCtrl, &g_focVars);
        }
        else
        {
            #if (defined SENSORLESS)
            #if (defined PULSE_INJECTION)
            Set_McStatus(ANGLE_JUDGE);
            #elif (defined INIT_POS_DETEC)
            Set_McStatus(INIT_POS);
            #else
            StartUp_Motor(&g_startUpCtrl, &g_focVars);
            Set_McStatus(ALIGN);
            #endif
            #endif
            #if (defined ENCODER_SENSORS)
            StartUp_Motor(&g_startUpCtrl, &g_focVars);
            Set_McStatus(ALIGN);
            #endif
        }
#endif

//#elif (defined FOC_ACIM)
//    StartUp_Motor();
//    g_mcStatus = ALIGN;
//#endif

    break;

    #if (defined PULSE_INJECTION)
    case ANGLE_JUDGE:
        if (EXE_SUCCESS == PulseInject_Process(PWM_MODULE_MOTOR1, &g_pulseInject, &g_startUpCtrl))
        {
            StartUp_Motor(PWM_MODULE_MOTOR1, &g_startUpCtrl, &g_focVarsCtrl);
            Set_McStatus(RUN);
        }
        else if (EXE_FAIL == PulseInject_Process(PWM_MODULE_MOTOR1, &g_pulseInject, &g_startUpCtrl))
        {
            StartUp_Motor(PWM_MODULE_MOTOR1, &g_startUpCtrl, &g_focVarsCtrl);
            Set_McStatus(ALIGN);
        }
        else
        {
        }
    break;
    #endif

    #if (defined TAIL_HEAD_WIND)
    case SPEED_JUDGE:
    if (Get_InitSpeed_Judge_State(&g_initSpeedJudge))
    {
        Set_McStatus(FLY_RUN);
        Clear_InitSpeed_Judge_State(&g_initSpeedJudge);
    }
    break;

    case FLY_RUN:
    g_mcStatus = Tail_Head_Wind_Process(PWM_MODULE_MOTOR1, &g_startUpCtrl, &g_focVarsCtrl, &g_speedCommand, &g_initSpeedJudge);
    break;
    #endif

    /* Align, obtain the initial position of rotor */
    case ALIGN:
//#if (defined FOC_PMSM)
#if (defined ENCODER_SENSORS)
        Angle_Prepositioning_Process(&g_focVars, &g_startUp);
        if (g_startUp.pStCtrl->alignState)
        {
            Encoder_GetAlignOffset(&g_encoderSensor, &g_STC);
            Set_McStatus(RUN);
        }
#else

    #if (defined HIGH_FREQUENCE_INJECTION)
        Set_McStatus(RUN);
    #else
        #if (defined ALIGNMENT_TWICE)
        Angle_Alignment_Process(&g_focVars, &g_startUp);
        #endif
        #if (defined ALIGNMENT_ONCE)
        Angle_Prepositioning_Process(&g_focVars, &g_startUp);
        #endif

        if (g_startUp.pStCtrl->alignState)
        {
            Set_McStatus(RUN);
        }
    #endif
#endif
//#elif (defined FOC_ACIM)
//    Acim_FocPreExcitStartup(&g_acimFocCtrl, &g_focVars, 1000);

//#endif
    break;

    #if (defined INIT_POS_DETEC)
    case INIT_POS:
        if (InitPosDetec_1msTask(PWM_MODULE_MOTOR1, &g_initPosDetecStruct, pInitPosDetecCfg, &g_STC))
        {
            if (g_focVars.pCfg->motorDir == 1)
            {
                g_startUpCtrl.startUpElecTheta = g_initPosDetecStruct.initPosDetec->estTheta - 16384;
            }
            else
            {
                g_startUpCtrl.startUpElecTheta = g_initPosDetecStruct.initPosDetec->estTheta + 16384;
            }
            StartUp_Motor(PWM_MODULE_MOTOR1, &g_startUpCtrl, &g_focVarsCtrl);
            Set_McStatus(RUN);
        }
    break;
    #endif

    /* Run, according to the configuration, different control schemes are implemented */
    case RUN:
//#if (defined FOC_PMSM)

#if ((defined HALL_SENSORS) || (defined ENCODER_HALL_SENSORS))
        if (defHallSensor.hallSelfLearnEnable == DISABLE)
        {
        #if (!defined POSITION_LOOP_CTRL)
            ASR_SpeedLoop_Process(&g_speedCommand, &g_pidAsr, &g_focVarsCtrl);
        #endif
        }
#elif (defined SENSORLESS)
    #if (defined IF_STARTUP) || (defined DIRECT_STARTUP)|| (defined VF_STARTUP)
        StartUp_Ctrl_1ms(&g_STC, &g_startUp, &g_startUpCurve, &g_pidAsr, &g_pidAcrQ, pPidCfg, &g_speedCommand);
    #endif
//    Acr_GetAvgIs(&g_focVarsCtrl);
#else
    #if (!defined POSITION_LOOP_CTRL)
        ASR_SpeedLoop_Process(&g_speedCommand, &g_pidAsr, &g_focVarsCtrl);
    #endif
#endif

#if (defined ELEC_BRAKE_ENABLE)
        ElecBrake_Calc(PWM_MODULE_MOTOR1, g_speedCommand.speedFbk, &g_eBrake, &g_STC, &g_acrCurIdPIOut, &g_acrCurIqPIOut);
#endif

#if (defined FLUX_WEAKENING)
        if (g_startUpCtrl.startUpFlag == 6)
        {
            g_fluxWeak.vdq = g_focVarsCtrl.vdqFilter;
            Fluxweak_CurRefCalc(&g_fluxWeak, &g_focVarsCtrl, &g_pidFw, &g_asrSpeedPIOut);
        }
//        Fluxweak_CurRef_Lookup(g_asrSpeedPIIn.refPu, g_asrSpeedPIOut.outPu, &g_focVarsCtrl, &g_fluxWeak);
#endif

#if (defined MTPA_ENABLE)
        Mtpa_Compensator(g_focVarsCtrl.ialphabeta, &g_thetaMtpaComp);
#endif

#if (defined SAVE_ENERGY)
    #if ((defined HALL_SENSORS) || (defined ENCODER_HALL_SENSORS))
        g_focVarsCtrl.idRef = EnergySave(&g_energySave, pEnergySaveCfg);
    #elif (defined SENSORLESS)
        if (g_startUpCtrl.startUpFlag == 6)
        {
            g_focVarsCtrl.idRef = EnergySave(&g_energySave, pEnergySaveCfg);
        }
    #endif
#endif

#if (defined STARTUP_CHECK_RESTART_ENABLE)
        g_mcStatus = Foc_RestartJudgeProcess(PWM_MODULE_MOTOR1, g_mcStatus, &g_protector, &g_restart, &g_speedCommand, &g_startUp);
#endif

#if (defined OVER_LOAD_PROTECT_ENABLE)
        if (g_startUpCtrl.startUpFlag == 6)
        {
            Is_OverLoad_FallFreq(&g_overLoadPro, &g_focVars, &g_speedCommand, &g_protector);
        }
#endif
        TorqueCtrlMode(&g_torqueCommand, &g_speedCommand, &g_pidAsr, &g_pidAcrQ);
//#elif (defined FOC_ACIM)
//#if (defined SENSOR_ACIM)
//    ASR_SpeedLoop_Process(&g_speedCommand, &g_pidAsr, &g_focVarsCtrl);
//#elif (defined SENSORLESS_ACIM)
//    Acim_Startup_1ms(&g_focVars, &g_startUp);
//#else
//#endif
//#if (defined ELEC_BRAKE_ENABLE)
//    ElecBrake_Calc(g_speedCommand.speedFbk, &g_elecBrake, pElecBrakeCfg, &g_STC, &g_acrCurIdPIOut, &g_acrCurIqPIOut);
//#endif
//    TorqueCtrlMode(&g_torqueCommand, &g_speedCommand, &g_pidAsr, &g_pidAcrQ);
//#endif

#if (defined SPEED_ADJUST_KNOB)
        ASR_GetSpeedRampWithKnob(&g_speedCommand);
#endif

    break;

    /* Slow_Down, decelerate the motor to a certain speed and then stop the motor */
    case SLOW_DOWN:
        g_speedCommand.speedMin = pFocVarsCfg->stopSpeed;
        ASR_SetSpeedTarget_Pu(&g_speedCommand, (pFocVarsCfg->stopSpeed * pFocVarsCfg->motorDir));

//#if (defined FOC_PMSM)
#if ((defined HALL_SENSORS) || (defined ENCODER_HALL_SENSORS))
        ASR_SpeedLoop_Process(&g_speedCommand, &g_pidAsr, &g_focVarsCtrl);
#elif (defined SENSORLESS)
        StartUp_Ctrl_1ms(&g_STC, &g_startUp, &g_startUpCurve, &g_pidAsr, &g_pidAcrQ, pPidCfg, &g_speedCommand);
#else
        ASR_SpeedLoop_Process(&g_speedCommand, &g_pidAsr, &g_focVarsCtrl);
#endif

#ifdef OVER_VOL_SUPPRESS
        OverVoltageSuppressProcess(g_focVarsCtrl.vdcPu, &g_overVoltSupp, &g_pidUdcLimit, &g_asrSpeedPIIn);
#endif

//#elif (defined FOC_ACIM)
//#if (defined SENSOR_ACIM)
//    ASR_SpeedLoop_Process(&g_speedCommand, &g_pidAsr, &g_focVarsCtrl);
//#elif (defined SENSORLESS_ACIM)
//    Acim_Startup_1ms(&g_focVars, &g_startUp);
//#else
//#endif
//#endif

#if (defined SENSORLESS)
    #if (defined OPEN_LOOP)
        if (Math_Qabs(g_speedCommand.speedTargetRamp) <= pFocVarsCfg->stopSpeed)
        {
            Set_McStatus(STOP);

            PWM_Disable(PWM_MODULE_MOTOR2);
        }
    #elif (defined CLOSE_LOOP)
        if (Math_Qabs(g_speedCommand.speedFbk) <= pFocVarsCfg->stopSpeed)
        {
            Set_McStatus(STOP);

            PWM_Disable(PWM_MODULE_MOTOR1);
        }
    #else
    #endif
#endif

    break;

    /* Stop brake, Short circuit brake */
    case STOP_BRAKE:
        ShortBrake_RunCaseBrake(&g_sBrake, &g_focVars);
        if (g_shortBrake.subStatus == (uint8_t)CONFIG_RESET)
        {
            PWM_Disable(PWM_MODULE_MOTOR1);
            ShortBrake_Init(&g_shortBrake);
            Set_McStatus(STOP);
        }
    break;

    /* Stop, stop the motor and clear the control parameters, then go to Idle state */
    case STOP:
        Stop_Motor(&g_startUpCtrl, &g_focVars);

        FOC_Parameters_Initialize();
        ACR_TorqueCommandInit(&g_torqueCommand);
        ASR_SpeedCommandSet(pFocVarsCfg, &g_speedCommand);
        PWM_Update(PWM_PERIOD_VALUE - 1, PWM_PERIOD_VALUE - 1, PWM_PERIOD_VALUE - 1, &g_focVars);

        Set_McStatus(IDLE);
    break;

    /* Identify, perform motor parameter identification function */
    case IDENTIFY:
        ParamIdentify(&g_paramIdentify, &g_STC, pHWBoardParam);
        if (g_paramIdentifyCtrl.initCtrl == 1)
        {
            ParameterIdentifySaveProcess(&g_paramIdentify, &g_paramIdentifyBuffer);
            Set_McStatus(IDLE);
        }
    break;

    /* Fault, stop the motor. If it is expected to run again, it is necessary to clear the fault and enter the stop state */
    case FAULT:
        Stop_Motor(&g_startUpCtrl, &g_focVars);
        if (g_protector.ErrFlag.all == 0)
        {
//            Set_McStatus(STOP);
        }

    break;

    /* Restart, re-do FOC start. when motor start process is abnormal, restart the motor */
#if (defined STARTUP_CHECK_RESTART_ENABLE)
    case RESTART:
        Foc_RestartProcess(PWM_MODULE_MOTOR1, &g_STC, &g_restart);
        Set_McStatus(START);
    break;
#endif

    /* flash task, do flash write operation to save user debug value of variables, and flash read operation to recovery flash data to variables */
#if (defined DATA_ACCESS_FLASH_ENABLE)
    case FLASH_TASK:
        DisableInterrupts;
        g_flashOperationStatus = Eflash_DataAccessEflashProcess(&g_dataAccessFlash, g_VarAccessArray);
        if (g_flashOperationStatus == STATUS_SUCCESS)
        {
            Set_McStatus(IDLE);

            Eflash_DataAccessEflashInit(&g_dataAccessFlash);
            g_flashTaskFinish = 1;
            EnableInterrupts;
        }
    break;
#endif

    /* Hall sensor self learn, do self learning to get Hall sensor installation information */
#if (defined HALL_SENSORS)
    case HALL_SELFLEARN:
        StartUp_Motor(&g_startUpCtrl, &g_focVars);

        if (defHallSensor.hallSelfLearnEnable == DISABLE)
        {
            Stop_Motor(&g_startUpCtrl, &g_focVars);
            Set_McStatus(STOP);
            HallSelfLearnResultInit(&g_hallSelfLearnBuffer);
        }
        if (g_hallSelfLearnFinishFlag == 1)
        {
            HallSelfLearnSaveProcess(&g_hallSelfLearnBuffer, &g_hall);
            defHallSensor.hallSelfLearnEnable = DISABLE;

            Stop_Motor(&g_startUpCtrl, &g_focVars);
            Set_McStatus(STOP);
        }
    break;
#endif

    default:
    break;
    }
#endif
}

/*!
* @brief The Main state machine of Motor_2, used to set the running state of the control system.
*
* @param[in] none
* @return none
*/
void MainStateMachine_M2(void)
{
#if (defined MOTOR_TWO_TYPE)
    PullOut_McBufStatus_M2();
    switch (Get_McStatus_M2())
    {
    /* Power on, jump to STOP after completion */
    case POWER_ON:
        Stop_Motor(&g_startUpCtrl_M2, &g_focVars_M2);

        FOC_Parameters_Initialize_M2();
        ACR_TorqueCommandInit_M2(&g_torqueCommand_M2);
        ASR_SpeedCommandSet_M2(pFocVarsCfg_M2, &g_speedCommand_M2);
        /* must behind adc init, check current sampling offset*/
        Current_Sensor_Check(&g_adcSampleReal_M2, &g_protector_M2);

#if (defined DATA_ACCESS_FLASH_ENABLE_M2)
        DisableInterrupts;
        g_flashOperationStatus_M2 = Eflash_DataAccess_DefaultSave_M2(&g_dataAccessFlash_M2, g_VarAccessArray_M2);
        if (g_flashOperationStatus_M2 == STATUS_SUCCESS)
        {
            Eflash_DataAccess_PowerOnRecovery_M2(&g_dataAccessFlash_M2, g_VarAccessArray_M2);
            Eflash_DataAccessEflashInit(&g_dataAccessFlash_M2);
            Set_McStatus_M2(IDLE);
            EnableInterrupts;
        }
#else
        Set_McStatus_M2(IDLE);
#endif

    break;

    /* Idle, system standby */
    case IDLE:
    #if (defined DATA_ACCESS_FLASH_ENABLE_M2)
        if (g_dataAccessFlash_M2.operateEnable == 1)
        {
            Set_McStatus_M2(FLASH_TASK);
        }
    #endif

        Current_Sensor_Check(&g_adcSampleReal_M2, &g_protector_M2);
        FOC_Parameters_Initialize_M2();
    #if (defined STARTUP_CHECK_RESTART_ENABLE)
        Foc_RestartInit(&g_restart_M2);
    #else
    #endif

    #if (defined SENSORLESS_M2)
        #if (defined HIGH_FREQUENCE_INJECTION_M2)
        ASR_SetSpeedTargetRamp_Pu(&g_speedCommand_M2, (HFI_INIT_FREQ_M2 * pFocVarsCfg_M2->motorDir));
        #else
        ASR_SetSpeedTargetRamp_Pu(&g_speedCommand_M2, (Math_IQ(OPEN_LOOP_SPEED_HZ_M2 * POLE_PAIR_NUM_M2 / BASE_FREQ_M2) * pFocVarsCfg_M2->motorDir));
        #endif
    #endif

    break;

    /* Start brake, Short circuit brake */
    case START_BRAKE:
        ShortBrake_RunCaseBrake(&g_sBrake_M2, &g_focVars_M2);
        if (g_shortBrake_M2.subStatus == (uint8_t)CONFIG_RESET)
        {
            PWM_Disable(PWM_MODULE_MOTOR2);
            ShortBrake_Init(&g_shortBrake_M2);
            Set_McStatus_M2(START);
        }
    break;

    /* Start, according to the configuration, different startup schemes are implemented */
    case START:
//#if (defined FOC_PMSM_M2)
    #if ((defined HALL_SENSORS_M2) || (defined ENCODER_HALL_SENSORS_M2))
        ASR_SetSpeedTargetRamp_Pu(&g_speedCommand_M2, (Math_IQ(0.05) * pFocVarsCfg_M2->motorDir));
        Set_McStatus_M2(RUN);
        StartUp_Motor(&g_startUpCtrl_M2, &g_focVars_M2);
    #endif

    #if ((defined SENSORLESS_M2) || (defined ENCODER_SENSORS_M2))
        if (g_startUp_M2.pStCtrl->alignState)
        {
            Set_McStatus_M2(RUN);

            StartUp_Motor(&g_startUpCtrl_M2, &g_focVars_M2);
        }
        else
        {
            #if (defined PULSE_INJECTION_M2)
            Set_McStatus_M2(ANGLE_JUDGE);
            #elif (defined INIT_POS_DETEC_M2)
            Set_McStatus_M2(INIT_POS);
            #else

            StartUp_Motor(&g_startUpCtrl_M2, &g_focVars_M2);
            Set_McStatus_M2(ALIGN);
            #endif
        }
    #endif

//#elif (defined FOC_ACIM)
//    StartUp_Motor(PWM_MODULE_MOTOR2, &g_startUpCtrl_M2, &g_focVarsCtrl_M2);
//    Set_McStatus_M2(ALIGN);
//#endif

    break;

    #if (defined PULSE_INJECTION_M2)
    case ANGLE_JUDGE:
        if (EXE_SUCCESS == PulseInject_Process(PWM_MODULE_MOTOR2, &g_pulseInject_M2, &g_startUpCtrl_M2))
        {
            StartUp_Motor(PWM_MODULE_MOTOR2, &g_startUpCtrl_M2, &g_focVarsCtrl_M2);
            Set_McStatus_M2(RUN);
        }
        else if (EXE_FAIL == PulseInject_Process(PWM_MODULE_MOTOR2, &g_pulseInject_M2, &g_startUpCtrl_M2))
        {
            StartUp_Motor(PWM_MODULE_MOTOR2, &g_startUpCtrl_M2, &g_focVarsCtrl_M2);
            Set_McStatus_M2(ALIGN);
        }
        else
        {
        }
    break;
    #endif

    #if (defined TAIL_HEAD_WIND_M2)
    case SPEED_JUDGE:
        if (Get_InitSpeed_Judge_State(&g_initSpeedJudge_M2))
        {
            Set_McStatus_M2(FLY_RUN);
            Clear_InitSpeed_Judge_State(&g_initSpeedJudge_M2);
        }
    break;

    case FLY_RUN:
        g_mcStatus_M2 = Tail_Head_Wind_Process(PWM_MODULE_MOTOR2, &g_startUpCtrl_M2, &g_focVarsCtrl_M2, &g_speedCommand_M2, &g_initSpeedJudge_M2);
    break;
    #endif

    /* Align, obtain the initial position of rotor */
    case ALIGN:
//#if (defined FOC_PMSM)
#if (defined ENCODER_SENSORS_M2)
        Angle_Prepositioning_Process_M2(&g_focVars_M2, &g_startUp_M2);
        if (g_startUp_M2.pStCtrl->alignState)
        {
            Encoder_GetAlignOffset_M2(&g_encoderSensor_M2, &g_STC_M2);
            Set_McStatus_M2(RUN);
        }
#else

    #if (defined HIGH_FREQUENCE_INJECTION_M2)
        Set_McStatus_M2(RUN);
    #else
        #if (defined ALIGNMENT_TWICE_M2)
        Angle_Alignment_Process_M2(&g_focVars_M2, &g_startUp_M2);
        #endif
        #if (defined ALIGNMENT_ONCE_M2)
        Angle_Prepositioning_Process_M2(&g_focVars_M2, &g_startUp_M2);
        #endif

        if (g_startUp_M2.pStCtrl->alignState)
        {
            Set_McStatus_M2(RUN);
        }
    #endif
#endif
//#elif (defined FOC_ACIM)
//    Acim_FocPreExcitStartup(&g_acimFocCtrl, &g_focVars, 1000);

//#endif
    break;

    #if (defined INIT_POS_DETEC_M2)
    case INIT_POS:
        if (InitPosDetec_1msTask(PWM_MODULE_MOTOR2, &g_initPosDetecStruct_M2, pInitPosDetecCfg_M2, &g_STC_M2))
        {
            if (g_focVars_M2.pCfg->motorDir == 1)
            {
                g_startUpCtrl_M2.startUpElecTheta = g_initPosDetecStruct_M2.initPosDetec->estTheta - 16384;
            }
            else
            {
                g_startUpCtrl_M2.startUpElecTheta = g_initPosDetecStruct_M2.initPosDetec->estTheta + 16384;
            }
            StartUp_Motor(PWM_MODULE_MOTOR2, &g_startUpCtrl_M2, &g_focVarsCtrl_M2);
            Set_McStatus_M2(RUN);
        }
    break;
    #endif

    /* Run, according to the configuration, different control schemes are implemented */
    case RUN:
#if ((defined HALL_SENSORS_M2) || (defined ENCODER_HALL_SENSORS_M2))
        if (defHallSensor_M2.hallSelfLearnEnable == DISABLE)
        {
        #if (!defined POSITION_LOOP_CTRL_M2)
            ASR_SpeedLoop_Process(&g_speedCommand_M2, &g_pidAsr_M2, &g_focVarsCtrl_M2);
        #endif
        }
#elif (defined SENSORLESS_M2)
    #if (defined IF_STARTUP_M2) || (defined DIRECT_STARTUP_M2)|| (defined VF_STARTUP_M2)
        StartUp_Ctrl_1ms_M2(&g_STC_M2, &g_startUp_M2, &g_startUpCurve_M2, &g_pidAsr_M2, &g_pidAcrQ_M2, pPidCfg_M2, &g_speedCommand_M2);
    #endif
//    Acr_GetAvgIs(&g_focVarsCtrl);
#else
    #if (!defined POSITION_LOOP_CTRL_M2)
        ASR_SpeedLoop_Process(&g_speedCommand_M2, &g_pidAsr_M2, &g_focVarsCtrl_M2);
    #endif
#endif

#if (defined ELEC_BRAKE_ENABLE_M2)
        ElecBrake_Calc(PWM_MODULE_MOTOR2, g_speedCommand_M2.speedFbk, &g_eBrake_M2, &g_STC_M2, &g_acrCurIdPIOut_M2, &g_acrCurIqPIOut_M2);
#endif

#if (defined FLUX_WEAKENING_M2)
        if (g_startUpCtrl_M2.startUpFlag == 6)
        {
            g_fluxWeak_M2.vdq = g_focVarsCtrl_M2.vdqFilter;
            Fluxweak_CurRefCalc(&g_fluxWeak_M2, &g_focVarsCtrl_M2, &g_pidFw_M2, &g_asrSpeedPIOut_M2);
        }
//        Fluxweak_CurRef_Lookup(g_asrSpeedPIIn_M2.refPu, g_asrSpeedPIOut_M2.outPu, &g_focVarsCtrl_M2, &g_fluxWeak_M2);
#endif

#if (defined MTPA_ENABLE_M2)
        Mtpa_Compensator(g_focVarsCtrl_M2.ialphabeta, &g_thetaMtpaComp_M2);
#endif

#if (defined SAVE_ENERGY_M2)
    #if ((defined HALL_SENSORS_M2) || (defined ENCODER_HALL_SENSORS_M2))
        g_focVarsCtrl_M2.idRef = EnergySave(&g_energySave_M2, pEnergySaveCfg_M2);
    #elif (defined SENSORLESS_M2)
        if (g_startUpCtrl_M2.startUpFlag == 6)
        {
            g_focVarsCtrl_M2.idRef = EnergySave(&g_energySave_M2, pEnergySaveCfg_M2);
        }
    #endif
#endif

#if (defined STARTUP_CHECK_RESTART_ENABLE)
        g_mcStatus_M2 = Foc_RestartJudgeProcess(PWM_MODULE_MOTOR2, g_mcStatus_M2, &g_protector_M2, &g_restart_M2, &g_speedCommand_M2, &g_startUp_M2);
#endif

#if (defined OVER_LOAD_PROTECT_ENABLE)
        if (g_startUpCtrl_M2.startUpFlag == 6)
        {
            Is_OverLoad_FallFreq(&g_overLoadPro_M2, &g_focVars_M2, &g_speedCommand_M2, &g_protector_M2);
        }
#endif
        TorqueCtrlMode_M2(&g_torqueCommand_M2, &g_speedCommand_M2, &g_pidAsr_M2, &g_pidAcrQ_M2);

#if (defined SPEED_ADJUST_KNOB_M2)
        ASR_GetSpeedRampWithKnob_M2(&g_speedCommand_M2);
#endif
    break;

    /* Slow_Down, decelerate the motor to a certain speed and then stop the motor */
    case SLOW_DOWN:
        g_speedCommand_M2.speedMin = pFocVarsCfg_M2->stopSpeed;
        ASR_SetSpeedTarget_Pu(&g_speedCommand_M2, (pFocVarsCfg_M2->stopSpeed * pFocVarsCfg_M2->motorDir));

#if ((defined HALL_SENSORS_M2) || (defined ENCODER_HALL_SENSORS_M2))
        ASR_SpeedLoop_Process(&g_speedCommand_M2, &g_pidAsr_M2, &g_focVarsCtrl_M2);
#elif (defined SENSORLESS_M2)
        StartUp_Ctrl_1ms_M2(&g_STC_M2, &g_startUp_M2, &g_startUpCurve_M2, &g_pidAsr_M2, &g_pidAcrQ_M2, pPidCfg_M2, &g_speedCommand_M2);
#else
        ASR_SpeedLoop_Process(&g_speedCommand_M2, &g_pidAsr_M2, &g_focVarsCtrl_M2);
#endif

#ifdef OVER_VOL_SUPPRESS_M2
        OverVoltageSuppressProcess(g_focVarsCtrl_M2.vdcPu, &g_overVoltSupp_M2, &g_pidUdcLimit_M2, &g_asrSpeedPIIn_M2);
#endif

#if (defined OPEN_LOOP_M2)
        if (Math_Qabs(g_speedCommand_M2.speedTargetRamp) <= pFocVarsCfg_M2->stopSpeed)
        {
            Set_McStatus_M2(STOP);

            PWM_Disable(PWM_MODULE_MOTOR2);
        }
#elif (defined CLOSE_LOOP_M2)
        if (Math_Qabs(g_speedCommand_M2.speedFbk) <= pFocVarsCfg_M2->stopSpeed)
        {
            Set_McStatus_M2(STOP);

            PWM_Disable(PWM_MODULE_MOTOR2);
        }
#else
#endif

    break;

    /* Stop brake, Short circuit brake */
    case STOP_BRAKE:
        ShortBrake_RunCaseBrake(&g_sBrake_M2, &g_focVars_M2);
        if (g_shortBrake_M2.subStatus == (uint8_t)CONFIG_RESET)
        {
            PWM_Disable(PWM_MODULE_MOTOR2);
            ShortBrake_Init(&g_shortBrake_M2);
            Set_McStatus_M2(STOP);
        }
    break;

    /* Stop, stop the motor and clear the control parameters, then go to Idle state */
    case STOP:
        Stop_Motor(&g_startUpCtrl_M2, &g_focVars_M2);

        FOC_Parameters_Initialize_M2();
        ACR_TorqueCommandInit_M2(&g_torqueCommand_M2);
        ASR_SpeedCommandSet_M2(pFocVarsCfg_M2, &g_speedCommand_M2);
        PWM_Update(PWM_PERIOD_VALUE - 1, PWM_PERIOD_VALUE - 1, PWM_PERIOD_VALUE - 1, &g_focVars_M2);

        Set_McStatus_M2(IDLE);
    break;

    /* Identify, perform motor parameter identification function */
    case IDENTIFY:
        ParamIdentify(&g_paramIdentify_M2, &g_STC_M2, pHWBoardParam);
        if (g_paramIdentifyCtrl_M2.initCtrl == 1)
        {
            ParameterIdentifySaveProcess(&g_paramIdentify_M2, &g_paramIdentifyBuffer_M2);
            Set_McStatus_M2(IDLE);
        }
    break;

    /* Fault, stop the motor. If it is expected to run again, it is necessary to clear the fault and enter the stop state */
    case FAULT:
        Stop_Motor(&g_startUpCtrl_M2, &g_focVars_M2);

        if (g_protector_M2.ErrFlag.all == 0)
        {
//            Set_McStatus_M2(STOP);
        }

    break;

    /* Restart, re-do FOC start. when motor start process is abnormal, restart the motor */
#if (defined STARTUP_CHECK_RESTART_ENABLE)
    case RESTART:
        Foc_RestartProcess_M2(PWM_MODULE_MOTOR2, &g_STC_M2, &g_restart_M2);
        Set_McStatus_M2(START);
    break;
#endif

    /* flash task, do flash write operation to save user debug value of variables, and flash read operation to recovery flash data to variables */
#if (defined DATA_ACCESS_FLASH_ENABLE_M2)
    case FLASH_TASK:
        DisableInterrupts;
        g_flashOperationStatus_M2 = Eflash_DataAccessEflashProcess_M2(&g_dataAccessFlash_M2, g_VarAccessArray_M2);
        if (g_flashOperationStatus_M2 == STATUS_SUCCESS)
        {
            Set_McStatus_M2(IDLE);

            Eflash_DataAccessEflashInit(&g_dataAccessFlash_M2);
            g_flashTaskFinish_M2 = 1;
            EnableInterrupts;
        }
    break;
#endif

    /* Hall sensor self learn, do self learning to get Hall sensor installation information */
#if (defined HALL_SENSORS_M2)
    case HALL_SELFLEARN:
        StartUp_Motor(&g_startUpCtrl_M2, &g_focVars_M2);
        if (defHallSensor_M2.hallSelfLearnEnable == DISABLE)
        {
            Stop_Motor(&g_startUpCtrl_M2, &g_focVars_M2);
            Set_McStatus_M2(STOP);
            HallSelfLearnResultInit(&g_hallSelfLearnBuffer_M2);
        }
        if (g_hallSelfLearnFinishFlag_M2 == 1)
        {
            HallSelfLearnSaveProcess(&g_hallSelfLearnBuffer_M2, &g_hall_M2);
            defHallSensor_M2.hallSelfLearnEnable = DISABLE;

            Stop_Motor(&g_startUpCtrl_M2, &g_focVars_M2);
            Set_McStatus_M2(STOP);
        }
    break;
#endif

    default:
    break;
    }
#endif
}

/*!
* @brief Foc task in 1 ms time base, do speed loop, speed ramp, etc. FOC 1ms task (sensorless, hall, encoder all in this), callback in task_scheduler.
*
* @param[in] none
* @return none
*/
void Motor_1ms_Task(void)
{
#if (defined MOTOR_ONE_TYPE)

#if (defined VF_CONTROL)
    VF_1ms_Task();
#elif (defined FOC_SINE_WAVE)
#if (defined POWER_DCCUR_EST_ENABLE)

    /* g_powerEst.powerInTrue means input power true value, increase 100 times */
    PowerEst_Calc(&g_STC, pPowerEstCfg, g_asrSpeedPIIn.fbkPu, g_dcCurEst.dcCurTrue, &g_powerEst);
    /* g_dcCurEst.dcCurTrue means bus current true value, increase 100 times */
    DcCurEst_Calc(&g_motorParamPu, &g_focVarsCtrl, &g_pwmValue, 100, &g_dcCurEst);

#endif  /* end of POWER_DCCUR_EST_ENABLE */

    MainStateMachine();

    Get_VoltageSample(&g_motorParamPu, &g_adcParam);

    Motor_Protection(&g_protector);

    ASR_SpeedTargetLimitProcess(&g_speedCommand);

    if ((Get_McStatus() == RUN) || (Get_McStatus() == SLOW_DOWN))
    {
#if (defined FOC_ACIM)
//        Acim_AsrPiCoefUpdate();
#elif (defined FOC_PMSM)
    #if (defined ASR_VARPI_ENABLE)
        ASR_SpeedVarPICalc(g_speedCommand.speedFbk, &g_asrSpeedVarPI, &g_asrSpeedCof);
    #endif      /* end of ASR_VARPI_ENABLE */

    #if (defined SENSORLESS)
        #if ((defined ATAN_ANGLE_ESTIMATION) && (defined ELEC_FREQUENCE_LOW))
        Asr_EstSpeedCalc(g_startUp.pStCtrl->elecThetaEst, &g_rotationalSpeed);
        #endif  /* end of ATAN_ANGLE_ESTIMATION */
    #elif ((defined ENCODER_SENSORS) || (defined ENCODER_HALL_SENSORS))
        #if ((!defined MT_SPEED_MEASURE) && (defined ELEC_FREQUENCE_LOW))
        Asr_EstSpeedCalc(g_focVarsCtrl.elecAngle, &g_rotationalSpeed);
        #endif  /* end of MT_SPEED_MEASURE */
    #else
    #endif      /* end of SENSORLESS */
#endif          /* end of FOC_ACIM */

        ASR_SpeedFbk_Update(&g_speedCommand);

#if (defined SCURVE_SPEED_RAMP)
    /* DO NOTHING IN 1MS TASK WHILE SELECT S_CURVE RAMP FUNCTION*/
#else
    #if (defined SENSORLESS)
        if (g_startUp.pStCtrl->startUpFlag == 6)
        {
            ASR_SpeedTargetRampCalc(pFocVarsCfg, &g_speedCommand);
        }
    #else
        if (Get_McStatus() == RUN || Get_McStatus() == SLOW_DOWN)
        {
            ASR_SpeedTargetRampCalc(pFocVarsCfg, &g_speedCommand);
        }
    #endif      /* end of SENSORLESS */
#endif          /* end of SCURVE_SPEED_RAMP */

        ASR_AvgSpeedCalc(&g_speedCommand);
    }
#else
#endif          /* end of VF_CONTROL */

#endif          /* end of MOTOR_ONE_TYPE */
}

/*!
* @brief Foc task in 1 ms time base of Motor_2, do speed loop, speed ramp, etc. FOC 1ms task (sensorless, hall, encoder all in this), callback in task_scheduler.
*
* @param[in] none
* @return none
*/
void Motor_1ms_Task_M2(void)
{
#if (defined MOTOR_TWO_TYPE)

//#if (defined VF_CONTROL_M2)
//    VF_1ms_Task();
//#elif (defined FOC_SINE_WAVE_M2)
#if (defined POWER_DCCUR_EST_ENABLE_M2)
    PowerEst_Calc(&g_STC_M2, pPowerEstCfg_M2, g_asrSpeedPIIn_M2.fbkPu, g_dcCurEst_M2.dcCurTrue, &g_powerEst_M2);
    /* g_dcCurEst.dcCurTrue means bus current true value, increase 100 times */
    DcCurEst_Calc(&g_motorParamPu_M2, &g_focVarsCtrl_M2, &g_pwmValue_M2, 100, &g_dcCurEst_M2);

#endif  /* end of POWER_DCCUR_EST_ENABLE_M2 */

    MainStateMachine_M2();

    Get_VoltageSample_M2(&g_motorParamPu_M2, &g_adcParam_M2);

    Motor_Protection_M2(&g_protector_M2);

    ASR_SpeedTargetLimitProcess_M2(&g_speedCommand_M2);

    if ((Get_McStatus_M2() == RUN) || (Get_McStatus_M2() == SLOW_DOWN))
    {
#if (defined FOC_ACIM_M2)
//        Acim_AsrPiCoefUpdate();
#elif (defined FOC_PMSM_M2)
    #if (defined ASR_VARPI_ENABLE_M2)
        ASR_SpeedVarPICalc(g_speedCommand_M2.speedFbk, &g_asrSpeedVarPI_M2, &g_asrSpeedCof_M2);
    #endif  /* end of ASR_VARPI_ENABLE_M2 */

    #if (defined SENSORLESS_M2)
        #if ((defined ATAN_ANGLE_ESTIMATION_M2) && (defined ELEC_FREQUENCE_LOW_M2))
        Asr_EstSpeedCalc(g_startUp_M2.pStCtrl->elecThetaEst, &g_rotationalSpeed_M2);
        #endif  /* end of ATAN_ANGLE_ESTIMATION_M2 */
    #elif ((defined ENCODER_SENSORS_M2) || (defined ENCODER_HALL_SENSORS_M2))
        #if ((!defined MT_SPEED_MEASURE_M2) && (defined ELEC_FREQUENCE_LOW_M2))
        Asr_EstSpeedCalc(g_focVarsCtrl_M2.elecAngle, &g_rotationalSpeed_M2);
        #endif  /* end of MT_SPEED_MEASURE_M2 */
    #else
    #endif      /* end of SENSORLESS_M2 */
#endif

        ASR_SpeedFbk_Update_M2(&g_speedCommand_M2);

#if (defined SCURVE_SPEED_RAMP_M2)
    /* DO NOTHING IN 1MS TASK WHILE SELECT S_CURVE RAMP FUNCTION*/
#else
    #if (defined SENSORLESS_M2)
        if (g_startUp_M2.pStCtrl->startUpFlag == 6)
        {
            ASR_SpeedTargetRampCalc(pFocVarsCfg_M2, &g_speedCommand_M2);
        }
    #else
        if (Get_McStatus_M2() == RUN || Get_McStatus_M2() == SLOW_DOWN)
        {
            ASR_SpeedTargetRampCalc(pFocVarsCfg_M2, &g_speedCommand_M2);
        }
    #endif  /* end of SENSORLESS_M2 */
#endif      /* end of SCURVE_SPEED_RAMP_M2 */
        ASR_AvgSpeedCalc(&g_speedCommand_M2);
    }

//#else
//#endif

#endif      /* end of MOTOR_TWO_TYPE */
}

/*!
* @brief Foc task in 2 ms time base. If TOOL_UART_FUNCTION macro is defined, the UART function with host PC tool is executed in 2ms time base task.
*
* @param[in] none
* @return none
*/
void Motor_2ms_Task(void)
{
#if (defined MOTOR_ONE_TYPE)

#if (defined VF_CONTROL)
    VF_2ms_Task();
#elif (defined FOC_SINE_WAVE)
    #if (defined TOOL_UART_FUNCTION)
    /* Motor Debug Tool command Process */
    UI_UART_ToolCmd_Proc(&g_uiRxBuf);
    UI_UART_TimeOut_Cnt();
    UI_MasterCmd_ProcCnt();
    #endif  /* end of TOOL_UART_FUNCTION */

    if ((Get_McStatus() == RUN) || (Get_McStatus() == SLOW_DOWN))
    {
    #if (defined SCURVE_SPEED_RAMP)
        #if (defined SENSORLESS)
        if (g_startUp.pStCtrl->startUpFlag == 6)
        {
            SCurve_MainProcess(pFocVarsCfg->motorDir, &g_sCurveData, &g_speedCommand, &g_startUpCurve);
        }
        #else
        if (g_focVarsCtrl.pwmOutMask == SVPWM_OUTPUT_ENABLE)
        {
            SCurve_MainProcess(pFocVarsCfg->motorDir, &g_sCurveData, &g_speedCommand, &g_startUpCurve);
        }
        #endif  /* end of SENSORLESS */
    #endif      /* end of SCURVE_SPEED_RAMP */
    }
#else
#endif          /* end of VF_CONTROL */

#endif          /* end of MOTOR_ONE_TYPE */
}

/*!
* @brief Foc task in 2 ms time base. If TOOL_UART_FUNCTION macro is defined, the UART function with host PC tool is executed in 2ms time base task.
*
* @param[in] none
* @return none
*/
void Motor_2ms_Task_M2(void)
{
#if (defined MOTOR_TWO_TYPE)

#if (defined VF_CONTROL_M2)
    VF_2ms_Task();
#elif (defined FOC_SINE_WAVE_M2)
//#if (defined TOOL_UART_FUNCTION)
//    /* Motor Debug Tool command Process */
//    UI_UART_ToolCmd_Proc(&g_uiRxBuf);
//    UI_UART_TimeOut_Cnt();
//    UI_MasterCmd_ProcCnt();
//#endif

    if ((Get_McStatus_M2() == RUN) || (Get_McStatus_M2() == SLOW_DOWN))
    {
    #if (defined SCURVE_SPEED_RAMP_M2)
        #if (defined SENSORLESS_M2)
        if (g_startUp_M2.pStCtrl->startUpFlag == 6)
        {
            SCurve_MainProcess_M2(pFocVarsCfg_M2->motorDir, &g_sCurveData_M2, &g_speedCommand_M2, &g_startUpCurve_M2);
        }
        #else
        if (g_focVarsCtrl_M2.pwmOutMask == SVPWM_OUTPUT_ENABLE)
        {
            SCurve_MainProcess_M2(pFocVarsCfg_M2->motorDir, &g_sCurveData_M2, &g_speedCommand_M2, &g_startUpCurve_M2);
        }
        #endif  /* end of SENSORLESS_M2 */
    #endif      /* end of SCURVE_SPEED_RAMP_M2 */
    }
#else
#endif          /* end of VF_CONTROL_M2 */

#endif          /* end of MOTOR_TWO_TYPE */
}

/*!
* @brief Foc task in 4 ms time base.
*
* @param[in] none
* @return none
*/
void Motor_4ms_Task(void)
{
#if (defined MOTOR_ONE_TYPE)
    if (Get_McStatus() == IDENTIFY)
    {
        ParamIdentify_4msTask(&g_paramIdentify, &g_STC);
    }
#endif  /* end of MOTOR_ONE_TYPE */
#if (defined MOTOR_TWO_TYPE)
    if (Get_McStatus_M2() == IDENTIFY)
    {
        ParamIdentify_4msTask(&g_paramIdentify_M2, &g_STC_M2);
    }
#endif  /* end of MOTOR_TWO_TYPE */
}

/*!
* @brief Foc task in 6 ms time base.
*
* @param[in] none
* @return none
*/
void Motor_6ms_Task(void)
{

}

/*!
* @brief Foc task in 10 ms time base.
*
* @param[in] none
* @return none
*/
void Motor_10ms_Task(void)
{
#if (defined MOTOR_ONE_TYPE)
    #if (defined POSITION_LOOP_CTRL)
    PositionCommandUpdate(&g_positionCommand, &g_encoderSensor);
    APR_PositionTargetRampCalc(&g_positionCommand);

    APR_PositionLoop_Process(&g_positionCommand, &g_pidApr, &g_focVarsCtrl);
    #endif  /* end of POSITION_LOOP_CTRL */
    #if (defined INIT_POS_DETEC)
    if (Get_McStatus() == INIT_POS)
    {
        InitPosDetec_LongPeriodTask(&g_initPosDetecStruct, &g_STC);
    }
    #endif  /* end of INIT_POS_DETEC */
    #if (defined MT_SPEED_MEASURE)
    Encoder_MT_SpeedCalc(&g_encoderSpeed);
    #endif  /* end of MT_SPEED_MEASURE */

    #if (defined INERTIA_ONLINE_IDENT)
    InertiaIdentify_Calc();
    #endif  /* end of INERTIA_ONLINE_IDENT */
#endif      /* end of MOTOR_ONE_TYPE */

#if (defined MOTOR_TWO_TYPE)
    #if (defined POSITION_LOOP_CTRL_M2)
    PositionCommandUpdate_M2(&g_positionCommand_M2, &g_encoderSensor_M2);
    APR_PositionTargetRampCalc_M2(&g_positionCommand_M2);

    APR_PositionLoop_Process(&g_positionCommand_M2, &g_pidApr_M2, &g_focVarsCtrl_M2);
    #endif  /* end of POSITION_LOOP_CTRL_M2 */
    #if (defined INIT_POS_DETEC_M2)
    if (Get_McStatus_M2() == INIT_POS)
    {
        InitPosDetec_LongPeriodTask(&g_initPosDetecStruct_M2, &g_STC_M2);
    }
    #endif  /* end of INIT_POS_DETEC_M2 */
    #if (defined MT_SPEED_MEASURE_M2)
    Encoder_MT_SpeedCalc_M2(&g_encoderSpeed_M2);
    #endif  /* end of MT_SPEED_MEASURE_M2 */

    #if (defined INERTIA_ONLINE_IDENT_M2)
    InertiaIdentify_Calc_M2();
    #endif  /* end of INERTIA_ONLINE_IDENT_M2 */
#endif  /* end of MOTOR_TWO_TYPE */

}

/*!
* @brief Foc task in 2 s time base.
*
* @param[in] none
* @return none
*/
void FOC_2000ms_Task(void)
{
//    if (g_protector.ErrFlag.all)
//    {
////        printf(" g_protector.ErrFlag.all = 0x%x \r\n", g_protector.ErrFlag.all);
//    }
}

/* =============================================  EOF  ============================================== */
