/* Copyright Statement:
*
* This software/firmware and related documentation ("AutoChips Software") are
* protected under relevant copyright laws. The information contained herein is
* confidential and proprietary to AutoChips Inc. and/or its licensors. Without
* the prior written permission of AutoChips inc. and/or its licensors, any
* reproduction, modification, use or disclosure of AutoChips Software, and
* information contained herein, in whole or in part, shall be strictly
* prohibited.
*
* AutoChips Inc. (C) 2020. All rights reserved.
*
* BY OPENING THIS FILE, RECEIVER HEREBY UNEQUIVOCALLY ACKNOWLEDGES AND AGREES
* THAT THE SOFTWARE/FIRMWARE AND ITS DOCUMENTATIONS ("AUTOCHIPS SOFTWARE")
* RECEIVED FROM AUTOCHIPS AND/OR ITS REPRESENTATIVES ARE PROVIDED TO RECEIVER
* ON AN "AS-IS" BASIS ONLY. AUTOCHIPS EXPRESSLY DISCLAIMS ANY AND ALL
* WARRANTIES, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
* WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
* NONINFRINGEMENT. NEITHER DOES AUTOCHIPS PROVIDE ANY WARRANTY WHATSOEVER WITH
* RESPECT TO THE SOFTWARE OF ANY THIRD PARTY WHICH MAY BE USED BY,
* INCORPORATED IN, OR SUPPLIED WITH THE AUTOCHIPS SOFTWARE, AND RECEIVER AGREES
* TO LOOK ONLY TO SUCH THIRD PARTY FOR ANY WARRANTY CLAIM RELATING THERETO.
* RECEIVER EXPRESSLY ACKNOWLEDGES THAT IT IS RECEIVER'S SOLE RESPONSIBILITY TO
* OBTAIN FROM ANY THIRD PARTY ALL PROPER LICENSES CONTAINED IN AUTOCHIPS
* SOFTWARE. AUTOCHIPS SHALL ALSO NOT BE RESPONSIBLE FOR ANY AUTOCHIPS SOFTWARE
* RELEASES MADE TO RECEIVER'S SPECIFICATION OR TO CONFORM TO A PARTICULAR
* STANDARD OR OPEN FORUM. RECEIVER'S SOLE AND EXCLUSIVE REMEDY AND AUTOCHIPS'S
* ENTIRE AND CUMULATIVE LIABILITY WITH RESPECT TO THE AUTOCHIPS SOFTWARE
* RELEASED HEREUNDER WILL BE, AT AUTOCHIPS'S OPTION, TO REVISE OR REPLACE THE
* AUTOCHIPS SOFTWARE AT ISSUE, OR REFUND ANY SOFTWARE LICENSE FEES OR SERVICE
* CHARGE PAID BY RECEIVER TO AUTOCHIPS FOR SUCH AUTOCHIPS SOFTWARE AT ISSUE.
*/

/*!
* @file pulse_inject_api.c
*
* @brief This file mainly provides mcu PWM and ADC module operation function aviod read or wirte mcu registers in libray.
*
*/

/* ===========================================  Includes  =========================================== */
#include "adc_hw.h"
#include "ctu_drv.h"
#include "pulse_inject_api.h"
#include "foc_isr.h"
#include "globalvar.h"
#include "pwmdrv.h"
#include "motorctrl_api.h"

/* ============================================  Define  ============================================ */
//#if 0
/* ===========================================  Typedef  ============================================ */

/* ==========================================  Variables  =========================================== */

/* ====================================  Functions declaration  ===================================== */

/* ======================================  Functions define  ======================================== */
/*!
* @brief Beacuse the motor manbe bldc or pmsm, the control mode is different.
*        In order to output pulse injection, PWM and ADC module will be configurated, and then go to the next status.
*
* @param[in] pwmModule: PWM instance of Motor
* @param[in] pulseInject: pulseInject data variable, pointer to PULSE_INJECT_DATA structure
* @return none
*/
void PulseInject_Config(uint8_t pwmModule, PULSE_INJECT_DATA *pulseInject)
{
#if (defined MOTOR_ONE_TYPE)
    PWM_Disable(pwmModule);

    #ifndef PULSE_SELF_LEARN
    pulseInject->selfLearnFlag = 1;
    pulseInject->pwmTs = PWM_SELF_LEARN_PRD;
    #else
    pulseInject->pwmTs = INIT_PULSE_PERIOD;
    #endif

//    CTU_SetAdcInjectTriggerSource(CTU_TRIGGER_ADC_PWM1_INIT);
    TRGMUX_DRV_SetTrigSourceForTargetModule(0, TRGMUX_TRIG_SOURCE_PWM3_INIT_TRIG, TRGMUX_TARGET_MODULE_ADC1_INJECTION0);
    
//    CTU_SetDelay1Time(0);

    ADC_SetInjectLength(ADC1, 1);

    /* IBUS */
//    ADC_SetInjectSequence(ADC0, ADC_CH_7, 0);
    ADC_SetInjectConversionChannel(ADC1, ADC_ISEQ_0, ADC_CH_11);
    
    ADC_SetChannelSampleTime(ADC1, ADC_CH_11, ADC_SPT_CLK_15);
    ADC_SetInjectEOCInterruptEnableFlag(ADC1, ADC_ISEQ_0, true);
//    ADC_SetIntervalMode(ADC0, DISABLE);
    ADC_SetIntervalEnableFlag(ADC1, DISABLE);
    PWM_DRV_SetMaxCountValue(pwmModule, (pulseInject->pwmTs - 1));

    /* set PWM2 Init Trigger Source */
    PWM_DRV_SetInitTrigger(pwmModule, true);

    PulseInj_SetChlVal(pwmModule, 7);

    PWM_Enable(pwmModule);

    PWM_DRV_SetSoftwareTriggerSync(pwmModule, ENABLE);

    pulseInject->subStatus = (uint8_t)SELF_LEARNING;
#endif
}

/*!
* @brief Beacuse the motor manbe bldc or pmsm, the control mode is different.
*        In order to output pulse injection, PWM and ADC module will be configurated, and then go to the next status.
*
* @param[in] pwmModule: PWM instance of Motor
* @param[in] pulseInject: PULSE_INJECT_DATA type pointer
* @return none
*/
void PulseInject_Config_M2(uint8_t pwmModule, PULSE_INJECT_DATA *pulseInject)
{
#if (defined MOTOR_TWO_TYPE)
    PWM_Disable(pwmModule);

    #ifndef PULSE_SELF_LEARN
    pulseInject->selfLearnFlag = 1;
    pulseInject->pwmTs = PWM_SELF_LEARN_PRD_M2;
    #else
    pulseInject->pwmTs = INIT_PULSE_PERIOD_M2;
    #endif

    TRGMUX_DRV_SetTrigSourceForTargetModule(0, TRGMUX_TRIG_SOURCE_PWM2_INIT_TRIG, TRGMUX_TARGET_MODULE_ADC0_INJECTION0);

    ADC_SetInjectLength(ADC0, 1);

    /* IBUS */
//    ADC_SetInjectSequence(ADC0, ADC_CH_7, 0);
    ADC_SetInjectConversionChannel(ADC0, ADC_ISEQ_0, ADC_CH_3);
    
    ADC_SetChannelSampleTime(ADC0, ADC_CH_3, ADC_SPT_CLK_15);
    ADC_SetInjectEOCInterruptEnableFlag(ADC0, ADC_ISEQ_0, true);
//    ADC_SetIntervalMode(ADC0, DISABLE);
    ADC_SetIntervalEnableFlag(ADC0, DISABLE);
    PWM_DRV_SetMaxCountValue(pwmModule, (pulseInject->pwmTs - 1));

    /* set PWM2 Init Trigger Source */
    PWM_DRV_SetInitTrigger(pwmModule, true);

    PulseInj_SetChlVal(pwmModule, 7);

    PWM_Enable(pwmModule);

    PWM_DRV_SetSoftwareTriggerSync(pwmModule, ENABLE);

    pulseInject->subStatus = (uint8_t)SELF_LEARNING;
#endif
}

/*!
* @brief Initial position pulse injection function finished, then recover configuration.
*        Because enter initial position module, bldc or pmsm control have different configuration, after initial position checked,
*        recover configuration according to control method.
*
* @param[in] pwmModule: PWM instance of Motor
* @param[in] pulseInject: pulseInject data variable, pointer to PULSE_INJECT_DATA structure
* @return none
*/
void PulseInject_ConfigReset(uint8_t pwmModule, PULSE_INJECT_DATA *pulseInject)
{
    NVIC_DisableIRQ(ADC1_IRQn);

    PWM_Disable(pwmModule);
    PulseInj_SetChlVal(pwmModule, 6);

    FOC_CTU_Initialize();
//    FOC_ADC_Initialize();
    FOC_ADC_Initialize_M1();

    /* must behind adc init */
    FOC_PWM_Initialize();

    NVIC_EnableIRQ(ADC1_IRQn);
    pulseInject->pulseInjDoneFlag = 1;
}

/*!
* @brief Initial position pulse injection function finished, then recover configuration.
*        Because enter initial position module, bldc or pmsm control have different configuration, after initial position checked,
*        recover configuration according to control method.
*
* @param[in] pwmModule: PWM instance of Motor
* @param[in] pulseInject: pulseInject data variable, pointer to PULSE_INJECT_DATA structure
* @return none
*/
void PulseInject_ConfigReset_M2(uint8_t pwmModule, PULSE_INJECT_DATA *pulseInject)
{
    NVIC_DisableIRQ(ADC0_IRQn);

    PWM_Disable(pwmModule);
    PulseInj_SetChlVal(pwmModule, 6);

    FOC_CTU_Initialize();
//    FOC_ADC_Initialize();
    FOC_ADC_Initialize_M2();

    /* must behind adc init */
    FOC_PWM_Initialize_M2();

    NVIC_EnableIRQ(ADC0_IRQn);
    pulseInject->pulseInjDoneFlag = 1;
}

/*!
* @brief Initial position checking need inject at different direction, so this function can change pwm output.
*        This function can fit to different hardboard, through change settings.
*        What's more, PWM configuration makes pwm output mask effective immediately.
*
* @param[in] pwmModule: PWM instance of Motor
* @param[in] section: Judge pulse inject direction
* @return none
*/
void PulseInj_SetChlVal(uint8_t pwmModule, uint8_t section)
{
    FOC_VARS_CFG *focCfg;

    if (pwmModule == PWM_INDEX(PWM_MODULE))
    {
        focCfg = pFocVarsCfg;
    }
    else if (pwmModule == PWM_INDEX(PWM_MODULE_M2))
    {
        focCfg = pFocVarsCfg_M2;
    }
    else
    {
        return;
    }

    switch (section)
    {
    /* A+:0, B-:3, C-:5 */
    case 0:
    PWM_DRV_SoftControlChannelLevel(focCfg->pwmId, (pwm_channel_type_t)(2 * focCfg->pwmChannelU), UP_BRIDGE_ACTIVE_STATE, true);
    PWM_DRV_SoftControlChannelLevel(focCfg->pwmId, (pwm_channel_type_t)(2 * focCfg->pwmChannelV + 1), DOWN_BRIDGE_ACTIVE_STATE, true);
    PWM_DRV_SoftControlChannelLevel(focCfg->pwmId, (pwm_channel_type_t)(2 * focCfg->pwmChannelW + 1), DOWN_BRIDGE_ACTIVE_STATE, true);
    PWM_DRV_SoftControlChannelLevel(focCfg->pwmId, (pwm_channel_type_t)(2 * focCfg->pwmChannelU + 1), DOWN_BRIDGE_INACTIVE_STATE, true);
    PWM_DRV_SoftControlChannelLevel(focCfg->pwmId, (pwm_channel_type_t)(2 * focCfg->pwmChannelV), UP_BRIDGE_INACTIVE_STATE, true);
    PWM_DRV_SoftControlChannelLevel(focCfg->pwmId, (pwm_channel_type_t)(2 * focCfg->pwmChannelW), UP_BRIDGE_INACTIVE_STATE, true);
    break;

    /* A-:1, B+:2, C+:4 */
    case 1:
    PWM_DRV_SoftControlChannelLevel(focCfg->pwmId, (pwm_channel_type_t)(2 * focCfg->pwmChannelU + 1), DOWN_BRIDGE_ACTIVE_STATE, true);
    PWM_DRV_SoftControlChannelLevel(focCfg->pwmId, (pwm_channel_type_t)(2 * focCfg->pwmChannelV), UP_BRIDGE_ACTIVE_STATE, true);
    PWM_DRV_SoftControlChannelLevel(focCfg->pwmId, (pwm_channel_type_t)(2 * focCfg->pwmChannelW), UP_BRIDGE_ACTIVE_STATE, true);
    PWM_DRV_SoftControlChannelLevel(focCfg->pwmId, (pwm_channel_type_t)(2 * focCfg->pwmChannelU), UP_BRIDGE_INACTIVE_STATE, true);
    PWM_DRV_SoftControlChannelLevel(focCfg->pwmId, (pwm_channel_type_t)(2 * focCfg->pwmChannelV + 1), DOWN_BRIDGE_INACTIVE_STATE, true);
    PWM_DRV_SoftControlChannelLevel(focCfg->pwmId, (pwm_channel_type_t)(2 * focCfg->pwmChannelW + 1), DOWN_BRIDGE_INACTIVE_STATE, true);
    break;

    /* A-:1, B+:2, C-:5 */
    case 2:
    PWM_DRV_SoftControlChannelLevel(focCfg->pwmId, (pwm_channel_type_t)(2 * focCfg->pwmChannelU + 1), DOWN_BRIDGE_ACTIVE_STATE, true);
    PWM_DRV_SoftControlChannelLevel(focCfg->pwmId, (pwm_channel_type_t)(2 * focCfg->pwmChannelV), UP_BRIDGE_ACTIVE_STATE, true);
    PWM_DRV_SoftControlChannelLevel(focCfg->pwmId, (pwm_channel_type_t)(2 * focCfg->pwmChannelW + 1), DOWN_BRIDGE_ACTIVE_STATE, true);
    PWM_DRV_SoftControlChannelLevel(focCfg->pwmId, (pwm_channel_type_t)(2 * focCfg->pwmChannelU), UP_BRIDGE_INACTIVE_STATE, true);
    PWM_DRV_SoftControlChannelLevel(focCfg->pwmId, (pwm_channel_type_t)(2 * focCfg->pwmChannelV + 1), DOWN_BRIDGE_INACTIVE_STATE, true);
    PWM_DRV_SoftControlChannelLevel(focCfg->pwmId, (pwm_channel_type_t)(2 * focCfg->pwmChannelW), UP_BRIDGE_INACTIVE_STATE, true);
    break;

    /* A+:0, B-:3, C+:4 */
    case 3:
    PWM_DRV_SoftControlChannelLevel(focCfg->pwmId, (pwm_channel_type_t)(2 * focCfg->pwmChannelU), UP_BRIDGE_ACTIVE_STATE, true);
    PWM_DRV_SoftControlChannelLevel(focCfg->pwmId, (pwm_channel_type_t)(2 * focCfg->pwmChannelV + 1), DOWN_BRIDGE_ACTIVE_STATE, true);
    PWM_DRV_SoftControlChannelLevel(focCfg->pwmId, (pwm_channel_type_t)(2 * focCfg->pwmChannelW), UP_BRIDGE_ACTIVE_STATE, true);
    PWM_DRV_SoftControlChannelLevel(focCfg->pwmId, (pwm_channel_type_t)(2 * focCfg->pwmChannelU + 1), DOWN_BRIDGE_INACTIVE_STATE, true);
    PWM_DRV_SoftControlChannelLevel(focCfg->pwmId, (pwm_channel_type_t)(2 * focCfg->pwmChannelV), UP_BRIDGE_INACTIVE_STATE, true);
    PWM_DRV_SoftControlChannelLevel(focCfg->pwmId, (pwm_channel_type_t)(2 * focCfg->pwmChannelW + 1), DOWN_BRIDGE_INACTIVE_STATE, true);
    break;

    /* A-:1, B-:3, C+:4 */
    case 4:
    PWM_DRV_SoftControlChannelLevel(focCfg->pwmId, (pwm_channel_type_t)(2 * focCfg->pwmChannelU + 1), DOWN_BRIDGE_ACTIVE_STATE, true);
    PWM_DRV_SoftControlChannelLevel(focCfg->pwmId, (pwm_channel_type_t)(2 * focCfg->pwmChannelV + 1), DOWN_BRIDGE_ACTIVE_STATE, true);
    PWM_DRV_SoftControlChannelLevel(focCfg->pwmId, (pwm_channel_type_t)(2 * focCfg->pwmChannelW), UP_BRIDGE_ACTIVE_STATE, true);
    PWM_DRV_SoftControlChannelLevel(focCfg->pwmId, (pwm_channel_type_t)(2 * focCfg->pwmChannelU), UP_BRIDGE_INACTIVE_STATE, true);
    PWM_DRV_SoftControlChannelLevel(focCfg->pwmId, (pwm_channel_type_t)(2 * focCfg->pwmChannelV), UP_BRIDGE_INACTIVE_STATE, true);
    PWM_DRV_SoftControlChannelLevel(focCfg->pwmId, (pwm_channel_type_t)(2 * focCfg->pwmChannelW + 1), DOWN_BRIDGE_INACTIVE_STATE, true);
    break;

    /* A+:0, B+:2, C-:5 */
    case 5:
    PWM_DRV_SoftControlChannelLevel(focCfg->pwmId, (pwm_channel_type_t)(2 * focCfg->pwmChannelU), UP_BRIDGE_ACTIVE_STATE, true);
    PWM_DRV_SoftControlChannelLevel(focCfg->pwmId, (pwm_channel_type_t)(2 * focCfg->pwmChannelV), UP_BRIDGE_ACTIVE_STATE, true);
    PWM_DRV_SoftControlChannelLevel(focCfg->pwmId, (pwm_channel_type_t)(2 * focCfg->pwmChannelW + 1), DOWN_BRIDGE_ACTIVE_STATE, true);
    PWM_DRV_SoftControlChannelLevel(focCfg->pwmId, (pwm_channel_type_t)(2 * focCfg->pwmChannelU + 1), DOWN_BRIDGE_INACTIVE_STATE, true);
    PWM_DRV_SoftControlChannelLevel(focCfg->pwmId, (pwm_channel_type_t)(2 * focCfg->pwmChannelV + 1), DOWN_BRIDGE_INACTIVE_STATE, true);
    PWM_DRV_SoftControlChannelLevel(focCfg->pwmId, (pwm_channel_type_t)(2 * focCfg->pwmChannelW), UP_BRIDGE_INACTIVE_STATE, true);
    break;

    /* recover */
    case 6:
    PWM_DRV_SetChannelSoftControl(focCfg->pwmId, (pwm_channel_type_t)(2 * focCfg->pwmChannelU), false);
    PWM_DRV_SetChannelSoftControl(focCfg->pwmId, (pwm_channel_type_t)(2 * focCfg->pwmChannelU + 1), false);
    PWM_DRV_SetChannelSoftControl(focCfg->pwmId, (pwm_channel_type_t)(2 * focCfg->pwmChannelV), false);
    PWM_DRV_SetChannelSoftControl(focCfg->pwmId, (pwm_channel_type_t)(2 * focCfg->pwmChannelV + 1), false);
    PWM_DRV_SetChannelSoftControl(focCfg->pwmId, (pwm_channel_type_t)(2 * focCfg->pwmChannelW), false);
    PWM_DRV_SetChannelSoftControl(focCfg->pwmId, (pwm_channel_type_t)(2 * focCfg->pwmChannelW + 1), false);
    break;

    case 7:
    default:
    PWM_DRV_SoftControlChannelLevel(focCfg->pwmId, (pwm_channel_type_t)(2 * focCfg->pwmChannelU), UP_BRIDGE_INACTIVE_STATE, true);
    PWM_DRV_SoftControlChannelLevel(focCfg->pwmId, (pwm_channel_type_t)(2 * focCfg->pwmChannelU + 1), DOWN_BRIDGE_INACTIVE_STATE, true);
    PWM_DRV_SoftControlChannelLevel(focCfg->pwmId, (pwm_channel_type_t)(2 * focCfg->pwmChannelV), UP_BRIDGE_INACTIVE_STATE, true);
    PWM_DRV_SoftControlChannelLevel(focCfg->pwmId, (pwm_channel_type_t)(2 * focCfg->pwmChannelV + 1), DOWN_BRIDGE_INACTIVE_STATE, true);
    PWM_DRV_SoftControlChannelLevel(focCfg->pwmId, (pwm_channel_type_t)(2 * focCfg->pwmChannelW), UP_BRIDGE_INACTIVE_STATE, true);
    PWM_DRV_SoftControlChannelLevel(focCfg->pwmId, (pwm_channel_type_t)(2 * focCfg->pwmChannelW + 1), DOWN_BRIDGE_INACTIVE_STATE, true);
    break;
    }
}

/*!
* @brief In order to make libray decouple with mcu registers, this function is called by libray to get bus current value.
*        It is called when pulse injection finished, and sample bus current.
*
* @param[in] none
* @return none
*/
void PulseInject_GetBusCur(PULSE_INJECT_DATA *pulseInj)
{
    ADC_DRV_GetSeqResult(ADC1_MODULE, ADC_ISEQ_0, &(pulseInj->busCurVal));
}

/*!
* @brief In order to make libray decouple with mcu registers, this function is called by libray to get bus current value.
*        It is called when pulse injection finished, and sample bus current.
*
* @param[in] none
* @return none
*/
void PulseInject_GetBusCur_M2(PULSE_INJECT_DATA *pulseInj)
{
//    return (ADC_GetInjectData(ADC0, ADC_ISEQ_0));
    ADC_DRV_GetSeqResult(ADC0_MODULE, ADC_ISEQ_0, &(pulseInj->busCurVal));
}

/*!
* @brief Pulse injection function data clear.
*
* @param[in] pulseInject: pulseInject data variable, pointer to PULSE_INJECT_DATA structure
* @return none
*/
void PulseInject_Init(PULSE_INJECT_DATA *pulseInject)
{
    memset(pulseInject, 0, sizeof(*pulseInject));
}

/*!
* @brief Pulse injection function data clear.
*
* @param[in] pulseInject: pulseInject data variable, pointer to PULSE_INJECT_DATA structure
* @return none
*/
void PulseInject_ReJdg_Init(PULSE_INJECT_DATA *pulseInject)
{
    pulseInject->pulseInjDoneFlag = 0;
    pulseInject->periodCnt = 0;
    pulseInject->pulseInjFlag = 0;
    pulseInject->selfLearnFlag = 0;
    pulseInject->subStatus = 0;
    pulseInject->wait = 0;
}

/*!
* @brief Multiple pulse injection process in main status machine(ANGLE_JUDGE).
*
* @param[in] pwmModule: PWM instance of Motor
* @param[in] pulseInject: pulseInject data variable, pointer to PULSE_INJECT_DATA structure
* @param[in] startUp: pointer to STARTUP_CTRL structure
* @return Multiple pulse inject function flag
*         0: pulse injection repeat unfinished, 1: pulse injection repeat finished, 2: pulse injection repeat results are inconsistent
*/
uint16_t PulseInject_Process(uint8_t pwmModule, PULSE_INJECT_DATA *pulseInject, STARTUP_CTRL *startUp)
{
    uint16_t pulseInjRepeatDoneFlag = INIT_UNKNOWN;
#if (defined MOTOR_ONE_TYPE)
    uint16_t index = 0;
    static uint16_t pulseInjectSection[PULSE_INJECTION_TIMES] = {0};
    static uint16_t pulseInjectCnt = 0;

    if (pulseInjectCnt < PULSE_INJECTION_TIMES)
    {
        pulseInjRepeatDoneFlag = INIT_UNKNOWN;
        if (PulseInject(pwmModule, pulseInject))
        {
            if (pulseInject->pulseInjErrCnt == 1)
            {
                pulseInjectCnt = 0;
                pulseInjRepeatDoneFlag = EXE_FAIL;
                PulseInject_ReJdg_Init(pulseInject);
                return (pulseInjRepeatDoneFlag);
            }
            pulseInjectSection[pulseInjectCnt] = pulseInject->flowState;
            if ((pulseInjectCnt >= 1) && (pulseInjectSection[pulseInjectCnt] != pulseInjectSection[pulseInjectCnt - 1]))
            {
                pulseInjectCnt = 0;
                pulseInjRepeatDoneFlag = EXE_FAIL;
                PulseInject_ReJdg_Init(pulseInject);
                /* init speed buffer */
                for (index = 0u; index < PULSE_INJECTION_TIMES; index++)
                {
                    pulseInjectSection[index] = 0;
                }
                return (pulseInjRepeatDoneFlag);
            }
            else
            {
            }
            pulseInjectCnt++;

            startUp->startUpElecTheta = InitPos_GetPosAngle(pulseInject);

            PulseInject_ReJdg_Init(pulseInject);
        }
    }
    else
    {
        pulseInjectCnt = 0;
        pulseInjRepeatDoneFlag = EXE_SUCCESS;
        /* init speed buffer */
        for (index = 0u; index < PULSE_INJECTION_TIMES; index++)
        {
            pulseInjectSection[index] = 0;
        }
    }
#endif
    return (pulseInjRepeatDoneFlag);
}

/*!
* @brief Multiple pulse injection process in main status machine(ANGLE_JUDGE).
*
* @param[in] pwmModule: PWM instance of Motor
* @param[in] pulseInject: pulseInject data variable, pointer to PULSE_INJECT_DATA structure
* @param[in] startUp: pointer to STARTUP_CTRL structure
* @return Multiple pulse inject function flag
*         0: pulse injection repeat unfinished, 1: pulse injection repeat finished, 2: pulse injection repeat results are inconsistent
*/
uint16_t PulseInject_Process_M2(uint8_t pwmModule, PULSE_INJECT_DATA *pulseInject, STARTUP_CTRL *startUp)
{
    uint16_t pulseInjRepeatDoneFlag = 0;
#if (defined MOTOR_TWO_TYPE)
    uint16_t index = 0;
    static uint16_t pulseInjectSection[PULSE_INJECTION_TIMES_M2] = {0};
    static uint16_t pulseInjectCnt = 0;

    if (pulseInjectCnt < PULSE_INJECTION_TIMES_M2)
    {
        pulseInjRepeatDoneFlag = 0;
        if (PulseInject_M2(pwmModule, pulseInject))
        {
            if (pulseInject->pulseInjErrCnt == 1)
            {
                pulseInjectCnt = 0;
                pulseInjRepeatDoneFlag = 2;
                PulseInject_ReJdg_Init(pulseInject);
                return (pulseInjRepeatDoneFlag);
            }
            pulseInjectSection[pulseInjectCnt] = pulseInject->flowState;
            if ((pulseInjectCnt >= 1) && (pulseInjectSection[pulseInjectCnt] != pulseInjectSection[pulseInjectCnt - 1]))
            {
                pulseInjectCnt = 0;
                pulseInjRepeatDoneFlag = 2;
                PulseInject_ReJdg_Init(pulseInject);
                /* init speed buffer */
                for (index = 0u; index < PULSE_INJECTION_TIMES_M2; index++)
                {
                    pulseInjectSection[index] = 0;
                }
                return (pulseInjRepeatDoneFlag);
            }
            else
            {
            }
            pulseInjectCnt++;

            startUp->startUpElecTheta = InitPos_GetPosAngle(pulseInject);

            PulseInject_ReJdg_Init(pulseInject);
        }
    }
    else
    {
        pulseInjectCnt = 0;
        pulseInjRepeatDoneFlag = 1;
        /* init speed buffer */
        for (index = 0u; index < PULSE_INJECTION_TIMES_M2; index++)
        {
            pulseInjectSection[index] = 0;
        }
    }
#endif
    return (pulseInjRepeatDoneFlag);
}

/*!
* @brief Initial position pulse injection function main status machine, which include five status.
*        CHIP_CONFIG: pulse injection pwm and adc configuration.
*        SELF_LEARNING:  get proper pulse injection width, which is different with each motor.
*        PULSE_INJECTING: pulse injection process and result calculate.
*        CHIP_CONFIG_RESET: pulse injection finished, recover pwm and adc configuration.
*        PULSE_INJECTION_ERR: during pulse injection process, maybe occur some errors.
*
* @param[in] pwmModule: PWM instance of Motor
* @param[in] pulseInject: pulseInject data variable, pointer to PULSE_INJECT_DATA structure
* @return Initial position pulse inject function finished flag
*/
uint16_t PulseInject(uint8_t pwmModule, PULSE_INJECT_DATA *pulseInject)
{
#if 1
#if (defined MOTOR_ONE_TYPE)
    switch (pulseInject->subStatus)
    {
    case CHIP_CONFIG:
    PulseInject_Config(pwmModule, pulseInject);
    break;

    case SELF_LEARNING:
    PulseInject_SelfLearn(pulseInject);
    break;

    case PULSE_INJECTING:
    PulseInject_DataHandle(pulseInject);
    break;

    case CHIP_CONFIG_RESET:
    PulseInject_ConfigReset(pwmModule, pulseInject);
    break;

    case PULSE_INJECTION_ERR:
    pulseInject->pulseInjErrCnt = 1;
    pulseInject->subStatus = (uint8_t)CHIP_CONFIG_RESET;

    break;

    default:
    break;
    }
#endif
#endif
    return (pulseInject->pulseInjDoneFlag);
}

/*!
* @brief Initial position pulse injection function main status machine, which include five status.
*        CHIP_CONFIG: pulse injection pwm and adc configuration.
*        SELF_LEARNING:  get proper pulse injection width, which is different with each motor.
*        PULSE_INJECTING: pulse injection process and result calculate.
*        CHIP_CONFIG_RESET: pulse injection finished, recover pwm and adc configuration.
*        PULSE_INJECTION_ERR: during pulse injection process, maybe occur some errors.
*
* @param[in] pwmModule: PWM instance of Motor
* @param[in] pulseInject: pulseInject data variable, pointer to PULSE_INJECT_DATA structure
* @return Initial position pulse inject function finished flag
*/
uint16_t PulseInject_M2(uint8_t pwmModule, PULSE_INJECT_DATA *pulseInject)
{
#if 1
#if (defined MOTOR_TWO_TYPE)
    switch (pulseInject->subStatus)
    {
    case CHIP_CONFIG:
    PulseInject_Config_M2(pwmModule, pulseInject);
    break;

    case SELF_LEARNING:
    PulseInject_SelfLearn(pulseInject);
    break;

    case PULSE_INJECTING:
    PulseInject_DataHandle(pulseInject);
    break;

    case CHIP_CONFIG_RESET:
    PulseInject_ConfigReset_M2(pwmModule, pulseInject);
    break;

    case PULSE_INJECTION_ERR:
    pulseInject->pulseInjErrCnt = 1;
    pulseInject->subStatus = (uint8_t)CHIP_CONFIG_RESET;

    break;

    default:
    break;
    }
#endif
#endif
    return (pulseInject->pulseInjDoneFlag);
}

//#endif
/* =============================================  EOF  ============================================== */
