/* Copyright Statement:
*
* This software/firmware and related documentation ("AutoChips Software") are
* protected under relevant copyright laws. The information contained herein is
* confidential and proprietary to AutoChips Inc. and/or its licensors. Without
* the prior written permission of AutoChips inc. and/or its licensors, any
* reproduction, modification, use or disclosure of AutoChips Software, and
* information contained herein, in whole or in part, shall be strictly
* prohibited.
*
* AutoChips Inc. (C) 2020. All rights reserved.
*
* BY OPENING THIS FILE, RECEIVER HEREBY UNEQUIVOCALLY ACKNOWLEDGES AND AGREES
* THAT THE SOFTWARE/FIRMWARE AND ITS DOCUMENTATIONS ("AUTOCHIPS SOFTWARE")
* RECEIVED FROM AUTOCHIPS AND/OR ITS REPRESENTATIVES ARE PROVIDED TO RECEIVER
* ON AN "AS-IS" BASIS ONLY. AUTOCHIPS EXPRESSLY DISCLAIMS ANY AND ALL
* WARRANTIES, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
* WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR
* NONINFRINGEMENT. NEITHER DOES AUTOCHIPS PROVIDE ANY WARRANTY WHATSOEVER WITH
* RESPECT TO THE SOFTWARE OF ANY THIRD PARTY WHICH MAY BE USED BY,
* INCORPORATED IN, OR SUPPLIED WITH THE AUTOCHIPS SOFTWARE, AND RECEIVER AGREES
* TO LOOK ONLY TO SUCH THIRD PARTY FOR ANY WARRANTY CLAIM RELATING THERETO.
* RECEIVER EXPRESSLY ACKNOWLEDGES THAT IT IS RECEIVER'S SOLE RESPONSIBILITY TO
* OBTAIN FROM ANY THIRD PARTY ALL PROPER LICENSES CONTAINED IN AUTOCHIPS
* SOFTWARE. AUTOCHIPS SHALL ALSO NOT BE RESPONSIBLE FOR ANY AUTOCHIPS SOFTWARE
* RELEASES MADE TO RECEIVER'S SPECIFICATION OR TO CONFORM TO A PARTICULAR
* STANDARD OR OPEN FORUM. RECEIVER'S SOLE AND EXCLUSIVE REMEDY AND AUTOCHIPS'S
* ENTIRE AND CUMULATIVE LIABILITY WITH RESPECT TO THE AUTOCHIPS SOFTWARE
* RELEASED HEREUNDER WILL BE, AT AUTOCHIPS'S OPTION, TO REVISE OR REPLACE THE
* AUTOCHIPS SOFTWARE AT ISSUE, OR REFUND ANY SOFTWARE LICENSE FEES OR SERVICE
* CHARGE PAID BY RECEIVER TO AUTOCHIPS FOR SUCH AUTOCHIPS SOFTWARE AT ISSUE.
*/

/*!
* @file speed_angle.c
*
* @brief This file provides speed command ramp and speed loop processing functions.
*
*/

/* ===========================================  Includes  =========================================== */
#include "globalvar.h"
#include "motorctrl_api.h"

/* ============================================  Define  ============================================ */

/* ===========================================  Typedef  ============================================ */

/* ==========================================  Variables  =========================================== */
#if (defined MOTOR_ONE_TYPE)
int16_t g_2000msCurveTrail[(POSITION_ACTION_MAX_TIME / 10)] =
{
    0,     0,     1,     2,     3,     5,     6,     8,     11,    13,
    16,    19,    23,    27,    31,    35,    40,    44,    50,    56,
    62,    67,    73,    79,    85,    91,    96,    102,   109,   114,
    120,   126,   132,   138,   143,   149,   155,   161,   167,   173,
    178,   184,   189,   194,   199,   203,   207,   211,   215,   218,
    221,   223,   226,   228,   230,   231,   232,   233,   234,   234,
    234,   234,   234,   234,   234,   234,   234,   234,   234,   234,
    234,   234,   234,   234,   234,   234,   234,   234,   234,   234,
    234,   234,   234,   234,   234,   234,   234,   234,   234,   234,
    234,   234,   234,   234,   234,   234,   234,   234,   234,   234,
    234,   234,   234,   234,   234,   234,   234,   234,   234,   234,
    234,   234,   234,   234,   234,   234,   234,   234,   234,   234,
    234,   234,   234,   234,   234,   234,   234,   234,   234,   234,
    234,   234,   234,   234,   234,   234,   234,   234,   234,   234,
    234,   234,   233,   232,   231,   230,   228,   226,   223,   221,
    218,   215,   211,   207,   203,   199,   194,   189,   184,   178,
    173,   167,   161,   155,   149,   143,   138,   132,   126,   120,
    114,   108,   102,   97,    91,    85,    79,    73,    67,    61,
    56,    50,    45,    40,    35,    31,    27,    23,    19,    16,
    13,    11,    8,     6,     4,     3,     2,     1,     0,     0
};
#endif

#if (defined MOTOR_TWO_TYPE)
int16_t g_2000msCurveTrail_M2[(POSITION_ACTION_MAX_TIME_M2 / 10)] =
{
    0,     0,     1,     2,     3,     5,     6,     8,     11,    13,
    16,    19,    23,    27,    31,    35,    40,    44,    50,    56,
    62,    67,    73,    79,    85,    91,    96,    102,   109,   114,
    120,   126,   132,   138,   143,   149,   155,   161,   167,   173,
    178,   184,   189,   194,   199,   203,   207,   211,   215,   218,
    221,   223,   226,   228,   230,   231,   232,   233,   234,   234,
    234,   234,   234,   234,   234,   234,   234,   234,   234,   234,
    234,   234,   234,   234,   234,   234,   234,   234,   234,   234,
    234,   234,   234,   234,   234,   234,   234,   234,   234,   234,
    234,   234,   234,   234,   234,   234,   234,   234,   234,   234,
    234,   234,   234,   234,   234,   234,   234,   234,   234,   234,
    234,   234,   234,   234,   234,   234,   234,   234,   234,   234,
    234,   234,   234,   234,   234,   234,   234,   234,   234,   234,
    234,   234,   234,   234,   234,   234,   234,   234,   234,   234,
    234,   234,   233,   232,   231,   230,   228,   226,   223,   221,
    218,   215,   211,   207,   203,   199,   194,   189,   184,   178,
    173,   167,   161,   155,   149,   143,   138,   132,   126,   120,
    114,   108,   102,   97,    91,    85,    79,    73,    67,    61,
    56,    50,    45,    40,    35,    31,    27,    23,    19,    16,
    13,    11,    8,     6,     4,     3,     2,     1,     0,     0
};
#endif

/* ====================================  Functions declaration  ===================================== */
/*
* @brief S curve type judgment.

* @param[in] pScurve: pointer to SCURVE_DATA structure
* @return none
*/
void SCurve_CurveTypeJudgeProcess(SCURVE_DATA *pScurve);

/* ======================================  Functions define  ======================================== */
/*!
* @brief Tail wind and head wind initial speed judge initialize.
*
* @param[in] judgeHandle: pointer to INITSPEED_JUDGE structure
* @return none
*/
void InitSpeed_Judge_Init(INITSPEED_JUDGE *judgeHandle)
{
    memset(judgeHandle, 0, sizeof(*judgeHandle));
}

/*!
* @brief The function is called in SPEED_JUDGE state to calculate the initial speed data, enter the FLY_RUN state after completion.
*
* @param[in] judgeHandle: pointer to INITSPEED_JUDGE structure
* @param[in] focVars: pointer to FOC_VARS structure
* @return none
*/
void Speed_Judge_Process(INITSPEED_JUDGE *judgeHandle, FOC_VARS *focVars)
{
    BEMF_SAMPLE bemf;

    bemf = Get_BemfSample();

    bemf.emfUV = bemf.emfU - bemf.emfV;
    bemf.emfUW = bemf.emfU - bemf.emfW;
    judgeHandle->bemfAlpha = bemf.emfUV;
    judgeHandle->bemfBeta = MathFloat_Mpy(INV_SQRT3_Q, (bemf.emfUV + ((int32_t)bemf.emfUW << 1)));

    Speed_Judge_Calc(judgeHandle, focVars);
}

/*!
* @brief The function is called in SPEED_JUDGE state to calculate the initial speed data, enter the FLY_RUN state after completion.
*
* @param[in] judgeHandle: pointer to INITSPEED_JUDGE structure
* @param[in] focVars: pointer to FOC_VARS structure
* @return none
*/
void Speed_Judge_Process_M2(INITSPEED_JUDGE *judgeHandle, FOC_VARS *focVars)
{
    BEMF_SAMPLE bemf;

    bemf = Get_BemfSample_M2();

    bemf.emfUV = bemf.emfU - bemf.emfV;
    bemf.emfUW = bemf.emfU - bemf.emfW;
    judgeHandle->bemfAlpha = bemf.emfUV;
    judgeHandle->bemfBeta = MathFloat_Mpy(INV_SQRT3_Q, (bemf.emfUV + ((int32_t)bemf.emfUW << 1)));

    Speed_Judge_Calc(judgeHandle, focVars);
}

/*!
* @brief The per-unit value of speed targe value is calculated according to the actual value of the speed command unit rpm.
*
* @param[in] command: pointer to SPEED_RAMP_HANDLE structure
* @param[in] speedCmdRpm: Given speed command unit rpm
* @return none
*/
void ASR_SetSpeedTarget_Rpm(SPEED_RAMP_HANDLE *command, float speedCmdRpm)
{
#if (defined MOTOR_ONE_TYPE)
    command->speedTarget = (float)speedCmdRpm / MOTOR_MAX_SPEED_RPM;
#endif
}

/*!
* @brief The per-unit value of speed targe value is calculated according to the actual value of the speed command unit rpm.
*
* @param[in] command: pointer to SPEED_RAMP_HANDLE structure
* @param[in] speedCmdRpm: Given speed command unit rpm
* @return none
*/
void ASR_SetSpeedTarget_Rpm_M2(SPEED_RAMP_HANDLE *command, int16_t speedCmdRpm)
{
#if (defined MOTOR_TWO_TYPE)
    command->speedTarget = (float)speedCmdRpm / MOTOR_MAX_SPEED_RPM_M2;
#endif
}

/*!
* @brief The per-unit value of speed targe value is calculated according to the speed command unit pu.
*
* @param[in] command: pointer to SPEED_RAMP_HANDLE structure
* @param[in] speedCmdPu: Given speed command unit pu
* @return none
*/
void ASR_SetSpeedTarget_Pu(SPEED_RAMP_HANDLE *command, float speedCmdPu)
{
    command->speedTarget = speedCmdPu;
}

/*!
* @brief The per-unit value of speed ramp value is calculated according to the speed command unit pu.
*
* @param[in] command: pointer to SPEED_RAMP_HANDLE structure
* @param[in] speedCmdPu: Given speed command unit pu
* @return none
*/
void ASR_SetSpeedTargetRamp_Pu(SPEED_RAMP_HANDLE *command, float speedCmdPu)
{
    command->speedTargetRamp = speedCmdPu;
}

/*!
* @brief Get the speed target value.
*
* @param[in] speedRamp: pointer to SPEED_RAMP_HANDLE structure
* @return none
*/
float ASR_GetSpeedTarget(SPEED_RAMP_HANDLE *speedRamp)
{
    return speedRamp->speedTarget;
}

/*!
* @brief Get the feedback speed unit in Rpm.
*
* @param[in] speed: pointer to SPEED_RAMP_HANDLE structure
* @return none
*/
float ASR_GetSpeedFbk_Rpm(SPEED_RAMP_HANDLE *speed)
{
    return speed->speedFbkRpm;
}

/*!
* @brief Get the feedback speed unit in Pu.
*
* @param[in] speed: pointer to SPEED_RAMP_HANDLE structure
* @return none
*/
float ASR_GetSpeedFbk_Pu(SPEED_RAMP_HANDLE *speed)
{
    return speed->speedFbk;
}

/*!
* @brief Set the per-unit value of feedback speed.
*
* @param[in] command: pointer to SPEED_RAMP_HANDLE structure
* @param[in] speedFbk: speed feedback unit pu
* @return none
*/
void ASR_SetSpeedFbk_Pu(SPEED_RAMP_HANDLE *command, float speedFbk)
{
    command->speedFbk = speedFbk;
}

/*!
* @brief Speedloop variable PI parameters initialize, usually it is initialized only once before running.
*
* @param[in] asrVarPI: pointer to ASR_SPEEDVARPI structure
* @param[in] asrPidCof: pointer to PID_REGULATOR_COF structure
* @return none
*/
void ASR_SpeedVarPIInit(ASR_SPEEDVARPI *asrVarPI, PID_REGULATOR_COF *asrPidCof)
{
#if (defined MOTOR_ONE_TYPE)
    asrVarPI->kpH = 0.9f;
    asrVarPI->kiH = 0.002f;
    asrVarPI->kpL = 0.7f;
    asrVarPI->kiL = 0.001f;
    asrVarPI->frepuH = (30.0f / BASE_FREQ);
    asrVarPI->frepuL = (10.0f / BASE_FREQ);
    asrVarPI->kpGain = (asrVarPI->kpH - asrVarPI->kpL) / (asrVarPI->frepuH - asrVarPI->frepuL);
    asrVarPI->kiGain = (asrVarPI->kiH - asrVarPI->kiL) / (asrVarPI->frepuH - asrVarPI->frepuL);

    Asr_SetSpeed_PICof(asrVarPI->kpL, asrVarPI->kiL, 0, 0, asrPidCof);
#endif
}

/*!
* @brief Speedloop variable PI parameters initialize of Motor 2, usually it is initialized only once before running.
*
* @param[in] asrVarPI: pointer to ASR_SPEEDVARPI structure
* @param[in] asrPidCof: pointer to PID_REGULATOR_COF structure
* @return none
*/
void ASR_SpeedVarPIInit_M2(ASR_SPEEDVARPI *asrVarPI, PID_REGULATOR_COF *asrPidCof)
{
#if (defined MOTOR_TWO_TYPE)
    asrVarPI->kpH = 0.9f;
    asrVarPI->kiH = 0.002f;
    asrVarPI->kpL = 0.7f;
    asrVarPI->kiL = 0.001f;
    asrVarPI->frepuH = (30.0f / BASE_FREQ_M2);
    asrVarPI->frepuL = (10.0f / BASE_FREQ_M2);
    asrVarPI->kpGain = (asrVarPI->kpH - asrVarPI->kpL) / (asrVarPI->frepuH - asrVarPI->frepuL);
    asrVarPI->kiGain = (asrVarPI->kiH - asrVarPI->kiL) / (asrVarPI->frepuH - asrVarPI->frepuL);

    Asr_SetSpeed_PICof(asrVarPI->kpL, asrVarPI->kiL, 0, 0, asrPidCof);
#endif
}

/*!
* @brief Speedloop variable PI parameters calculate, it divides the speed loop PI parameters into three sections
         according to the speed, low speed PI is used at low speed, high speed PI is used at high speed, and the
         intermediate speed is linearly transitioned.
*
* @param[in] freq: motor electrical frequency pu value in Q15
* @param[in] asrVarPI: pointer to ASR_SPEEDVARPI structure
* @param[in] asrPidCof: pointer to PID_REGULATOR_COF structure
* @return none
*/
void ASR_SpeedVarPICalc(float freq, ASR_SPEEDVARPI *asrVarPI, PID_REGULATOR_COF *asrPidCof)
{
    float s_absFre = 0;

    s_absFre = MathFloat_Abs(freq);
    if (s_absFre > asrVarPI->frepuH)
    {
        asrPidCof->kpPu = asrVarPI->kpH;
        asrPidCof->kiPu = asrVarPI->kiH;
    }
    else if (s_absFre > asrVarPI->frepuL)
    {
        asrPidCof->kpPu = asrVarPI->kpL + MathFloat_Mpy((s_absFre - asrVarPI->frepuL), asrVarPI->kpGain);
        asrPidCof->kiPu = asrVarPI->kiL + MathFloat_Mpy((s_absFre - asrVarPI->frepuL), asrVarPI->kiGain);
    }
    else
    {
        asrPidCof->kpPu = asrVarPI->kpL;
        asrPidCof->kiPu = asrVarPI->kiL;
    }
}

/*!
* @brief Speed loop fast desaturation function, when the speed loop is saturated, the speed
         response is accelerated by weakening the speed loop integral term.
*
* @param[in] asrPid: pointer to PID_REGULATOR structure
* @param[in] acrPidQ: pointer to q axis PID_REGULATOR_OUT structure
* @return none
*/
void ASR_Quick_Desaturation(PID_REGULATOR *asrPid, PID_REGULATOR_OUT *acrPidQ)
{
#if (defined MOTOR_ONE_TYPE)
    static uint8_t windUpFlag = 0;

    if ((asrPid->pPidInput->refPu < (asrPid->pPidInput->fbkPu - 0.015f)) && (asrPid->pPidOutput->uiPu >= 0.915f))
    {
        windUpFlag = 1;
    }
    if (windUpFlag == 1)
    {
        asrPid->pPidOutput->uiPu -= 0.015f;
        asrPid->pPidCoef->kiPu = 0;
        if (MathFloat_Abs(acrPidQ->errPu) < 0.061f)
        {
            windUpFlag = 0;
        }
    }
    else
    {
        asrPid->pPidCoef->kiPu = PID_SPEED_KI_DEFAULT;
    }
#endif
}

/*!
* @brief Speed loop fast desaturation function of Motor 2, when the speed loop is saturated, the speed
         response is accelerated by weakening the speed loop integral term.
*
* @param[in] asrPid: pointer to PID_REGULATOR structure
* @param[in] acrPidQ: pointer to q axis PID_REGULATOR_OUT structure
* @return none
*/
void ASR_Quick_Desaturation_M2(PID_REGULATOR *asrPid, PID_REGULATOR_OUT *acrPidQ)
{
#if (defined MOTOR_TWO_TYPE)
    static uint8_t windUpFlag = 0;

    if ((asrPid->pPidInput->refPu < (asrPid->pPidInput->fbkPu - 0.015f)) && (asrPid->pPidOutput->uiPu >= 0.915f))
    {
        windUpFlag = 1;
    }
    if (windUpFlag == 1)
    {
        asrPid->pPidOutput->uiPu -= 0.015f;
        asrPid->pPidCoef->kiPu = 0;
        if (MathFloat_Abs(acrPidQ->errPu) < 0.061f)
        {
            windUpFlag = 0;
        }
    }
    else
    {
        asrPid->pPidCoef->kiPu = PID_SPEED_KI_DEFAULT_M2;
    }
#endif
}

/*!
* @brief Clears the estimated speed buffer.
*
* @param[in] pHandle: pointer to SPEED_RAMP_HANDLE structure
* @return none
*/
static void ASR_SpeedBufferInit(SPEED_RAMP_HANDLE *pHandle)
{
    uint8_t index = 0;
    uint8_t speedBufferSize = pHandle->speedBufferSize;

    /* init speed buffer */
    for (index = 0u; index < speedBufferSize; index++)
    {
        pHandle->speedBuffer[index] = 0;
    }

    pHandle->speedBufferIndex = 0;
}

/*!
* @brief Rotation speed Coefficient initialization.
*
* @param[in] speed: pointer to SPEED_RAMP_HANDLE structure
* @return none
*/
void ASR_RotationSpeedInit(SPEED_RAMP_HANDLE *speed)
{
#if (defined MOTOR_ONE_TYPE)
    memset(speed->speedCalc, 0, sizeof(*speed->speedCalc));
    speed->speedBufferSize = SPEED_FIFO_DEPTH;

    speed->initStatus = UNKNOWN;
    #if (defined ELEC_FREQUENCE_HIGH)
    speed->speedCalc->speedK1 = (FOC_FREQ / BASE_FREQ);
    speed->speedCalc->speedK2 = ATAN_SPEED_FILT_H;
    speed->speedCalc->speedK3 = 1.0f - ATAN_SPEED_FILT_H;
    #else
    speed->speedCalc->speedK1 = 1000.0f / BASE_FREQ;
    speed->speedCalc->speedK2 = ATAN_SPEED_FILT_L;
    speed->speedCalc->speedK3 = 1.0f - ATAN_SPEED_FILT_L;
    #endif
    ASR_SpeedBufferInit(speed);
#endif
}

/*!
* @brief Rotation speed Coefficient initialization of Motor 2.
*
* @param[in] speed: pointer to SPEED_RAMP_HANDLE structure
* @return none
*/
void ASR_RotationSpeedInit_M2(SPEED_RAMP_HANDLE *speed)
{
#if (defined MOTOR_TWO_TYPE)
    memset(speed->speedCalc, 0, sizeof(*speed->speedCalc));
    speed->speedBufferSize = SPEED_FIFO_DEPTH_M2;

    speed->initStatus = UNKNOWN;
    #if (defined ELEC_FREQUENCE_HIGH_M2)
    speed->speedCalc->speedK1 = (FOC_FREQ / BASE_FREQ);
    speed->speedCalc->speedK2 = ATAN_SPEED_FILT_H;
    speed->speedCalc->speedK3 = 1.0f - ATAN_SPEED_FILT_H;
    #else
    speed->speedCalc->speedK1 = 1000.0f / BASE_FREQ_M2;
    speed->speedCalc->speedK2 = ATAN_SPEED_FILT_L_M2;
    speed->speedCalc->speedK3 = 1.0f - ATAN_SPEED_FILT_L_M2;
    #endif
    ASR_SpeedBufferInit(speed);
#endif
}

/*!
* @brief Torque control command initialization.
*
* @param[in] torque: pointer to TORQUE_COMMAND structure
* @return none
*/
void ACR_TorqueCommandInit(TORQUE_COMMAND *torque)
{
#if (defined MOTOR_ONE_TYPE)
    memset(torque, 0, sizeof(*torque));
    torque->iqChangeStep = TORQUE_MODE_IQ_ADD_STEP;
    torque->iqMax = TORQUE_MODE_IQ_CMD_MAX;
    torque->iqMin = TORQUE_MODE_IQ_CMD_MIN;
    torque->speedLimit = MAX_SPEED_VALUE;
#endif
}

/*!
* @brief Torque control command initialization of Motor 2.
*
* @param[in] torque: pointer to TORQUE_COMMAND structure
* @return none
*/
void ACR_TorqueCommandInit_M2(TORQUE_COMMAND *torque)
{
#if (defined MOTOR_TWO_TYPE)
    memset(torque, 0, sizeof(*torque));
    torque->iqChangeStep = TORQUE_MODE_IQ_ADD_STEP_M2;
    torque->iqMax = TORQUE_MODE_IQ_CMD_MAX_M2;
    torque->iqMin = TORQUE_MODE_IQ_CMD_MIN_M2;
    torque->speedLimit = MAX_SPEED_VALUE_M2;
#endif
}

/*!
* @brief Speed ramp calculation initialization.
*
* @param[in] focVars: pointer to FOC_VARS_CFG structure
* @param[in] speed: pointer to SPEED_RAMP_HANDLE structure
* @param[in] transCfg: pointer to TRANSFORM_CFG structure
* @return none
*/
void ASR_SpeedCommandInit(FOC_VARS_CFG *focVars, SPEED_RAMP_HANDLE *speed, TRANSFORM_CFG *transCfg)
{
    speed->avgSpeedFbk = 0;
    speed->avgSpeedFbkRpm = 0;
    speed->initSpeedJudgeBemfAmp = 0;
    speed->speedTargetLimit = 0;
    speed->speedFbk = 0;
    speed->speedFbkRpm = 0;
    speed->varianceSpeed = 0;
    speed->initSpeedJudgeSpeed = 0;

    speed->speedAcceleration = (focVars->motorAcceleration / 10);
    speed->speedDeceleration = (focVars->motorDeceleration / 10);
    if (speed->speedAcceleration < 0.00003f)
    {
        speed->speedAcceleration = 0.00003f;
    }
    if (speed->speedDeceleration < 0.00003f)
    {
        speed->speedDeceleration = 0.00003f;
    }

    speed->speedMax = focVars->speedMaxLimit;
    speed->speedMin = focVars->speedMinLimit;
    speed->speedFactor = transCfg->speedFactor;
}

/*!
* @brief Position ramp calculation initialization.
*
* @param[in] focVars: pointer to FOC_VARS_CFG structure
* @param[in] positionCmd: pointer to POSITION_RAMP_HANDLE structure
* @return none
*/
void APR_PositionCommandInit(FOC_VARS_CFG *focVars, POSITION_RAMP_HANDLE *positionCmd)
{
#if (defined MOTOR_ONE_TYPE)
    memset(positionCmd, 0, sizeof(*positionCmd));
    positionCmd->actionGear = POSITION_GEAR_DEFAULT;
    positionCmd->timeMax = POSITION_ACTION_MAX_TIME;
    positionCmd->angleFactor = (float)(360.0f * POSITION_ANGLE_BASE / PULSE_PER_REVOLUTION);
#endif
}

/*!
* @brief Position ramp calculation initialization of Motor 2.
*
* @param[in] focVars: pointer to FOC_VARS_CFG structure
* @param[in] positionCmd: pointer to POSITION_RAMP_HANDLE structure
* @return none
*/
void APR_PositionCommandInit_M2(FOC_VARS_CFG *focVars, POSITION_RAMP_HANDLE *positionCmd)
{
#if (defined MOTOR_TWO_TYPE)
    memset(positionCmd, 0, sizeof(*positionCmd));
    positionCmd->actionGear = POSITION_GEAR_DEFAULT_M2;
    positionCmd->timeMax = POSITION_ACTION_MAX_TIME_M2;
    positionCmd->angleFactor = (float)(360.0f * POSITION_ANGLE_BASE_M2 / PULSE_PER_REVOLUTION_M2);
#endif
}

/*!
* @brief Speed ramp configuration.
*
* @param[in] focVars: pointer to FOC_VARS_CFG structure
* @param[in] speed: pointer to SPEED_RAMP_HANDLE structure
* @return none
*/
void ASR_SpeedCommandSet(FOC_VARS_CFG *focVars, SPEED_RAMP_HANDLE *speed)
{
#if (defined MOTOR_ONE_TYPE)
#if (defined FOC_PMSM)
    #if (defined SENSORLESS)
        #if (defined HIGH_FREQUENCE_INJECTION)
        ASR_SetSpeedTarget_Pu(speed, (HFI_INIT_FREQ * focVars->motorDir));
        ASR_SetSpeedTargetRamp_Pu(speed, (HFI_INIT_FREQ * POLE_PAIR_NUM / BASE_FREQ) * focVars->motorDir);
        #else
        ASR_SetSpeedTarget_Pu(speed, (DEFAULT_TARGET_FREQ_START * focVars->motorDir));
        ASR_SetSpeedTargetRamp_Pu(speed, (OPEN_LOOP_SPEED_HZ * POLE_PAIR_NUM / BASE_FREQ) * focVars->motorDir);
        #endif
    #elif ((defined HALL_SENSORS) || (defined ENCODER_HALL_SENSORS) || (defined ENCODER_SENSORS))
        ASR_SetSpeedTarget_Pu(speed, (DEFAULT_TARGET_FREQ_START * focVars->motorDir));
        ASR_SetSpeedTargetRamp_Pu(speed, 0);
    #else
    #endif
#elif (defined FOC_ACIM)
//    ASR_SetSpeedTarget_Pu(speed, (DEFAULT_TARGET_FREQ_START * focVars->motorDir));
//    ASR_SetSpeedTargetRamp_Pu(speed, 0);
#endif
#endif
}

/*!
* @brief Speed ramp configuration of Motor 2.
*
* @param[in] focVars: pointer to FOC_VARS_CFG structure
* @param[in] speed: pointer to SPEED_RAMP_HANDLE structure
* @return none
*/
void ASR_SpeedCommandSet_M2(FOC_VARS_CFG *focVars, SPEED_RAMP_HANDLE *speed)
{
#if (defined MOTOR_TWO_TYPE)
#if (defined FOC_PMSM_M2)
    #if (defined SENSORLESS_M2)
        #if (defined HIGH_FREQUENCE_INJECTION_M2)
        ASR_SetSpeedTarget_Pu(speed, (HFI_INIT_FREQ_M2 * focVars->motorDir));
        ASR_SetSpeedTargetRamp_Pu(speed, (HFI_INIT_FREQ * POLE_PAIR_NUM_M2 / BASE_FREQ_M2) * focVars->motorDir);
        #else
        ASR_SetSpeedTarget_Pu(speed, (DEFAULT_TARGET_FREQ_START_M2 * focVars->motorDir));
        ASR_SetSpeedTargetRamp_Pu(speed, (OPEN_LOOP_SPEED_HZ_M2 * POLE_PAIR_NUM_M2 / BASE_FREQ_M2) * focVars->motorDir);
        #endif
    #elif ((defined HALL_SENSORS_M2) || (defined ENCODER_HALL_SENSORS_M2) || (defined ENCODER_SENSORS_M2))
        ASR_SetSpeedTarget_Pu(speed, (DEFAULT_TARGET_FREQ_START_M2 * focVars->motorDir));
        ASR_SetSpeedTargetRamp_Pu(speed, 0);
    #else
    #endif
#elif (defined FOC_ACIM_M2)
//    ASR_SetSpeedTarget_Pu(speed, (DEFAULT_TARGET_FREQ_START_M2 * focVars->motorDir));
//    ASR_SetSpeedTargetRamp_Pu(speed, 0);
#endif
#endif
}

/*!
* @brief Limit the max and min value of speed target.
*
* @param[in] motorDir: motor rotation direction
* @param[in] speedRamp: pointer to SPEED_RAMP_HANDLE structure
* @return none
*/
void ASR_SpeedTargetLimit(int16_t motorDir, SPEED_RAMP_HANDLE *speedRamp)
{
    if (motorDir == FORWARD_ROTATE)
    {
        if (speedRamp->speedTarget < speedRamp->speedMin)
        {
            speedRamp->speedTarget = speedRamp->speedMin;
        }
        else if (speedRamp->speedTarget > speedRamp->speedMax)
        {
            speedRamp->speedTarget = speedRamp->speedMax;
        }
        else
        {
        }
    }
    else if (motorDir == REVERSE_ROTATE)
    {
        if (speedRamp->speedTarget > -(speedRamp->speedMin))
        {
            speedRamp->speedTarget = -(speedRamp->speedMin);
        }
        else if (speedRamp->speedTarget < -(speedRamp->speedMax))
        {
            speedRamp->speedTarget = -(speedRamp->speedMax);
        }
        else
        {
        }
    }
    else
    {
    }
}

/*!
* @brief Speed ramp calculation. calculate the ramp output according to the speed target value.
*
* @param[in] focVars: pointer to FOC_VARS_CFG structure
* @param[in] speedRamp: pointer to SPEED_RAMP_HANDLE structure
* @return none
*/
void ASR_SpeedTargetRampCalc(FOC_VARS_CFG *focVars, SPEED_RAMP_HANDLE *speedRamp)
{
    ASR_SpeedTargetLimit(focVars->motorDir, speedRamp);

    speedRamp->countSpdCmd++;
    /* calculate speed ramp every 10ms, 9 = 10 - 1, s_countSpdCmd range is 1~10, then back to 0, and becomes to 1 after '++' */
    if (speedRamp->countSpdCmd > 9)
    {
        speedRamp->countSpdCmd = 0;
        if (focVars->motorDir == FORWARD_ROTATE)
        {
            if ((speedRamp->speedTarget - speedRamp->speedTargetRamp) > speedRamp->speedAcceleration)
            {
                speedRamp->speedTargetRamp += speedRamp->speedAcceleration;
            }
            else if ((speedRamp->speedTargetRamp - speedRamp->speedTarget) > speedRamp->speedDeceleration)
            {
                speedRamp->speedTargetRamp -= speedRamp->speedDeceleration;
            }
            else
            {
                speedRamp->speedTargetRamp = speedRamp->speedTarget;
            }
        }
        else if (focVars->motorDir == REVERSE_ROTATE)
        {
            if ((speedRamp->speedTargetRamp - speedRamp->speedTarget) > speedRamp->speedAcceleration)
            {
                speedRamp->speedTargetRamp -= speedRamp->speedAcceleration;
            }
            else if ((speedRamp->speedTarget - speedRamp->speedTargetRamp) > speedRamp->speedDeceleration)
            {
                speedRamp->speedTargetRamp += speedRamp->speedDeceleration;
            }
            else
            {
                speedRamp->speedTargetRamp = speedRamp->speedTarget;
            }
        }
        else
        {
        }
    }
}

/*!
* @brief Update action direction and adjust value according to target angle.
*
* @param[in] positionCmd: pointer to POSITION_RAMP_HANDLE structure
* @param[in] encoder: pointer to ENCODER_TYPE structure
* @return none
*/
void PositionCommandUpdate(POSITION_RAMP_HANDLE *positionCmd, ENCODER_TYPE *encoder)
{
#if (defined MOTOR_ONE_TYPE)
#if (defined POSITION_LOOP_CTRL)

//#if (defined ENCODER_HALL_SENSORS)
//    positionCmd->angleFbkPulse = encoder->encoderAngleCnt;
//#else
    positionCmd->angleFbkPulse = (encoder->encoderAngleCnt + encoder->encoderCntOffSet);
//#endif

    positionCmd->angleFbk = positionCmd->angleFbkPulse / POSITION_ANGLE_BASE;
    positionCmd->angleTargetPulse = positionCmd->angleTarget * POSITION_ANGLE_BASE;
    positionCmd->angleFbkDegree = MathFloat_Mpy(positionCmd->angleFactor, positionCmd->angleFbk);

    if (positionCmd->angleTarget != positionCmd->angleTargetBuf)
    {
        if (positionCmd->angleTarget != positionCmd->angleTargetRamp)
        {
            if (positionCmd->angleTarget > positionCmd->angleTargetRamp)
            {
                positionCmd->actionDirection = ANGLE_ACC;
            }
            else
            {
                positionCmd->actionDirection = ANGLE_DEC;
            }
            /* angleAdjust value has include angle adjust direction and sign */
            positionCmd->angleAdjust = (positionCmd->angleTarget - positionCmd->angleTargetRamp);
        }
        else
        {
            positionCmd->angleAdjust = 0;
            positionCmd->actionDirection = ANGLE_HOLD;
        }
        positionCmd->angleTimeCnt = 0;
        positionCmd->angleDelta = 0;
    }

    positionCmd->angleTargetBuf = positionCmd->angleTarget;
#endif
#endif
}

/*!
* @brief Update action direction and adjust value according to target angle of Motor 2.
*
* @param[in] positionCmd: pointer to POSITION_RAMP_HANDLE structure
* @param[in] encoder: pointer to ENCODER_TYPE structure
* @return none
*/
void PositionCommandUpdate_M2(POSITION_RAMP_HANDLE *positionCmd, ENCODER_TYPE *encoder)
{
#if (defined MOTOR_TWO_TYPE)
#if (defined POSITION_LOOP_CTRL_M2)

//#if (defined ENCODER_HALL_SENSORS_M2)
//    positionCmd->angleFbkPulse = encoder->encoderAngleCnt;
//#else
    positionCmd->angleFbkPulse = (encoder->encoderAngleCnt + encoder->encoderCntOffSet);
//#endif

    positionCmd->angleFbk = positionCmd->angleFbkPulse / POSITION_ANGLE_BASE_M2;
    positionCmd->angleTargetPulse = positionCmd->angleTarget * POSITION_ANGLE_BASE_M2;
    positionCmd->angleFbkDegree = MathFloat_Mpy(positionCmd->angleFactor, positionCmd->angleFbk);

    if (positionCmd->angleTarget != positionCmd->angleTargetBuf)
    {
        if (positionCmd->angleTarget != positionCmd->angleTargetRamp)
        {
            if (positionCmd->angleTarget > positionCmd->angleTargetRamp)
            {
                positionCmd->actionDirection = ANGLE_ACC;
            }
            else
            {
                positionCmd->actionDirection = ANGLE_DEC;
            }
            /* angleAdjust value has include angle adjust direction and sign */
            positionCmd->angleAdjust = (positionCmd->angleTarget - positionCmd->angleTargetRamp);
        }
        else
        {
            positionCmd->angleAdjust = 0;
            positionCmd->actionDirection = ANGLE_HOLD;
        }
        positionCmd->angleTimeCnt = 0;
        positionCmd->angleDelta = 0;
    }

    positionCmd->angleTargetBuf = positionCmd->angleTarget;
#endif
#endif
}

/*!
* @brief Position ramp calculation. calculate the ramp output according to the position target value.
*
* @param[in] positionRamp: pointer to POSITION_RAMP_HANDLE structure
* @return none
*/
void APR_PositionTargetRampCalc(POSITION_RAMP_HANDLE *positionRamp)
{
#if (defined MOTOR_ONE_TYPE)
#if (defined POSITION_LOOP_CTRL)
    if (positionRamp->actionDirection != ANGLE_HOLD)
    {
        switch (positionRamp->actionGear)
        {
            case SPEED_GEAR0:
                positionRamp->angleDelta = g_2000msCurveTrail[positionRamp->angleTimeCnt];
            break;

            case SPEED_GEAR1:
                positionRamp->angleDelta = g_2000msCurveTrail[positionRamp->angleTimeCnt] \
                                            + g_2000msCurveTrail[positionRamp->angleTimeCnt + 1];
            break;

            case SPEED_GEAR2:
                positionRamp->angleDelta = g_2000msCurveTrail[positionRamp->angleTimeCnt] \
                                            + g_2000msCurveTrail[positionRamp->angleTimeCnt + 1] \
                                            + g_2000msCurveTrail[positionRamp->angleTimeCnt + 2] \
                                            + g_2000msCurveTrail[positionRamp->angleTimeCnt + 3];
            break;

            default:
            break;
        }
        /* angle adjust direction and sign is indicate by angleAdjust,angleDelta always positive. */
        if (positionRamp->actionDirection == ANGLE_ACC)
        {
            if (positionRamp->angleTarget > positionRamp->angleTargetRamp)
            {
                positionRamp->angleTargetRamp += MathFloat_Mpy(positionRamp->angleDelta, positionRamp->angleAdjust);
            }
            else
            {
                positionRamp->angleTargetRamp = positionRamp->angleTarget;
                positionRamp->angleAdjust = 0;
                positionRamp->actionDirection = ANGLE_HOLD;
            }
        }
        else
        {
            if (positionRamp->angleTargetRamp > positionRamp->angleTarget)
            {
                positionRamp->angleTargetRamp += MathFloat_Mpy(positionRamp->angleDelta, positionRamp->angleAdjust);
            }
            else
            {
                positionRamp->angleTargetRamp = positionRamp->angleTarget;
                positionRamp->angleAdjust = 0;
                positionRamp->actionDirection = ANGLE_HOLD;
            }
        }
        /* 10 means Scurve trail update every 10 ms. */
        if (positionRamp->angleTimeCnt < ((positionRamp->timeMax / 10) - (1 << positionRamp->actionGear)))
        {
            positionRamp->angleTimeCnt += (1 << positionRamp->actionGear);
        }
        else
        {
            positionRamp->angleTimeCnt = 0;
            positionRamp->angleTargetRamp = positionRamp->angleTarget;
            positionRamp->angleAdjust = 0;
            positionRamp->actionDirection = ANGLE_HOLD;
        }
    }
#endif
#endif
}

/*!
* @brief Position ramp calculation of Motor 2. calculate the ramp output according to the position target value.
*
* @param[in] positionRamp: pointer to POSITION_RAMP_HANDLE structure
* @return none
*/
void APR_PositionTargetRampCalc_M2(POSITION_RAMP_HANDLE *positionRamp)
{
#if (defined MOTOR_TWO_TYPE)
#if (defined POSITION_LOOP_CTRL_M2)
    if (positionRamp->actionDirection != ANGLE_HOLD)
    {
        switch (positionRamp->actionGear)
        {
            case SPEED_GEAR0:
                positionRamp->angleDelta = g_2000msCurveTrail_M2[positionRamp->angleTimeCnt];
            break;

            case SPEED_GEAR1:
                positionRamp->angleDelta = g_2000msCurveTrail_M2[positionRamp->angleTimeCnt] \
                                            + g_2000msCurveTrail_M2[positionRamp->angleTimeCnt + 1];
            break;

            case SPEED_GEAR2:
                positionRamp->angleDelta = g_2000msCurveTrail_M2[positionRamp->angleTimeCnt] \
                                            + g_2000msCurveTrail_M2[positionRamp->angleTimeCnt + 1] \
                                            + g_2000msCurveTrail_M2[positionRamp->angleTimeCnt + 2] \
                                            + g_2000msCurveTrail_M2[positionRamp->angleTimeCnt + 3];
            break;

            default:
            break;
        }
        /* angle adjust direction and sign is indicate by angleAdjust,angleDelta always positive. */
        if (positionRamp->actionDirection == ANGLE_ACC)
        {
            if (positionRamp->angleTarget > positionRamp->angleTargetRamp)
            {
                positionRamp->angleTargetRamp += MathFloat_Mpy(positionRamp->angleDelta, positionRamp->angleAdjust);
            }
            else
            {
                positionRamp->angleTargetRamp = positionRamp->angleTarget;
                positionRamp->angleAdjust = 0;
                positionRamp->actionDirection = ANGLE_HOLD;
            }
        }
        else
        {
            if (positionRamp->angleTargetRamp > positionRamp->angleTarget)
            {
                positionRamp->angleTargetRamp += MathFloat_Mpy(positionRamp->angleDelta, positionRamp->angleAdjust);
            }
            else
            {
                positionRamp->angleTargetRamp = positionRamp->angleTarget;
                positionRamp->angleAdjust = 0;
                positionRamp->actionDirection = ANGLE_HOLD;
            }
        }
        /* 10 means Scurve trail update every 10 ms. */
        if (positionRamp->angleTimeCnt < ((positionRamp->timeMax / 10) - (1 << positionRamp->actionGear)))
        {
            positionRamp->angleTimeCnt += (1 << positionRamp->actionGear);
        }
        else
        {
            positionRamp->angleTimeCnt = 0;
            positionRamp->angleTargetRamp = positionRamp->angleTarget;
            positionRamp->angleAdjust = 0;
            positionRamp->actionDirection = ANGLE_HOLD;
        }
    }
#endif
#endif
}

/*!
* @brief Stores estimated rotation speed to FIFO.
*
* @param[in] speed: pointer to SPEED_RAMP_HANDLE structure
* @return none
*/
void ASR_StoreRotorSpeed(SPEED_RAMP_HANDLE *speed)
{
    uint8_t buffer_index = speed->speedBufferIndex;

    buffer_index++;
    if (buffer_index == speed->speedBufferSize)
    {
        buffer_index = 0u;
        speed->speedBufferReady = 1u;
    }
    else
    {
        speed->speedBufferReady = 0u;
    }

    speed->speedBuffer[buffer_index] = speed->speedFbk;
    speed->speedBufferIndex = buffer_index;
}

/*!
* @brief Calculated average estimated rotation speed and speed variance value from FIFO.
*
* @param[in] speed: pointer to SPEED_RAMP_HANDLE structure
* @return none
*/
void ASR_AvgSpeedCalc(SPEED_RAMP_HANDLE *speed)
{
    uint8_t index = 0;
    int32_t speedError = 0;
    float avgSpeedSum = 0;
    float quadraticErrorSum = 0;

    ASR_StoreRotorSpeed(speed);
    if (speed->speedBufferReady == 1)
    {
        for (index = 0u; index < speed->speedBufferSize; index++)
        {
            avgSpeedSum += (float)(speed->speedBuffer[index]);
        }
        speed->avgSpeedFbk = avgSpeedSum / speed->speedBufferSize;
        speed->avgSpeedFbkRpm = MathFloat_Mpy(speed->speedFactor, speed->avgSpeedFbk);
        for (index = 0u; index < speed->speedBufferSize; index++)
        {
            speedError = (float)(speed->speedBuffer[index]) - speed->avgSpeedFbk;
            speedError = (speedError * speedError);
            quadraticErrorSum += speedError;
        }

        /* It computes the measurement variance */
        speed->varianceSpeed = quadraticErrorSum / speed->speedBufferSize;
    }
}

/*!
* @brief Get the speed ramp value. Assign to speed loop reference value.
*
* @param[in] speedRamp: pointer to SPEED_RAMP_HANDLE structure
* @return speedTargetRamp: speed targe ramp value
*/
float ASR_GetSpeedRamp(SPEED_RAMP_HANDLE *speedRamp)
{
    return speedRamp->speedTargetRamp;
}

/*!
* @brief Get the speed ramp value from the knob on the board.
*
* @param[in] speed: pointer to SPEED_RAMP_HANDLE structure
* @return none
*/
void ASR_GetSpeedRampWithKnob(SPEED_RAMP_HANDLE *speed)
{
#if (defined MOTOR_ONE_TYPE)
    int32_t temp = 0;
    float temp1 = 0;
    static int32_t knobValue[8];
    static uint8_t i = 0;
    static int32_t sum = 0;

    if (i < 8)
    {
        knobValue[i] = (g_ADC1RegularBuffer[1] + g_ADC1RegularBuffer[3]) >> 1;
        sum += knobValue[i];
        i++;
    }
    else
    {
        temp = pFocVarsCfg->motorDir * (sum >> 3);
        temp1 = temp * speed->speedMax / 4096.0f;
        speed->speedTarget = temp1;
        if (MathFloat_Abs(temp1) < MIN_SPEED_VALUE)
        {
            speed->speedTarget = pFocVarsCfg->motorDir * speed->speedMin;
        }
        if (MathFloat_Abs(temp1) > MAX_SPEED_VALUE)
        {
            speed->speedTarget = pFocVarsCfg->motorDir * speed->speedMax;
        }

        i = 0;
        sum = 0;
        knobValue[0] = knobValue[1] = knobValue[2] = knobValue[3] = knobValue[4] = knobValue[5] = knobValue[6] = knobValue[7] = 0;
    }

#endif
}

/*!
* @brief Get the speed ramp value from the knob on the board.
*
* @param[in] speed: pointer to SPEED_RAMP_HANDLE structure
* @return none
*/
void ASR_GetSpeedRampWithKnob_M2(SPEED_RAMP_HANDLE *speed)
{
#if (defined MOTOR_TWO_TYPE)
    int32_t temp = 0;
    float temp1 = 0;
    static int32_t knobValue[8];
    static uint8_t i = 0;
    static int32_t sum = 0;

    if (i < 8)
    {
        knobValue[i] = (g_ADCRegularBuffer[3] + g_ADCRegularBuffer[8]) >> 1;
        sum += knobValue[i];
        i++;
    }
    else
    {
        temp = pFocVarsCfg_M2->motorDir * (sum >> 3);
        temp1 = temp * speed->speedMax / 4096.0f;
        speed->speedTarget = temp1;
        if (Math_Qabs(temp1) < MIN_SPEED_VALUE_M2)
        {
            speed->speedTarget = pFocVarsCfg_M2->motorDir * speed->speedMin;
        }
        if (Math_Qabs(temp1) > MAX_SPEED_VALUE_M2)
        {
            speed->speedTarget = pFocVarsCfg_M2->motorDir * speed->speedMax;
        }

        i = 0;
        sum = 0;
        knobValue[0] = knobValue[1] = knobValue[2] = knobValue[3] = knobValue[4] = knobValue[5] = knobValue[6] = knobValue[7] = 0;
    }

#endif
}

/*!
* @brief Speed loop processing of motor control.
*
* @param[in] speed: pointer to SPEED_RAMP_HANDLE structure
* @param[in] pid: pointer to PID_REGULATOR structure
* @param[in] focVars: pointer to FOC_VARS_CTRL structure
* @return none
*/
void ASR_SpeedLoop_Process(SPEED_RAMP_HANDLE *speed, PID_REGULATOR *pid, FOC_VARS_CTRL *focVars)
{
    Set_PID_Ref(pid, speed->speedTargetRamp);
    Set_PID_Fbk(pid, speed->speedFbk);
    focVars->iqRef = PID_ParallelRegulator(pid);
}

/*!
* @brief Position loop processing of motor control.
*
* @param[in] position: pointer to POSITION_RAMP_HANDLE structure
* @param[in] pid: pointer to PID_REGULATOR structure
* @param[in] focVars: pointer to FOC_VARS_CTRL structure
* @return none
*/
void APR_PositionLoop_Process(POSITION_RAMP_HANDLE *position, PID_REGULATOR *pid, FOC_VARS_CTRL *focVars)
{
    Set_PID_Ref(pid, position->angleTargetRamp);
    Set_PID_Fbk(pid, position->angleFbk);
    focVars->iqRef = PID_ParallelRegulator(pid);
}

/*!
* @brief Update the feedback speed of motor in different modes(Hall/Encode/sensorless).
*
* @param[in] command: pointer to SPEED_RAMP_HANDLE structure
* @return none
*/
void ASR_SpeedFbk_Update(SPEED_RAMP_HANDLE *command)
{
#if (defined MOTOR_ONE_TYPE)
    if (g_paramIdentifyCtrl.identifyCtrl == 1)
    {
        ASR_SetSpeedFbk_Pu(command, g_paramFluxIdentify.freSet);
    }
    else
    {
#if (defined FOC_PMSM)
    #if (defined SENSORLESS)
        #if (defined HIGH_FREQUENCE_INJECTION)
        ASR_SetSpeedFbk_Pu(command, g_hfinj.weightFreq);
        #else
        #if (defined PLL_ANGLE_ESTIMATION)
        ASR_SetSpeedFbk_Pu(command, g_pidPll.pPidOutput->outPuFliter);
        #else
        ASR_SetSpeedFbk_Pu(command, command->speedCalc->estSpeed);
        #endif
        #endif
    #elif (defined HALL_SENSORS)
        ASR_SetSpeedFbk_Pu(command, g_hall.pHallSensor->estFre);
//    #elif (defined ENCODER_HALL_SENSORS)
//        ASR_SetSpeedFbk_Pu(command, g_rotationalSpeed.estSpeed);
    #else
        #if (defined MT_SPEED_MEASURE)
        ASR_SetSpeedFbk_Pu(command, g_encoderSpeed.mtSpeed);
        #else
        ASR_SetSpeedFbk_Pu(command, command->speedCalc->estSpeed);
        #endif
    #endif
//#elif (defined FOC_ACIM)
//        ASR_SetSpeedFbk_Pu(command, g_acimFocCtrl.speed->rotorSpeed);
#else
        ASR_SetSpeedFbk_Pu(command, command->speedCalc->estSpeed);
#endif
    }
    command->speedFbkRpm = MathFloat_Mpy(command->speedFactor, command->speedFbk);
#endif
}

/*!
* @brief Update the feedback speed of motor in different modes(Hall/Encode/sensorless).
*
* @param[in] command: pointer to SPEED_RAMP_HANDLE structure
* @return none
*/
void ASR_SpeedFbk_Update_M2(SPEED_RAMP_HANDLE *command)
{
#if (defined MOTOR_TWO_TYPE)
    if (g_paramIdentifyCtrl_M2.identifyCtrl == 1)
    {
        ASR_SetSpeedFbk_Pu(command, g_paramFluxIdentify_M2.freSet);
    }
    else
    {
#if (defined FOC_PMSM_M2)
    #if (defined SENSORLESS_M2)
        #if (defined HIGH_FREQUENCE_INJECTION_M2)
        ASR_SetSpeedFbk_Pu(command, g_hfinj_M2.weightFreq);
        #else
        #if (defined PLL_ANGLE_ESTIMATION_M2)
        ASR_SetSpeedFbk_Pu(command, g_pidPll_M2.pPidOutput->outPuFliter);
        #else
        ASR_SetSpeedFbk_Pu(command, command->speedCalc->estSpeed);
        #endif
        #endif
    #elif (defined HALL_SENSORS_M2)
        ASR_SetSpeedFbk_Pu(command, g_hall_M2.pHallSensor->estFre);
//    #elif (defined ENCODER_HALL_SENSORS_M2)
//        ASR_SetSpeedFbk_Pu(command, g_rotationalSpeed.estSpeed);
    #else
        #if (defined MT_SPEED_MEASURE_M2)
        ASR_SetSpeedFbk_Pu(command, g_encoderSpeed_M2.mtSpeed);
        #else
        ASR_SetSpeedFbk_Pu(command, command->speedCalc->estSpeed);
        #endif
    #endif
//#elif (defined FOC_ACIM_M2)
//        ASR_SetSpeedFbk_Pu(command, g_acimFocCtrl.speed->rotorSpeed);
#else
        ASR_SetSpeedFbk_Pu(command, command->speedCalc->estSpeed);
#endif
    }
    command->speedFbkRpm = MathFloat_Mpy(command->speedFactor, command->speedFbk);
#endif
}

/*!
* @brief Get electric angle of induction motor with sensor.
*
* @param[in] acimFoc: pointer to ACIM_FOC_CTRL structure
* @return induction motor synchronous electric angle
*/
float Get_AcimSensorElecAngle(ACIM_FOC_CTRL *acimFoc)
{
    return (acimFoc->flux->rotorFluxAngle);
}

/*!
* @brief Get electric angle according to the different sensors.
*
* @param[in] focVars: pointer to FOC_VARS_CTRL structure
* @return none
*/
void Get_ElecAngle(FOC_VARS_CTRL *focVars)
{
#if (defined MOTOR_ONE_TYPE)
    if (g_paramIdentifyCtrl.identifyCtrl == 1)
    {
        focVars->elecAngle = Get_ParamIdentify_ElecAngle(&g_paramIdentifyCtrl);
    }
    else
    {
#if (defined FOC_PMSM)
    #if (defined HALL_SENSORS)
        focVars->elecAngle = Get_HALL_ElecAngle(&g_hall);
//    #elif (defined ENCODER_HALL_SENSORS)
//        if (defHallSensor.hallSelfLearnEnable == ENABLE)
//        {
//            focVars->elecAngle = Get_HALL_ElecAngle(&g_hall);
//        }
//        else
//        {
//            focVars->elecAngle = EncoderHall_GetElecAngle(&g_encoderHallSensor);
//        }
    #elif (defined ENCODER_SENSORS)
        focVars->elecAngle = Encoder_GetElecAngle(&g_encoderSensor);
    #elif (defined SENSORLESS)
        #if (defined HIGH_FREQUENCE_INJECTION)
        #else
        focVars->elecAngle = Get_Sensorless_ElecAngle(&g_startUpCtrl);
        #endif
    #else
    #endif
//#elif (defined FOC_ACIM)
//    #if (defined SENSORLESS_ACIM)
//        focVars->elecAngle = Get_Sensorless_ElecAngle(&g_startUpCtrl);
//    #elif (defined SENSOR_ACIM)
//        focVars->elecAngle = Get_AcimSensorElecAngle(&g_acimFocCtrl);
//    #endif
#endif
    }
#endif
}

/*!
* @brief Get electric angle according to the different sensors.
*
* @param[in] focVars: pointer to FOC_VARS_CTRL structure
* @return none
*/
void Get_ElecAngle_M2(FOC_VARS_CTRL *focVars)
{
#if (defined MOTOR_TWO_TYPE)
    if (g_paramIdentifyCtrl_M2.identifyCtrl == 1)
    {
        focVars->elecAngle = Get_ParamIdentify_ElecAngle(&g_paramIdentifyCtrl_M2);
    }
    else
    {
#if (defined FOC_PMSM_M2)
    #if (defined HALL_SENSORS_M2)
        focVars->elecAngle = Get_HALL_ElecAngle(&g_hall_M2);
//    #elif (defined ENCODER_HALL_SENSORS_M2)
//        if (defHallSensor_M2.hallSelfLearnEnable == ENABLE)
//        {
//            focVars->elecAngle = Get_HALL_ElecAngle(&g_hall_M2);
//        }
//        else
//        {
//            focVars->elecAngle = EncoderHall_GetElecAngle(&g_encoderHallSensor_M2);
//        }
    #elif (defined ENCODER_SENSORS_M2)
        focVars->elecAngle = Encoder_GetElecAngle(&g_encoderSensor_M2);
    #elif (defined SENSORLESS_M2)
        #if (defined HIGH_FREQUENCE_INJECTION_M2)
        #else

        focVars->elecAngle = Get_Sensorless_ElecAngle(&g_startUpCtrl_M2);

        #endif
    #else
    #endif
//#elif (defined FOC_ACIM_M2)
//    #if (defined SENSORLESS_ACIM_M2)
//        focVars->elecAngle = Get_Sensorless_ElecAngle(&g_startUpCtrl);
//    #elif (defined SENSOR_ACIM_M2)
//        focVars->elecAngle = Get_AcimSensorElecAngle(&g_acimFocCtrl);
//    #endif
#endif
    }
#endif
}

/*!
* @brief Structure variables initialization.
*
* @param[in] pScurve: pointer to SCURVE structure
* @return none
*/
void SCurve_InitProcess(SCURVE *pScurve)
{
    SCURVE_DATA *sCurveCtrl = pScurve->scurveCtrl;
    SCURVE_CFG *sCurveCfg = pScurve->scurveCfg;

    memset(sCurveCtrl, 0, sizeof(*sCurveCtrl));

    sCurveCtrl->accStartRatio = sCurveCfg->accStartRatio;
    sCurveCtrl->accEndRatio = sCurveCfg->accEndRatio;
    sCurveCtrl->decStartRatio = sCurveCfg->decStartRatio;
    sCurveCtrl->decEndRatio = sCurveCfg->decEndRatio;

    if ((sCurveCtrl->accStartRatio + sCurveCtrl->accEndRatio) > 1.0f)
    {
        sCurveCtrl->accEndRatio = 1.0f - sCurveCtrl->accStartRatio;
    }
    if ((sCurveCtrl->decStartRatio + sCurveCtrl->decEndRatio) > 1.0f)
    {
        sCurveCtrl->decEndRatio = 1.0f - sCurveCtrl->decStartRatio;
    }

    SCurve_CurveTypeJudgeProcess(sCurveCtrl);
}

/*!
* @brief Motor motion mode judgment in S-curve ramp of Motor 1.
*
* @param[in] pScurve: pointer to SCURVE_DATA structure
* @param[in] speedRamp: pointer to SPEED_RAMP_HANDLE structure
* @param[in] startCurve: pointer to STARTUP_CURVE structure
* @return none
*/
void SCurve_MotionTypeJudgeProcess(SCURVE_DATA *pScurve, SPEED_RAMP_HANDLE *speedRamp, STARTUP_CURVE *startCurve)
{
    #if (defined SENSORLESS)
    pScurve->target = MathFloat_Sat(MathFloat_Abs(speedRamp->speedTarget), speedRamp->speedMax, speedRamp->speedMin);
    #else
    pScurve->target = speedRamp->speedTarget;
    #endif
    pScurve->timeCnt++;
    if (pScurve->target != pScurve->targetBuf)
    {
        pScurve->curveEndFlag = 0U;

        if (pScurve->target > pScurve->freqOutput)
        {
            #if (defined SENSORLESS)
            if ((pScurve->freqOutput == 0.0f) && (pScurve->target < startCurve->curveOut->frqRefValue))
            {
                pScurve->motionType = SCURVE_MOTION_DEC;
                pScurve->freqStart = startCurve->curveOut->frqRefValue;
                pScurve->freqEnd = pScurve->target;
                pScurve->freqDelta = pScurve->freqStart - pScurve->freqEnd;
                pScurve->decDuration = (uint32_t)(pScurve->freqDelta * 5.0f / speedRamp->speedDeceleration);  /* 5: S curve calculated in 2ms */
                pScurve->timeCnt = 0U;
            }
            else if ((pScurve->freqOutput == 0.0f) && (pScurve->target > startCurve->curveOut->frqRefValue))
            {
                pScurve->motionType = SCURVE_MOTION_ACC;
                pScurve->freqStart = startCurve->curveOut->frqRefValue;
                pScurve->freqEnd = pScurve->target;
                pScurve->freqDelta = pScurve->freqEnd - pScurve->freqStart;
                pScurve->accDuration = (uint32_t)(pScurve->freqDelta * 5.0f / speedRamp->speedAcceleration);
                pScurve->timeCnt = 0U;
            }
            else if ((pScurve->freqOutput == 0.0f) && (pScurve->target == startCurve->curveOut->frqRefValue))
            {
                pScurve->motionType = SCURVE_MOTION_UNIF;
                pScurve->timeCnt = 0U;
                pScurve->freqStart = pScurve->target;
                pScurve->freqEnd = pScurve->target;
            }
            else if (pScurve->freqOutput != 0.0f)
            {
                pScurve->motionType = SCURVE_MOTION_ACC;
                if (pScurve->curveCalcFlag == 1U)
                {
                    pScurve->freqStart = pScurve->freqOutput;
                    pScurve->freqEnd = pScurve->target;
                }
                else
                {
                    pScurve->freqStart = pScurve->targetBuf;
                    pScurve->freqEnd = pScurve->target;
                }
                pScurve->freqDelta = pScurve->freqEnd - pScurve->freqStart;
                pScurve->accDuration = (uint32_t)(pScurve->freqDelta * 5.0f / speedRamp->speedAcceleration);
                pScurve->timeCnt = 0U;
            }
            else
            {
            }
            #else
            pScurve->motionType = SCURVE_MOTION_ACC;
            if (pScurve->curveCalcFlag == 1U)
            {
                pScurve->freqStart = pScurve->freqOutput;
                pScurve->freqEnd = pScurve->target;
            }
            else
            {
                pScurve->freqStart = pScurve->targetBuf;
                pScurve->freqEnd = pScurve->target;
            }
            if (pScurve->freqEnd > speedRamp->speedMax)
            {
                pScurve->freqEnd = speedRamp->speedMax;
            }
            pScurve->freqDelta = pScurve->freqEnd - pScurve->freqStart;
            pScurve->accDuration = (uint32_t)(pScurve->freqDelta * 5.0f / speedRamp->speedAcceleration);
            pScurve->timeCnt = 0U;
            #endif
        }
        else if (pScurve->target < pScurve->freqOutput)
        {
            pScurve->motionType = SCURVE_MOTION_DEC;
            if (pScurve->curveCalcFlag == 1U)
            {
                pScurve->freqStart = pScurve->freqOutput;
                pScurve->freqEnd = pScurve->target;
            }
            else
            {
                pScurve->freqStart = pScurve->targetBuf;
                pScurve->freqEnd = pScurve->target;
            }
            #if (defined SENSORLESS)
            if (pScurve->freqStart < speedRamp->speedMin)
            {
                pScurve->freqStart = speedRamp->speedMin;
            }
            #endif
            if (pScurve->freqEnd > speedRamp->speedMax)
            {
                pScurve->freqEnd = speedRamp->speedMax;
            }
            pScurve->freqDelta = pScurve->freqStart - pScurve->freqEnd;
            pScurve->decDuration = (uint32_t)(pScurve->freqDelta * 5.0f / speedRamp->speedDeceleration);
            pScurve->timeCnt = 0U;
        }
        else
        {
            pScurve->motionType = SCURVE_MOTION_UNIF;
            pScurve->timeCnt = 0U;
            pScurve->freqStart = pScurve->target;
            pScurve->freqEnd = pScurve->target;
            #if (defined SENSORLESS)
            if (pScurve->freqStart < startCurve->curveOut->frqRefValue)
            {
                pScurve->freqStart = startCurve->curveOut->frqRefValue;
            }
            #endif
            if (pScurve->freqEnd > speedRamp->speedMax)
            {
                pScurve->freqEnd = speedRamp->speedMax;
            }
        }
    }
    else
    {
        if (pScurve->curveEndFlag == 1U)
        {
            pScurve->motionType = SCURVE_MOTION_UNIF;
            pScurve->timeCnt = 0U;
            pScurve->freqStart = pScurve->target;
            pScurve->freqEnd = pScurve->target;
            #if (defined SENSORLESS)
            if (pScurve->freqStart < startCurve->curveOut->frqRefValue)
            {
                pScurve->freqStart = startCurve->curveOut->frqRefValue;
            }
            #endif
            if (pScurve->freqEnd > speedRamp->speedMax)
            {
                pScurve->freqEnd = speedRamp->speedMax;
            }
        }
        else
        {
            if (pScurve->curveCalcFlag != 1U)
            {
                pScurve->motionType = SCURVE_MOTION_UNIF;
                pScurve->timeCnt = 0;
            }
            else
            {
            }
            #if (defined SENSORLESS)
            if (pScurve->freqStart < startCurve->curveOut->frqRefValue)
            {
                pScurve->freqStart = startCurve->curveOut->frqRefValue;
            }
            #endif
            if (pScurve->freqEnd > speedRamp->speedMax)
            {
                pScurve->freqEnd = speedRamp->speedMax;
            }
        }
    }

    pScurve->targetBuf = pScurve->target;
}

/*!
* @brief Motor motion mode judgment in S-curve ramp of Motor 2.
*
* @param[in] pScurve: pointer to SCURVE_DATA structure
* @param[in] speedRamp: pointer to SPEED_RAMP_HANDLE structure
* @param[in] startCurve: pointer to STARTUP_CURVE structure
* @return none
*/
void SCurve_MotionTypeJudgeProcess_M2(SCURVE_DATA *pScurve, SPEED_RAMP_HANDLE *speedRamp, STARTUP_CURVE *startCurve)
{
    #if (defined SENSORLESS_M2)
    pScurve->target = MathFloat_Sat(MathFloat_Abs(speedRamp->speedTarget), speedRamp->speedMax, speedRamp->speedMin);
    #else
    pScurve->target = speedRamp->speedTarget;
    #endif
    pScurve->timeCnt++;
    if (pScurve->target != pScurve->targetBuf)
    {
        pScurve->curveEndFlag = 0U;

        if (pScurve->target > pScurve->freqOutput)
        {
            #if (defined SENSORLESS_M2)
            if ((pScurve->freqOutput == 0.0f) && (pScurve->target < startCurve->curveOut->frqRefValue))
            {
                pScurve->motionType = SCURVE_MOTION_DEC;
                pScurve->freqStart = startCurve->curveOut->frqRefValue;
                pScurve->freqEnd = pScurve->target;
                pScurve->freqDelta = pScurve->freqStart - pScurve->freqEnd;
                pScurve->decDuration = (uint32_t)(pScurve->freqDelta * 5.0f / speedRamp->speedDeceleration);  /* 5: S curve calculated in 2ms */
                pScurve->timeCnt = 0U;
            }
            else if ((pScurve->freqOutput == 0.0f) && (pScurve->target > startCurve->curveOut->frqRefValue))
            {
                pScurve->motionType = SCURVE_MOTION_ACC;
                pScurve->freqStart = startCurve->curveOut->frqRefValue;
                pScurve->freqEnd = pScurve->target;
                pScurve->freqDelta = pScurve->freqEnd - pScurve->freqStart;
                pScurve->accDuration = (uint32_t)(pScurve->freqDelta * 5.0f / speedRamp->speedAcceleration);
                pScurve->timeCnt = 0U;
            }
            else if ((pScurve->freqOutput == 0.0f) && (pScurve->target == startCurve->curveOut->frqRefValue))
            {
                pScurve->motionType = SCURVE_MOTION_UNIF;
                pScurve->timeCnt = 0U;
                pScurve->freqStart = pScurve->target;
                pScurve->freqEnd = pScurve->target;
            }
            else if (pScurve->freqOutput != 0.0f)
            {
                pScurve->motionType = SCURVE_MOTION_ACC;
                if (pScurve->curveCalcFlag == 1U)
                {
                    pScurve->freqStart = pScurve->freqOutput;
                    pScurve->freqEnd = pScurve->target;
                }
                else
                {
                    pScurve->freqStart = pScurve->targetBuf;
                    pScurve->freqEnd = pScurve->target;
                }
                pScurve->freqDelta = pScurve->freqEnd - pScurve->freqStart;
                pScurve->accDuration = (uint32_t)(pScurve->freqDelta * 5.0f / speedRamp->speedAcceleration);
                pScurve->timeCnt = 0U;
            }
            else
            {
            }
            #else
            pScurve->motionType = SCURVE_MOTION_ACC;
            if (pScurve->curveCalcFlag == 1U)
            {
                pScurve->freqStart = pScurve->freqOutput;
                pScurve->freqEnd = pScurve->target;
            }
            else
            {
                pScurve->freqStart = pScurve->targetBuf;
                pScurve->freqEnd = pScurve->target;
            }
            if (pScurve->freqEnd > speedRamp->speedMax)
            {
                pScurve->freqEnd = speedRamp->speedMax;
            }
            pScurve->freqDelta = pScurve->freqEnd - pScurve->freqStart;

            pScurve->accDuration = (uint32_t)(pScurve->freqDelta * 5.0f / speedRamp->speedAcceleration);
            pScurve->timeCnt = 0U;
            #endif
        }
        else if (pScurve->target < pScurve->freqOutput)
        {
            pScurve->motionType = SCURVE_MOTION_DEC;
            if (pScurve->curveCalcFlag == 1U)
            {
                pScurve->freqStart = pScurve->freqOutput;
                pScurve->freqEnd = pScurve->target;
            }
            else
            {
                pScurve->freqStart = pScurve->targetBuf;
                pScurve->freqEnd = pScurve->target;
            }
            #if (defined SENSORLESS_M2)
            if (pScurve->freqStart < speedRamp->speedMin)
            {
                pScurve->freqStart = speedRamp->speedMin;
            }
            #endif
            if (pScurve->freqEnd > speedRamp->speedMax)
            {
                pScurve->freqEnd = speedRamp->speedMax;
            }
            pScurve->freqDelta = pScurve->freqStart - pScurve->freqEnd;
            pScurve->decDuration = (uint32_t)(pScurve->freqDelta * 5.0f / speedRamp->speedDeceleration);
            pScurve->timeCnt = 0U;
        }
        else
        {
            pScurve->motionType = SCURVE_MOTION_UNIF;
            pScurve->timeCnt = 0U;
            pScurve->freqStart = pScurve->target;
            pScurve->freqEnd = pScurve->target;
            #if (defined SENSORLESS_M2)
            if (pScurve->freqStart < startCurve->curveOut->frqRefValue)
            {
                pScurve->freqStart = startCurve->curveOut->frqRefValue;
            }
            #endif
            if (pScurve->freqEnd > speedRamp->speedMax)
            {
                pScurve->freqEnd = speedRamp->speedMax;
            }
        }
    }
    else
    {
        if (pScurve->curveEndFlag == 1U)
        {
            pScurve->motionType = SCURVE_MOTION_UNIF;
            pScurve->timeCnt = 0U;
            pScurve->freqStart = pScurve->target;
            pScurve->freqEnd = pScurve->target;
            #if (defined SENSORLESS_M2)
            if (pScurve->freqStart < startCurve->curveOut->frqRefValue)
            {
                pScurve->freqStart = startCurve->curveOut->frqRefValue;
            }
            #endif
            if (pScurve->freqEnd > speedRamp->speedMax)
            {
                pScurve->freqEnd = speedRamp->speedMax;
            }
        }
        else
        {
            if (pScurve->curveCalcFlag != 1U)
            {
                pScurve->motionType = SCURVE_MOTION_UNIF;
                pScurve->timeCnt = 0;
            }
            else
            {
            }
            #if (defined SENSORLESS_M2)
            if (pScurve->freqStart < startCurve->curveOut->frqRefValue)
            {
                pScurve->freqStart = startCurve->curveOut->frqRefValue;
            }
            #endif
            if (pScurve->freqEnd > speedRamp->speedMax)
            {
                pScurve->freqEnd = speedRamp->speedMax;
            }
        }
    }

    pScurve->targetBuf = pScurve->target;
}

/*!
* @brief S curve type judgment.
*
* @param[in] pScurve: pointer to SCURVE_DATA structure
* @return none
*/
void SCurve_CurveTypeJudgeProcess(SCURVE_DATA *pScurve)
{
    if ((pScurve->accStartRatio != 0.0f) && (pScurve->accEndRatio != 0.0f))
    {
        pScurve->accCurveType = SCURVE_TYPE_1;
    }
    else if ((pScurve->accStartRatio != 0.0f) && (pScurve->accEndRatio == 0.0f))
    {
        pScurve->accCurveType = SCURVE_TYPE_2;
    }
    else if ((pScurve->accStartRatio == 0.0f) && (pScurve->accEndRatio != 0.0f))
    {
        pScurve->accCurveType = SCURVE_TYPE_3;
    }
    else
    {
        pScurve->accCurveType = SCURVE_TYPE_4;
    }

    if ((pScurve->decStartRatio != 0.0f) && (pScurve->decEndRatio != 0.0f))
    {
        pScurve->decCurveType = SCURVE_TYPE_1;
    }
    else if ((pScurve->decStartRatio != 0.0f) && (pScurve->decEndRatio == 0.0f))
    {
        pScurve->decCurveType = SCURVE_TYPE_2;
    }
    else if ((pScurve->decStartRatio == 0.0f) && (pScurve->decEndRatio != 0.0f))
    {
        pScurve->decCurveType = SCURVE_TYPE_3;
    }
    else
    {
        pScurve->decCurveType = SCURVE_TYPE_4;
    }
}

/*!
* @brief S curve acc mode type 1 parameters calculation.
*
* @param[in] pScurve: pointer to SCURVE_DATA structure
* @return none
*/
void SCurve_AccCurveType1ParamCalcProcess(SCURVE_DATA *pScurve)
{
    float temp1;
    uint32_t temp2;

    temp1 = (2.0f - pScurve->accStartRatio - pScurve->accEndRatio) * pScurve->accStartRatio;
    temp2 = pScurve->accDuration;
    pScurve->accStartJerk = 2.0f * pScurve->freqDelta / (temp1 * temp2 * temp2);
    pScurve->accEndJerk = -pScurve->accStartJerk * pScurve->accStartRatio / pScurve->accEndRatio;

    pScurve->accA1 = 0.5f * pScurve->accStartJerk;
    pScurve->accB1 = 0.0f;

    pScurve->accA2 = pScurve->accStartJerk * pScurve->accStartRatio * temp2;

    pScurve->accA3 = 0.5f * pScurve->accEndJerk;
    pScurve->accB3 = -pScurve->accEndJerk * temp2;

    pScurve->accTimePoint1 = (uint32_t)(pScurve->accDuration * pScurve->accStartRatio);
    pScurve->accTimePoint2 = (uint32_t)(pScurve->accDuration * (1.0f - pScurve->accEndRatio));
    pScurve->accTimePoint3 = pScurve->accDuration;
}

/*!
* @brief S curve dec mode type 1 parameters calculation.
*
* @param[in] pScurve: pointer to SCURVE_DATA structure
* @return none
*/
void SCurve_DecCurveType1ParamCalcProcess(SCURVE_DATA *pScurve)
{
    float temp1;
    uint32_t temp2;

    temp1 = (2.0f - pScurve->decStartRatio - pScurve->decEndRatio) * pScurve->decStartRatio;
    temp2 = pScurve->decDuration;
    pScurve->decStartJerk = -2.0f * pScurve->freqDelta / (temp1 * temp2 * temp2);
    pScurve->decEndJerk = -pScurve->decStartJerk * pScurve->decStartRatio / pScurve->decEndRatio;

    pScurve->decA1 = 0.5f * pScurve->decStartJerk;
    pScurve->decB1 = 0.0f;

    pScurve->decA2 = pScurve->decStartJerk * pScurve->decStartRatio * temp2;

    pScurve->decA3 = 0.5f * pScurve->decEndJerk;
    pScurve->decB3 = -pScurve->decEndJerk * temp2;

    pScurve->decTimePoint1 = (uint32_t)(pScurve->decDuration * pScurve->decStartRatio);
    pScurve->decTimePoint2 = (uint32_t)(pScurve->decDuration * (1.0f - pScurve->decEndRatio));
    pScurve->decTimePoint3 = pScurve->decDuration;
}

/*!
* @brief S curve acc mode type 2 parameters calculation.
*
* @param[in] pScurve: pointer to SCURVE_DATA structure
* @return none
*/
void SCurve_AccCurveType2ParamCalcProcess(SCURVE_DATA *pScurve)
{
    float temp1;
    uint32_t temp2;

    temp1 = (2.0f - pScurve->accStartRatio) * pScurve->accStartRatio;
    temp2 = pScurve->accDuration;
    pScurve->accStartJerk = 2.0f * pScurve->freqDelta / (temp1 * temp2 * temp2);
    pScurve->accEndJerk = 0.0f;

    pScurve->accA1 = 0.5f * pScurve->accStartJerk;
    pScurve->accB1 = 0.0f;

    pScurve->accA2 = pScurve->accStartJerk * pScurve->accStartRatio * temp2;

    pScurve->accTimePoint1 = (uint32_t)(pScurve->accDuration * pScurve->accStartRatio);
    pScurve->accTimePoint2 = pScurve->accDuration;
}

/*!
* @brief S curve dec mode type 2 parameters calculation.
*
* @param[in] pScurve: pointer to SCURVE_DATA structure
* @return none
*/
void SCurve_DecCurveType2ParamCalcProcess(SCURVE_DATA *pScurve)
{
    float temp1;
    uint32_t temp2;

    temp1 = (2.0f - pScurve->decStartRatio) * pScurve->decStartRatio / 32768U;
    temp2 = pScurve->decDuration;
    pScurve->decStartJerk = -2.0f * pScurve->freqDelta / (temp1 * temp2 * temp2);
    pScurve->decEndJerk = 0.0f;

    pScurve->decA1 = 0.5f * pScurve->decStartJerk;
    pScurve->decB1 = 0.0f;

    pScurve->decA2 = pScurve->decStartJerk * pScurve->decStartRatio * temp2;

    pScurve->decTimePoint1 = (uint32_t)(pScurve->decDuration * pScurve->decStartRatio);
    pScurve->decTimePoint2 = pScurve->decDuration;
}

/*!
* @brief S curve acc mode type 3 parameters calculation.
*
* @param[in] pScurve: pointer to SCURVE_DATA structure
* @return none
*/
void SCurve_AccCurveType3ParamCalcProcess(SCURVE_DATA *pScurve)
{
    float temp1;
    uint32_t temp2;

    temp1 = (2.0f - pScurve->accEndRatio) * pScurve->accEndRatio;
    temp2 = pScurve->accDuration;
    pScurve->accStartJerk = 0.0f;
    pScurve->accEndJerk = -2.0f * pScurve->freqDelta / (temp1 * temp2 * temp2);

    pScurve->accA2 = -pScurve->accEndJerk * pScurve->accEndRatio * temp2;

    pScurve->accA3 = 0.5f * pScurve->accEndJerk;
    pScurve->accB3 = -pScurve->accEndJerk * temp2;

    pScurve->accTimePoint1 = (uint32_t)(pScurve->accDuration * (1.0f - pScurve->accEndRatio));
    pScurve->accTimePoint2 = pScurve->accDuration;
}

/*!
* @brief S curve dec mode type 3 parameters calculation.
*
* @param[in] pScurve: pointer to SCURVE_DATA structure
* @return none
*/
void SCurve_DecCurveType3ParamCalcProcess(SCURVE_DATA *pScurve)
{
    float temp1;
    uint32_t temp2;

    temp1 = (2.0f - pScurve->decEndRatio) * pScurve->decEndRatio;
    temp2 = pScurve->decDuration;
    pScurve->decStartJerk = 0.0f;
    pScurve->decEndJerk = 2.0f * pScurve->freqDelta / (temp1 * temp2 * temp2);

    pScurve->decA2 = -pScurve->decEndJerk * pScurve->decEndRatio * temp2;

    pScurve->decA3 = 0.5f * pScurve->decEndJerk;
    pScurve->decB3 = -pScurve->decEndJerk * temp2;

    pScurve->decTimePoint1 = (uint32_t)(pScurve->decDuration * (1.0f - pScurve->decEndRatio));
    pScurve->decTimePoint2 = pScurve->decDuration;
}

/*!
* @brief S curve acc mode type 4 parameters calculation.
*
* @param[in] pScurve: pointer to SCURVE_DATA structure
* @return none
*/
void SCurve_AccCurveType4ParamCalcProcess(SCURVE_DATA *pScurve)
{
    pScurve->accStartJerk = 0.0f;
    pScurve->accEndJerk = 0.0f;

    pScurve->accA2 = pScurve->freqDelta / pScurve->accDuration;
    pScurve->accTimePoint1 = pScurve->accDuration;
}

/*!
* @brief S curve dec mode type 4 parameters calculation.
*
* @param[in] pScurve: pointer to SCURVE_DATA structure
* @return none
*/
void SCurve_DecCurveType4ParamCalcProcess(SCURVE_DATA *pScurve)
{
    pScurve->decStartJerk = 0.0f;
    pScurve->decEndJerk = 0.0f;

    pScurve->decA2 = (0.0f - pScurve->freqDelta) / pScurve->decDuration;
    pScurve->decTimePoint1 = pScurve->decDuration;
}

/*!
* @brief S curve parameters calculation.
*
* @param[in] pScurve: pointer to SCURVE_DATA structure
* @return none
*/
void SCurve_CurveParamCalcProcess(SCURVE_DATA *pScurve)
{
    if (pScurve->motionType == SCURVE_MOTION_ACC)
    {
        switch (pScurve->accCurveType)
        {
        case SCURVE_TYPE_1:
            SCurve_AccCurveType1ParamCalcProcess(pScurve);
        break;

        case SCURVE_TYPE_2:
            SCurve_AccCurveType2ParamCalcProcess(pScurve);
        break;

        case SCURVE_TYPE_3:
            SCurve_AccCurveType3ParamCalcProcess(pScurve);
        break;

        case SCURVE_TYPE_4:
            SCurve_AccCurveType4ParamCalcProcess(pScurve);
        break;

        default:
            break;
        }
    }
    else if (pScurve->motionType == SCURVE_MOTION_DEC)
    {
        switch (pScurve->decCurveType)
        {
        case SCURVE_TYPE_1:
            SCurve_DecCurveType1ParamCalcProcess(pScurve);
            break;

        case SCURVE_TYPE_2:
            SCurve_DecCurveType2ParamCalcProcess(pScurve);
            break;

        case SCURVE_TYPE_3:
            SCurve_DecCurveType3ParamCalcProcess(pScurve);
            break;

        case SCURVE_TYPE_4:
            SCurve_DecCurveType4ParamCalcProcess(pScurve);
            break;

        default:
            break;
        }
    }
    else
    {
    }
}

/*!
* @brief S curve acc mode type 1 calculation.
*
* @param[in] pScurve: pointer to SCURVE_DATA structure
* @return none
*/
void SCurve_AccCurveType1CalcProcess(SCURVE_DATA *pScurve)
{
    if (pScurve->timeCnt < pScurve->accTimePoint1)
    {
        if (pScurve->timeCnt < 1U)
        {
            pScurve->freqCalc = pScurve->freqStart;
        }
        else
        {
            pScurve->delta = pScurve->accA1 * (pScurve->timeCnt * 2U - 1U) + pScurve->accB1;
            pScurve->freqCalc += pScurve->delta;
        }
        pScurve->curveCalcFlag = 1U;
    }
    else if (pScurve->timeCnt < pScurve->accTimePoint2)
    {
        if (pScurve->timeCnt == pScurve->accTimePoint1)
        {
            pScurve->freqCalc += pScurve->delta;
        }
        else
        {
            pScurve->delta = pScurve->accA2;
            pScurve->freqCalc += pScurve->delta;
        }
        pScurve->curveCalcFlag = 1U;
    }
    else if (pScurve->timeCnt < pScurve->accTimePoint3)
    {
        if (pScurve->timeCnt == pScurve->accTimePoint2)
        {
            pScurve->freqCalc += pScurve->delta;
        }
        else
        {
            pScurve->delta = pScurve->accA3 * (pScurve->timeCnt * 2U - 1U) + pScurve->accB3;
            pScurve->freqCalc += pScurve->delta;
        }
        pScurve->curveCalcFlag = 1U;
    }
    else
    {
        pScurve->freqCalc = pScurve->freqEnd;
        pScurve->timeCnt = pScurve->accTimePoint3;
        pScurve->curveEndFlag = 1U;
        pScurve->curveCalcFlag = 0U;
    }
}

/*!
* @brief S curve acc mode type 2 calculation.
*
* @param[in] pScurve: pointer to SCURVE_DATA structure
* @return none
*/
void SCurve_AccCurveType2CalcProcess(SCURVE_DATA *pScurve)
{
    if (pScurve->timeCnt < pScurve->accTimePoint1)
    {
        if (pScurve->timeCnt < 1U)
        {
            pScurve->freqCalc = pScurve->freqStart;
        }
        else
        {
            pScurve->delta = pScurve->accA1 * (pScurve->timeCnt * 2U - 1U) + pScurve->accB1;
            pScurve->freqCalc += pScurve->delta;
        }
        pScurve->curveCalcFlag = 1U;
    }
    else if (pScurve->timeCnt < pScurve->accTimePoint2)
    {
        if (pScurve->timeCnt == pScurve->accTimePoint1)
        {
            pScurve->freqCalc += pScurve->delta;
        }
        else
        {
            pScurve->delta = pScurve->accA2;
            pScurve->freqCalc += pScurve->delta;
        }
        pScurve->curveCalcFlag = 1U;
    }
    else
    {
        pScurve->freqCalc = pScurve->freqEnd;
        pScurve->timeCnt = pScurve->accTimePoint2;
        pScurve->curveEndFlag = 1U;
        pScurve->curveCalcFlag = 0U;
    }
}

/*!
* @brief S curve acc mode type 3 calculation.
*
* @param[in] pScurve: pointer to SCURVE_DATA structure
* @return none
*/
void SCurve_AccCurveType3CalcProcess(SCURVE_DATA *pScurve)
{
    if (pScurve->timeCnt < pScurve->accTimePoint1)
    {
        if (pScurve->timeCnt < 1U)
        {
            pScurve->freqCalc = pScurve->freqStart;
        }
        else
        {
            pScurve->delta = pScurve->accA2;
            pScurve->freqCalc += pScurve->delta;
        }
        pScurve->curveCalcFlag = 1U;
    }
    else if (pScurve->timeCnt < pScurve->accTimePoint2)
    {
        if (pScurve->timeCnt == pScurve->accTimePoint1)
        {
            pScurve->freqCalc += pScurve->delta;
        }
        else
        {
            pScurve->delta = pScurve->accA3 * (pScurve->timeCnt * 2U - 1U) + pScurve->accB3;
            pScurve->freqCalc += pScurve->delta;
        }
        pScurve->curveCalcFlag = 1U;
    }
    else
    {
        pScurve->freqCalc = pScurve->freqEnd;
        pScurve->timeCnt = pScurve->accTimePoint2;
        pScurve->curveEndFlag = 1U;
        pScurve->curveCalcFlag = 0U;
    }
}

/*!
* @brief S curve acc mode type 4 calculation.
*
* @param[in] pScurve: pointer to SCURVE_DATA structure
* @return none
*/
void SCurve_AccCurveType4CalcProcess(SCURVE_DATA *pScurve)
{
    if (pScurve->timeCnt < pScurve->accTimePoint1)
    {
        if (pScurve->timeCnt < 1U)
        {
            pScurve->freqCalc = pScurve->freqStart;
        }
        else
        {
            pScurve->delta = pScurve->accA2;
            pScurve->freqCalc += pScurve->delta;
        }
        pScurve->curveCalcFlag = 1U;
    }
    else
    {
        pScurve->freqCalc = pScurve->freqEnd;
        pScurve->timeCnt = pScurve->accTimePoint2;
        pScurve->curveEndFlag = 1U;
        pScurve->curveCalcFlag = 0U;
    }
}

/*!
* @brief S curve dec mode type 1 calculation.
*
* @param[in] pScurve: pointer to SCURVE_DATA structure
* @return none
*/
void SCurve_DecCurveType1CalcProcess(SCURVE_DATA *pScurve)
{
    if (pScurve->timeCnt < pScurve->decTimePoint1)
    {
        if (pScurve->timeCnt < 1U)
        {
            pScurve->freqCalc = pScurve->freqStart;
        }
        else
        {
            pScurve->delta = pScurve->decA1 * (pScurve->timeCnt * 2U - 1U) + pScurve->decB1;
            pScurve->freqCalc += pScurve->delta;
        }
        pScurve->curveCalcFlag = 1U;
    }
    else if (pScurve->timeCnt < pScurve->decTimePoint2)
    {
        if (pScurve->timeCnt == pScurve->decTimePoint1)
        {
            pScurve->freqCalc += pScurve->delta;
        }
        else
        {
            pScurve->delta = pScurve->decA2;
            pScurve->freqCalc += pScurve->delta;
        }
        pScurve->curveCalcFlag = 1U;
    }
    else if (pScurve->timeCnt < pScurve->decTimePoint3)
    {
        if (pScurve->timeCnt == pScurve->decTimePoint2)
        {
            pScurve->freqCalc += pScurve->delta;
        }
        else
        {
            pScurve->delta = pScurve->decA3 * (pScurve->timeCnt * 2U - 1U) + pScurve->decB3;
            pScurve->freqCalc += pScurve->delta;
        }
        pScurve->curveCalcFlag = 1U;
    }
    else
    {
        pScurve->freqCalc = pScurve->freqEnd;
        pScurve->timeCnt = pScurve->decTimePoint3;
        pScurve->curveEndFlag = 1U;
        pScurve->curveCalcFlag = 0U;
    }
}

/*!
* @brief S curve dec mode type 2 calculation.
*
* @param[in] pScurve: pointer to SCURVE_DATA structure
* @return none
*/
void SCurve_DecCurveType2CalcProcess(SCURVE_DATA *pScurve)
{
    if (pScurve->timeCnt < pScurve->decTimePoint1)
    {
        if (pScurve->timeCnt < 1U)
        {
            pScurve->freqCalc = pScurve->freqStart;
        }
        else
        {
            pScurve->delta = pScurve->decA1 * (pScurve->timeCnt * 2U - 1U) + pScurve->decB1;
            pScurve->freqCalc += pScurve->delta;
        }
        pScurve->curveCalcFlag = 1U;
    }
    else if (pScurve->timeCnt < pScurve->decTimePoint2)
    {
        if (pScurve->timeCnt == pScurve->decTimePoint1)
        {
            pScurve->freqCalc += pScurve->delta;
        }
        else
        {
            pScurve->delta = pScurve->decA2;
            pScurve->freqCalc += pScurve->delta;
        }
        pScurve->curveCalcFlag = 1U;
    }
    else
    {
        pScurve->freqCalc = pScurve->freqEnd;
        pScurve->timeCnt = pScurve->decTimePoint2;
        pScurve->curveEndFlag = 1U;
        pScurve->curveCalcFlag = 0U;
    }
}

/*!
* @brief S curve dec mode type 3 calculation.
*
* @param[in] pScurve: pointer to SCURVE_DATA structure
* @return none
*/
void SCurve_DecCurveType3CalcProcess(SCURVE_DATA *pScurve)
{
    if (pScurve->timeCnt < pScurve->decTimePoint1)
    {
        if (pScurve->timeCnt < 1U)
        {
            pScurve->freqCalc = pScurve->freqStart;
        }
        else
        {
            pScurve->delta = pScurve->decA2;
            pScurve->freqCalc += pScurve->delta;
        }
        pScurve->curveCalcFlag = 1U;
    }
    else if (pScurve->timeCnt < pScurve->decTimePoint2)
    {
        if (pScurve->timeCnt == pScurve->decTimePoint1)
        {
            pScurve->freqCalc += pScurve->delta;
        }
        else
        {
            pScurve->delta = pScurve->decA3 * (pScurve->timeCnt * 2U - 1U) + pScurve->decB3;
            pScurve->freqCalc += pScurve->delta;
        }
        pScurve->curveCalcFlag = 1U;
    }
    else
    {
        pScurve->freqCalc = pScurve->freqEnd;
        pScurve->timeCnt = pScurve->decTimePoint2;
        pScurve->curveEndFlag = 1U;
        pScurve->curveCalcFlag = 0U;
    }
}

/*!
* @brief S curve dec mode type 4 calculation.
*
* @param[in] pScurve: pointer to SCURVE_DATA structure
* @return none
*/
void SCurve_DecCurveType4CalcProcess(SCURVE_DATA *pScurve)
{
    if (pScurve->timeCnt < pScurve->decTimePoint1)
    {
        if (pScurve->timeCnt < 1U)
        {
            pScurve->freqCalc = pScurve->freqStart;
        }
        else
        {
            pScurve->delta = pScurve->decA2;
            pScurve->freqCalc += pScurve->delta;
        }
        pScurve->curveCalcFlag = 1U;
    }
    else
    {
        pScurve->freqCalc = pScurve->freqEnd;
        pScurve->timeCnt = pScurve->decTimePoint2;
        pScurve->curveEndFlag = 1U;
        pScurve->curveCalcFlag = 0U;
    }
}

/*!
* @brief S curve output calculation.
*
* @param[in] pScurve: pointer to SCURVE_DATA structure
* @return none
*/
void SCurve_CurveCalcProcess(SCURVE_DATA *pScurve)
{
    if (pScurve->motionType == SCURVE_MOTION_ACC)
    {
        switch (pScurve->accCurveType)
        {
        case SCURVE_TYPE_1:
            SCurve_AccCurveType1CalcProcess(pScurve);
            break;

        case SCURVE_TYPE_2:
            SCurve_AccCurveType2CalcProcess(pScurve);
            break;

        case SCURVE_TYPE_3:
            SCurve_AccCurveType3CalcProcess(pScurve);
            break;

        case SCURVE_TYPE_4:
            SCurve_AccCurveType4CalcProcess(pScurve);
            break;

        default:
            break;
        }
    }
    else if (pScurve->motionType == SCURVE_MOTION_DEC)
    {
        switch (pScurve->decCurveType)
        {
        case SCURVE_TYPE_1:
            SCurve_DecCurveType1CalcProcess(pScurve);
            break;

        case SCURVE_TYPE_2:
            SCurve_DecCurveType2CalcProcess(pScurve);
            break;

        case SCURVE_TYPE_3:
            SCurve_DecCurveType3CalcProcess(pScurve);
            break;

        case SCURVE_TYPE_4:
            SCurve_DecCurveType4CalcProcess(pScurve);
            break;

        default:
            break;
        }
    }
    else if (pScurve->motionType == SCURVE_MOTION_UNIF)
    {
        pScurve->freqCalc = pScurve->target;
    }
    else
    {
    }
    pScurve->freqOutput = pScurve->freqCalc;
}

/*!
* @brief S curve calculation main process.
*
* @param[in] dir: motor operation direction
* @param[in] pScurve: pointer to SCURVE_DATA structure
* @param[in] speedRamp: pointer to SPEED_RAMP_HANDLE structure
* @param[in] startCurve: pointer to STARTUP_CURVE structure
* @return none
*/
void SCurve_MainProcess(int16_t dir, SCURVE_DATA *pScurve, SPEED_RAMP_HANDLE *speedRamp, STARTUP_CURVE *startCurve)
{
    ASR_SpeedTargetLimit(dir, speedRamp);
    SCurve_MotionTypeJudgeProcess(pScurve, speedRamp, startCurve);

    if (pScurve->curveEndFlag != 1U)
    {
        if (pScurve->timeCnt < 1U)
        {
            SCurve_CurveParamCalcProcess(pScurve);
        }
    }

    SCurve_CurveCalcProcess(pScurve);

#if (defined SENSORLESS)
    speedRamp->speedTargetRamp = pScurve->freqOutput * dir;
#else
    speedRamp->speedTargetRamp = pScurve->freqOutput;
#endif
}

/*!
* @brief S curve calculation main process of Motor 2.
*
* @param[in] dir: motor operation direction
* @param[in] pScurve: pointer to SCURVE_DATA structure
* @param[in] speedRamp: pointer to SPEED_RAMP_HANDLE structure
* @param[in] startCurve: pointer to STARTUP_CURVE structure
* @return none
*/
void SCurve_MainProcess_M2(int16_t dir, SCURVE_DATA *pScurve, SPEED_RAMP_HANDLE *speedRamp, STARTUP_CURVE *startCurve)
{
    ASR_SpeedTargetLimit(dir, speedRamp);
    SCurve_MotionTypeJudgeProcess_M2(pScurve, speedRamp, startCurve);

    if (pScurve->curveEndFlag != 1U)
    {
        if (pScurve->timeCnt < 1U)
        {
            SCurve_CurveParamCalcProcess(pScurve);
        }
    }

    SCurve_CurveCalcProcess(pScurve);

#if (defined SENSORLESS_M2)
    speedRamp->speedTargetRamp = pScurve->freqOutput * dir;
#else
    speedRamp->speedTargetRamp = pScurve->freqOutput;
#endif
}

/*!
* @brief Rotor speed and synchronous angle calculation with encoder of induction motor.
*
* @param[in] acimFoc: pointer to ACIM_FOC_CTRL structure
* @param[in] STC: pointer to STC_PARAM structure
* @param[in] encoderSensor: pointer to ENCODER_TYPE structure
* @param[in] speedCalc: pointer to ESTIMATED_SPEED structure
* @return none
*/
//void Acim_SensorSpeedCalc(ACIM_FOC_CTRL *acimFoc, STC_PARAM *STC, ENCODER_TYPE *encoderSensor, ESTIMATED_SPEED *speedCalc)
//{
//    uint16_t tempRotorTheta;
//    int16_t tempRotorSpeed;

//    tempRotorTheta = Encoder_ElecAngleCalc(encoderSensor, STC->pMotor);
//    Asr_EstSpeedCalc(tempRotorTheta, speedCalc);
//    tempRotorSpeed = speedCalc->estSpeed;
//    acimFoc->speed->rotorSpeed = (tempRotorSpeed + (int32_t)acimFoc->speed->rotorSpeed * 15) >> 4;
//    Acim_SensorSlipFreqCalc(acimFoc, STC);
//    acimFoc->speed->syncSpeed = acimFoc->speed->rotorSpeed + acimFoc->speed->slipSpeed;
//    acimFoc->flux->rotorFluxAngle += (Math_Mpy(acimFoc->speed->syncSpeed, acimFoc->mot->timeCoef));
//}

/* =============================================  EOF  ============================================== */
